@@ -4,16 +4,20 @@
 #ifndef MANDELBROTWIDGET_H
 #define MANDELBROTWIDGET_H
 
-#include <QGestureEvent>
-#include <QPixmap>
-#include <QWidget>
 #include "renderthread.h"
 
+#include <QCoreApplication>
+#include <QPixmap>
+#include <QWidget>
+
+QT_BEGIN_NAMESPACE
+class QGestureEvent;
+QT_END_NAMESPACE
 
 //! [0]
 class MandelbrotWidget : public QWidget
 {
-    Q_OBJECT
+    Q_DECLARE_TR_FUNCTIONS(MandelbrotWidget)
 
 public:
     MandelbrotWidget(QWidget *parent = nullptr);
@@ -33,11 +37,9 @@ protected:
     bool event(QEvent *event) override;
 #endif
 
-private slots:
+private:
     void updatePixmap(const QImage &image, double scaleFactor);
     void zoom(double zoomFactor);
-
-private:
     void scroll(int deltaX, int deltaY);
 #ifndef QT_NO_GESTURES
     bool gestureEvent(QGestureEvent *event);
