@@ -2,9 +2,9 @@
 
 #define JPEG_LIB_VERSION 80
 
-#define LIBJPEG_TURBO_VERSION 2.1.4
+#define LIBJPEG_TURBO_VERSION 3.1.3
 
-#define LIBJPEG_TURBO_VERSION_NUMBER 2001004
+#define LIBJPEG_TURBO_VERSION_NUMBER 3001003
 
 #define C_ARITH_CODING_SUPPORTED 1
 
@@ -12,6 +12,9 @@
 
 #define MEM_SRCDST_SUPPORTED 1
 
-#define BITS_IN_JSAMPLE 8
+#ifndef BITS_IN_JSAMPLE
+#define BITS_IN_JSAMPLE  8
+#endif
 
 #define NO_PUTENV
+
