@@ -9,16 +9,12 @@ import android.app.Activity;
 import android.content.pm.ActivityInfo;
 import android.content.pm.PackageManager;
 import android.content.res.Configuration;
-import android.content.res.TypedArray;
 import android.graphics.drawable.ColorDrawable;
 import android.graphics.drawable.Drawable;
 import android.graphics.Rect;
-import android.graphics.Color;
-import android.os.Build;
 import android.util.DisplayMetrics;
 import android.util.Log;
 import android.util.TypedValue;
-import android.view.Display;
 import android.view.ViewTreeObserver;
 import android.view.animation.AccelerateInterpolator;
 import android.view.animation.AlphaAnimation;
@@ -43,6 +39,7 @@ class QtActivityDelegate extends QtActivityDelegateBase
     private boolean m_splashScreenSticky = false;
     private boolean m_backendsRegistered = false;
 
+    private View m_dummyView = null;
     private final HashMap<Integer, View> m_nativeViews = new HashMap<>();
 
     QtActivityDelegate(Activity activity)
@@ -56,6 +53,8 @@ class QtActivityDelegate extends QtActivityDelegateBase
         super.initMembers();
         setActionBarVisibility(false);
         setActivityBackgroundDrawable();
+        if (QtNativeAccessibility.accessibilitySupported())
+            m_accessibilityDelegate.initLayoutAccessibility(m_layout);
     }
 
     void registerBackends()
@@ -95,7 +94,6 @@ class QtActivityDelegate extends QtActivityDelegateBase
         QtNative.runAction(() -> {
             if (m_layout != null) {
                 m_displayManager.setSystemUiVisibility(isFullScreen, expandedToCutout);
-                m_layout.requestLayout();
                 QtWindow.updateWindows();
             }
         });
@@ -143,14 +141,10 @@ class QtActivityDelegate extends QtActivityDelegateBase
         m_activity.setContentView(m_layout,
                                   new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT,
                                                              ViewGroup.LayoutParams.MATCH_PARENT));
-        QtDisplayManager.handleOrientationChanges(m_activity);
 
-        handleUiModeChange(m_activity.getResources().getConfiguration().uiMode & Configuration.UI_MODE_NIGHT_MASK);
+        handleUiModeChange();
 
-        Display display = (Build.VERSION.SDK_INT < Build.VERSION_CODES.R)
-                ? m_activity.getWindowManager().getDefaultDisplay()
-                : m_activity.getDisplay();
-        QtDisplayManager.handleRefreshRateChanged(QtDisplayManager.getRefreshRate(display));
+        m_displayManager.initDisplayProperties();
 
         m_layout.getViewTreeObserver().addOnPreDrawListener(() -> {
             if (!m_inputDelegate.isKeyboardVisible())
@@ -159,7 +153,7 @@ class QtActivityDelegate extends QtActivityDelegateBase
             Rect r = new Rect();
             m_activity.getWindow().getDecorView().getWindowVisibleDisplayFrame(r);
             DisplayMetrics metrics = new DisplayMetrics();
-            m_activity.getWindowManager().getDefaultDisplay().getMetrics(metrics);
+            QtDisplayManager.getDisplay(m_activity).getMetrics(metrics);
             final int kbHeight = metrics.heightPixels - r.bottom;
             if (kbHeight < 0) {
                 m_inputDelegate.setKeyboardVisibility(false, System.nanoTime());
@@ -205,19 +199,6 @@ class QtActivityDelegate extends QtActivityDelegateBase
                                                             ViewGroup.LayoutParams.MATCH_PARENT,
                                                             ViewGroup.LayoutParams.MATCH_PARENT));
                 m_layout.addView(m_splashScreen);
-
-                // Set DayNight theme as layout background so splash screen
-                // is not visible with opaque windows.
-                TypedArray typedArray = m_activity.getTheme().obtainStyledAttributes(
-                                        android.R.style.Theme_DeviceDefault_DayNight,
-                                        new int[]{ android.R.attr.colorBackground });
-                try {
-                    int backgroundColor = typedArray.getColor(0, Color.WHITE);
-                    Drawable background = new ColorDrawable(backgroundColor);
-                    m_layout.setBackground(background);
-                } finally {
-                    typedArray.recycle();
-                }
             }
         } catch (Exception e) {
             e.printStackTrace();
@@ -290,21 +271,17 @@ class QtActivityDelegate extends QtActivityDelegateBase
         m_accessibilityDelegate.notifyValueChanged(viewId, value);
     }
 
+    @Override public void notifyDescriptionOrNameChanged(int viewId, String value)
+    {
+        m_accessibilityDelegate.notifyDescriptionOrNameChanged(viewId, value);
+    }
+
     @Override
     public void notifyScrolledEvent(int viewId)
     {
         m_accessibilityDelegate.notifyScrolledEvent(viewId);
     }
 
-    @Override
-    public void initializeAccessibility()
-    {
-        QtNative.runAction(() -> {
-            // FIXME make QtAccessibilityDelegate window based
-            m_accessibilityDelegate.initLayoutAccessibility(m_layout);
-        });
-    }
-
     @Override
     public void notifyAnnouncementEvent(int viewId, String message)
     {
@@ -395,9 +372,15 @@ class QtActivityDelegate extends QtActivityDelegateBase
             if (m_layout == null)
                 return;
 
+            if (m_topLevelWindows.isEmpty()) {
+                if (m_dummyView != null) {
+                    m_layout.removeView(m_dummyView);
+                    m_dummyView = null;
+                }
+            }
+
             m_layout.addView(window, m_topLevelWindows.size());
             m_topLevelWindows.put(window.getId(), window);
-            window.setToDestroy(false);
             if (!m_splashScreenSticky)
                 hideSplashScreen();
         });
@@ -411,10 +394,10 @@ class QtActivityDelegate extends QtActivityDelegateBase
             if (m_topLevelWindows.containsKey(id)) {
                 QtWindow window = m_topLevelWindows.remove(id);
                 window.setOnApplyWindowInsetsListener(null); // Set in QtWindow for safe margins
-                if (window.isFrontmostVisibleWindow()) {
-                    window.setToDestroy(true);
-                    // Keep current shown window open during shutdown transition
-                    m_layout.postDelayed(() -> { window.destroySurface(); }, 500);
+                if (m_topLevelWindows.isEmpty()) {
+                   // Keep last frame in stack until it is replaced to get correct
+                   // shutdown transition
+                   m_dummyView = window;
                } else if (m_layout != null) {
                    m_layout.removeView(window);
                }
@@ -473,6 +456,11 @@ class QtActivityDelegate extends QtActivityDelegateBase
             return;
 
         QtNative.runAction(()-> {
+            if (m_dummyView != null) {
+                m_layout.removeView(m_dummyView);
+                m_dummyView = null;
+            }
+
             if (m_nativeViews.containsKey(id))
                 m_layout.removeView(m_nativeViews.remove(id));
 
