@@ -1,6 +1,7 @@
 // Copyright (C) 2016 BogDan Vatra <bogdan@kde.org>
 // Copyright (C) 2023 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:data-parser
 
 package org.qtproject.qt.android;
 
@@ -14,12 +15,14 @@ import android.net.Uri;
 import android.os.Handler;
 import android.os.IBinder;
 import android.os.Looper;
+import android.system.Os;
 import android.util.Log;
 import android.view.ContextMenu;
 import android.view.Menu;
 import android.view.View;
 
 import java.lang.ref.WeakReference;
+import java.lang.UnsatisfiedLinkError;
 import java.security.KeyStore;
 import java.security.cert.X509Certificate;
 import java.util.ArrayList;
@@ -41,9 +44,8 @@ public class QtNative
 
     static final String QtTAG = "Qt JAVA";
 
-    // a list containing all actions which could not be performed (e.g. the main activity is destroyed, etc.)
-    private static final ArrayList<Runnable> m_lostActions = new ArrayList<>();
-
+    // a list of all actions which could not be performed (e.g. the main activity is destroyed, etc.)
+    private static final BackgroundActionsTracker m_backgroundActionsTracker = new BackgroundActionsTracker();
     private static QtThread m_qtThread = null;
     private static final Object m_qtThreadLock = new Object();
     private static ClassLoader m_classLoader = null;
@@ -67,6 +69,14 @@ public class QtNative
     {
         synchronized (m_mainActivityMutex) {
             m_activity = new WeakReference<>(qtMainActivity);
+            try {
+                if (m_stateDetails.isStarted)
+                    updateNativeActivity();
+            } catch (UnsatisfiedLinkError ignored) {
+                // No-op - this happens in certain e.g. QtQuick for Android cases when we set the
+                // Activity for the first time, before Qt native libraries have been loaded. The
+                // C++ side will update its reference when the library is loaded.
+            }
         }
     }
 
@@ -241,17 +251,25 @@ public class QtNative
         m_stateDetails.nativePluginIntegrationReady = ready;
         notifyNativePluginIntegrationReadyChanged(ready);
         notifyAppStateDetailsChanged(m_stateDetails);
+
+        // Only set queue size when the plugin is fully loaded.
+        final String bufferSize = Os.getenv("QT_ANDROID_BACKGROUND_ACTIONS_QUEUE_SIZE");
+        if (bufferSize != null) {
+            try {
+                final int size = Integer.parseInt(bufferSize);
+                m_backgroundActionsTracker.setMaxAllowedActions(size);
+            } catch (NumberFormatException exception) {
+                Log.e(QtTAG, "Parsing failed, QT_ANDROID_BACKGROUND_ACTIONS_QUEUE_SIZE value is not an integer");
+            }
+        }
     }
 
     static void setApplicationState(int state)
     {
         synchronized (m_mainActivityMutex) {
             m_stateDetails.state = state;
-            if (state == ApplicationState.ApplicationActive) {
-                for (Runnable mLostAction : m_lostActions)
-                    runAction(mLostAction);
-                m_lostActions.clear();
-            }
+            if (state == ApplicationState.ApplicationActive)
+                m_backgroundActionsTracker.processActions();
         }
         updateApplicationState(state);
         notifyAppStateDetailsChanged(m_stateDetails);
@@ -303,7 +321,7 @@ public class QtNative
                         && (m_stateDetails.state != ApplicationState.ApplicationHidden);
                 final boolean active = (isActivityValid() && isStateVisible) || isServiceValid();
                 if (!active || !handler.post(action))
-                    m_lostActions.add(action);
+                    m_backgroundActionsTracker.enqueue(action);
             } else {
                 handler.post(action);
             }
@@ -370,13 +388,15 @@ public class QtNative
 
     static void quitQt()
     {
-        terminateQt();
-        m_stateDetails.isStarted = false;
-        notifyAppStateDetailsChanged(m_stateDetails);
-        getQtThread().exit();
-        synchronized (m_qtThreadLock) {
-            m_qtThread = null;
-        }
+        runAction(() -> {
+            terminateQt();
+            m_stateDetails.isStarted = false;
+            notifyAppStateDetailsChanged(m_stateDetails);
+            getQtThread().exit();
+            synchronized (m_qtThreadLock) {
+                m_qtThread = null;
+            }
+        });
     }
 
     @UsedFromNativeCode
@@ -426,7 +446,7 @@ public class QtNative
             if (list != null) {
                 for (String file : list) {
                     try {
-                        String[] isDir = asset.list(path.length() > 0 ? path + "/" + file : file);
+                        String[] isDir = asset.list(!path.isEmpty() ? path + "/" + file : file);
                         if (isDir != null && isDir.length > 0)
                             file += "/";
                         res.add(file);
@@ -451,10 +471,6 @@ public class QtNative
     static native boolean updateNativeActivity();
     // application methods
 
-    // window methods
-    static native void updateWindow();
-    // window methods
-
     // application methods
     static native void updateApplicationState(int state);
     static native void updateLocale();
@@ -476,7 +492,7 @@ public class QtNative
 
     static native void runPendingCppRunnables();
 
-    static native void sendRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults);
+    static native void sendRequestPermissionsResult(int requestCode, int[] grantResults);
     // activity methods
 
     // service methods
