@@ -22,23 +22,22 @@ class QtSurface extends SurfaceView implements SurfaceHolder.Callback
         setFocusableInTouchMode(false);
         setZOrderMediaOverlay(onTop);
         m_surfaceCallback = surfaceCallback;
-        getHolder().addCallback(this);
-        if (imageDepth == 16)
-            getHolder().setFormat(PixelFormat.RGB_565);
-        else
-            getHolder().setFormat(PixelFormat.RGBA_8888);
+        SurfaceHolder holder = getHolder();
+        holder.setFormat(imageDepth == 16 ? PixelFormat.RGB_565 : PixelFormat.RGBA_8888);
+        holder.addCallback(this);
     }
 
     @Override
     public void surfaceCreated(SurfaceHolder holder)
     {
+        if (m_surfaceCallback != null)
+            m_surfaceCallback.onSurfaceChanged(holder.getSurface());
     }
 
     @Override
     public void surfaceChanged(SurfaceHolder holder, int format, int width, int height)
     {
-        if (width < 1 || height < 1)
-            return;
+        // FIXME: this can sometimes cause flicker on orientation change
         if (m_surfaceCallback != null)
             m_surfaceCallback.onSurfaceChanged(holder.getSurface());
     }
