@@ -29,13 +29,25 @@
 #define QT_NO_EXCEPTIONS
 #endif
 
+#undef QT_DEBUG
+#undef QT_FORCE_ASSERTS
+#ifndef QT_NO_DEBUG
+#  define QT_NO_DEBUG
+#endif
+#define QT_NO_DEBUG_OUTPUT
+#define QT_NO_DEBUG_STREAM
+#define QT_NO_INFO_OUTPUT
+#define QT_NO_WARNING_OUTPUT
+
 #define QT_NO_USING_NAMESPACE
 #define QT_NO_DEPRECATED
 
 // Keep feature-test macros in alphabetic order by feature name:
 #define QT_FEATURE_cborstreamreader -1
 #define QT_FEATURE_cborstreamwriter 1
-#define QT_CRYPTOGRAPHICHASH_ONLY_SHA1
+#define QT_FEATURE_commandlineparser 1
+#define QT_NO_COMPRESS
+#define QT_FEATURE_copy_file_range -1
 #define QT_FEATURE_cxx17_filesystem -1
 #define QT_NO_DATASTREAM
 #define QT_FEATURE_datestring 1
@@ -43,22 +55,16 @@
 #define QT_FEATURE_dup3 -1
 #define QT_FEATURE_easingcurve -1
 #define QT_FEATURE_etw -1
-#if defined(__linux__) || defined(__GLIBC__)
-#define QT_FEATURE_getauxval (__has_include(<sys/auxv.h>) ? 1 : -1)
-#else
-#define QT_FEATURE_getauxval -1
-#endif
-#define QT_FEATURE_getentropy -1
-#define QT_NO_GEOM_VARIANT
-#define QT_FEATURE_hijricalendar -1
-#define QT_FEATURE_icu -1
-#define QT_FEATURE_islamiccivilcalendar -1
-#define QT_FEATURE_jalalicalendar -1
-#define QT_FEATURE_journald -1
 #define QT_FEATURE_futimens -1
 #undef QT_FEATURE_future
 #define QT_FEATURE_future -1
+#define QT_FEATURE_hijricalendar -1
+#define QT_FEATURE_icu -1
 #define QT_FEATURE_itemmodel -1
+#define QT_FEATURE_islamiccivilcalendar -1
+#define QT_FEATURE_jalalicalendar -1
+#define QT_FEATURE_jemalloc -1
+#define QT_FEATURE_journald -1
 #define QT_FEATURE_library -1
 #ifdef __linux__
 # define QT_FEATURE_linkat 1
@@ -69,6 +75,7 @@
 #define QT_FEATURE_memmem -1
 #define QT_FEATURE_memrchr -1
 #define QT_NO_QOBJECT
+#define QT_FEATURE_permissions -1
 #define QT_FEATURE_process -1
 #define QT_FEATURE_regularexpression 1
 #ifdef __GLIBC_PREREQ
@@ -89,18 +96,6 @@
 #define QT_NO_TRANSLATION
 #define QT_FEATURE_translation -1
 #define QT_NO_VARIANT -1
-
-#define QT_NO_COMPRESS
-
-// rcc.pro will DEFINES+= this
-#ifndef QT_FEATURE_zstd
-#define QT_FEATURE_zstd -1
-#endif
-
-#define QT_FEATURE_commandlineparser 1
-#define QT_FEATURE_settings -1
-#define QT_FEATURE_permissions -1
-
-#define QT_NO_TEMPORARYFILE
+#define QT_FEATURE_winsdkicu -1
 
 #endif // QT_BOOTSTRAPPED
