@@ -102,7 +102,7 @@ Q_LOGGING_CATEGORY(lcPermissions, "qt.permissions", QtWarningMsg);
     application, please \l{Information Property List Files}
     {point the build system to your custom \c Info.plist}.
 
-    \sa {Information Property List Files}.
+    \sa {Information Property List Files}
 
     \section3 Android
     \target android-uses-permission
@@ -118,11 +118,16 @@ Q_LOGGING_CATEGORY(lcPermissions, "qt.permissions", QtWarningMsg);
 
     To ensure the relevant permission backend is included with your
     application, please \l {QT_ANDROID_PACKAGE_SOURCE_DIR}
-    {point the build system to your custom \c AndroidManifest.xml}.
+    {point the build system to your custom \c AndroidManifest.xml}
+    or use \l {qt_add_android_permission()}.
 
     The relevant permission names are described in the documentation
     for each permission type.
 
+    \note When using this API, the \c{<!-- %%INSERT_PERMISSIONS -->} tag must be present in
+    the AndroidManifest.xml. For further information on the use of this tag,
+    see \l {Qt Permissions and Features}
+
     \sa {Qt Creator: Editing Manifest Files}.
 
     \section1 Available Permissions
@@ -131,6 +136,10 @@ Q_LOGGING_CATEGORY(lcPermissions, "qt.permissions", QtWarningMsg);
 
     \annotatedlist permissions
 
+    \note The available permission types cover core functionality of Qt modules
+    like Qt Multimedia and Qt Positioning, but do not encompass all platform-specific
+    permissions. Custom permission types are not currently supported.
+
     \section1 Best Practices
 
     To ensure the best possible user experience for the end user we recommend
@@ -242,8 +251,8 @@ Q_LOGGING_CATEGORY(lcPermissions, "qt.permissions", QtWarningMsg);
     You do not need to construct this type explicitly, as the type is automatically
     used when checking or requesting permissions.
 
-    This constructor participates in overload resolution only if \c T is one of
-    the \l{typed permission} classes:
+    \constraints
+    \c T is one of the \l{typed permission} classes:
 
     \annotatedlist permissions
 */
@@ -256,8 +265,8 @@ Q_LOGGING_CATEGORY(lcPermissions, "qt.permissions", QtWarningMsg);
 
     Use type() for dynamically choosing which typed permission to request.
 
-    This function participates in overload resolution only if \c T is one of
-    the \l{typed permission} classes:
+    \constraints
+    \c T is one of the \l{typed permission} classes:
 
     \annotatedlist permissions
 */
