@@ -18,16 +18,10 @@
 
 #include "../../3rdparty/sha1/sha1.cpp"
 
-#if defined(QT_BOOTSTRAPPED) && !defined(QT_CRYPTOGRAPHICHASH_ONLY_SHA1)
-#  error "Are you sure you need the other hashing algorithms besides SHA-1?"
-#endif
-
 // Header from rfc6234
 #include "../../3rdparty/rfc6234/sha.h"
 
-#ifndef QT_CRYPTOGRAPHICHASH_ONLY_SHA1
 #if !QT_CONFIG(openssl_hash)
-// qdoc and qmake only need SHA-1
 #include "../../3rdparty/md5/md5.h"
 #include "../../3rdparty/md5/md5.cpp"
 #include "../../3rdparty/md4/md4.h"
@@ -120,7 +114,6 @@ QT_WARNING_DISABLE_MSVC(4505)
 #include "../../3rdparty/blake2/src/blake2s-ref.c"
 QT_WARNING_POP
 #endif
-#endif // QT_CRYPTOGRAPHICHASH_ONLY_SHA1
 
 #if !defined(QT_BOOTSTRAPPED) && QT_CONFIG(openssl_hash)
 #define USING_OPENSSL30
@@ -138,7 +131,6 @@ static constexpr int hashLengthInternal(QCryptographicHash::Algorithm method) no
         return Size \
     /*end*/
     CASE(Sha1, 20);
-#ifndef QT_CRYPTOGRAPHICHASH_ONLY_SHA1
     CASE(Md4, 16);
     CASE(Md5, 16);
     CASE(Sha224, SHA224HashSize);
@@ -166,7 +158,6 @@ static constexpr int hashLengthInternal(QCryptographicHash::Algorithm method) no
     case QCryptographicHash::Keccak_512:
     case QCryptographicHash::Blake2b_512:
         return 512 / 8;
-#endif
 #undef CASE
     case QCryptographicHash::NumAlgorithms: ;
         // fall through
@@ -244,6 +235,8 @@ public:
     // when not called from the static hash() function, this function needs to be
     // called with finalizeMutex held (finalize() will do that):
     void finalizeUnchecked() noexcept;
+    QSpan<uchar> finalizeUnchecked(QSpan<uchar> buffer) noexcept;
+
     // END functions that need to be called with finalizeMutex held
     QByteArrayView resultView() const noexcept { return result.toByteArrayView(); }
     static bool supportsAlgorithm(QCryptographicHash::Algorithm method);
@@ -277,7 +270,7 @@ public:
 
         explicit EVP(QCryptographicHash::Algorithm method);
         void reset() noexcept;
-        void finalizeUnchecked(HashResult &result) noexcept;
+        void finalizeUnchecked(QSpan<uchar> buffer) noexcept;
     };
 #endif
 
@@ -290,10 +283,9 @@ public:
 
         void reset(QCryptographicHash::Algorithm method) noexcept;
         void addData(QCryptographicHash::Algorithm method, QByteArrayView data) noexcept;
-        void finalizeUnchecked(QCryptographicHash::Algorithm method, HashResult &result) noexcept;
+        void finalizeUnchecked(QCryptographicHash::Algorithm method, QSpan<uchar> buffer) noexcept;
 
         Sha1State sha1Context;
-#ifndef QT_CRYPTOGRAPHICHASH_ONLY_SHA1
 #ifdef USING_OPENSSL30
         EVP evp;
 #else
@@ -307,10 +299,9 @@ public:
         SHA3Context sha3Context;
 
         enum class Sha3Variant { Sha3, Keccak };
-        static void sha3Finish(SHA3Context &ctx, HashResult &result, Sha3Variant sha3Variant);
+        static void sha3Finish(SHA3Context &ctx, QSpan<uchar> result, Sha3Variant sha3Variant);
         blake2b_state blake2bContext;
         blake2s_state blake2sContext;
-#endif
     } state;
     // protects result in finalize()
     QBasicMutex finalizeMutex;
@@ -319,8 +310,7 @@ public:
     const QCryptographicHash::Algorithm method;
 };
 
-#ifndef QT_CRYPTOGRAPHICHASH_ONLY_SHA1
-void QCryptographicHashPrivate::State::sha3Finish(SHA3Context &ctx, HashResult &result,
+void QCryptographicHashPrivate::State::sha3Finish(SHA3Context &ctx, QSpan<uchar> result,
                                                   Sha3Variant sha3Variant)
 {
     /*
@@ -355,7 +345,6 @@ void QCryptographicHashPrivate::State::sha3Finish(SHA3Context &ctx, HashResult &
 
     sha3Final(&ctx, result.data());
 }
-#endif
 
 /*!
   \class QCryptographicHash
@@ -600,12 +589,6 @@ QCryptographicHashPrivate::State::State(QCryptographicHash::Algorithm method)
     case QCryptographicHash::Sha1:
         new (&sha1Context) Sha1State;
         break;
-#ifdef QT_CRYPTOGRAPHICHASH_ONLY_SHA1
-    default:
-        Q_ASSERT_X(false, "QCryptographicHash", "Method not compiled in");
-        Q_UNREACHABLE();
-        break;
-#else
     case QCryptographicHash::Md4:
         new (&md4Context) md4_context;
         break;
@@ -646,7 +629,6 @@ QCryptographicHashPrivate::State::State(QCryptographicHash::Algorithm method)
     case QCryptographicHash::Blake2s_256:
         new (&blake2sContext) blake2s_state;
         break;
-#endif
     case QCryptographicHash::NumAlgorithms:
         Q_UNREACHABLE();
     }
@@ -726,12 +708,6 @@ void QCryptographicHashPrivate::State::reset(QCryptographicHash::Algorithm metho
     case QCryptographicHash::Sha1:
         sha1InitState(&sha1Context);
         break;
-#ifdef QT_CRYPTOGRAPHICHASH_ONLY_SHA1
-    default:
-        Q_ASSERT_X(false, "QCryptographicHash", "Method not compiled in");
-        Q_UNREACHABLE();
-        break;
-#else
     case QCryptographicHash::Md4:
         md4_init(&md4Context);
         break;
@@ -772,7 +748,6 @@ void QCryptographicHashPrivate::State::reset(QCryptographicHash::Algorithm metho
     case QCryptographicHash::Blake2s_256:
         blake2s_init(&blake2sContext, hashLengthInternal(method));
         break;
-#endif
     case QCryptographicHash::NumAlgorithms:
         Q_UNREACHABLE();
     }
@@ -879,12 +854,6 @@ void QCryptographicHashPrivate::State::addData(QCryptographicHash::Algorithm met
         case QCryptographicHash::Sha1:
             sha1Update(&sha1Context, (const unsigned char *)data, length);
             break;
-#ifdef QT_CRYPTOGRAPHICHASH_ONLY_SHA1
-        default:
-            Q_ASSERT_X(false, "QCryptographicHash", "Method not compiled in");
-            Q_UNREACHABLE();
-            break;
-#else
         case QCryptographicHash::Md4:
             md4_update(&md4Context, (const unsigned char *)data, length);
             break;
@@ -925,7 +894,6 @@ void QCryptographicHashPrivate::State::addData(QCryptographicHash::Algorithm met
         case QCryptographicHash::Blake2s_256:
             blake2s_update(&blake2sContext, reinterpret_cast<const uint8_t *>(data), length);
             break;
-#endif
         case QCryptographicHash::NumAlgorithms:
             Q_UNREACHABLE();
         }
@@ -1015,9 +983,23 @@ void QCryptographicHashPrivate::finalizeUnchecked() noexcept
     state.finalizeUnchecked(method, result);
 }
 
+/*!
+    \internal
+
+    Must be called with finalizeMutex held, except when called from the static
+    hash() function, where no sharing can take place.
+*/
+QSpan<uchar> QCryptographicHashPrivate::finalizeUnchecked(QSpan<uchar> buffer) noexcept
+{
+    buffer = buffer.first(hashLengthInternal(method));
+    state.finalizeUnchecked(method, buffer);
+    Q_ASSERT(result.size() == 0); // internal buffer wasn't used
+    return buffer;
+}
+
 #ifdef USING_OPENSSL30
 void QCryptographicHashPrivate::State::finalizeUnchecked(QCryptographicHash::Algorithm method,
-                                                         HashResult &result) noexcept
+                                                         QSpan<uchar> result) noexcept
 {
     switch (method) {
     case QCryptographicHash::Keccak_224:
@@ -1064,7 +1046,7 @@ void QCryptographicHashPrivate::State::finalizeUnchecked(QCryptographicHash::Alg
     }
 }
 
-void QCryptographicHashPrivate::EVP::finalizeUnchecked(HashResult &result) noexcept
+void QCryptographicHashPrivate::EVP::finalizeUnchecked(QSpan<uchar> result) noexcept
 {
     if (!initializationFailed) {
         EVP_MD_CTX_ptr copy = EVP_MD_CTX_ptr(EVP_MD_CTX_new());
@@ -1077,7 +1059,7 @@ void QCryptographicHashPrivate::EVP::finalizeUnchecked(HashResult &result) noexc
 #else // USING_OPENSSL30
 
 void QCryptographicHashPrivate::State::finalizeUnchecked(QCryptographicHash::Algorithm method,
-                                                         HashResult &result) noexcept
+                                                         QSpan<uchar> result) noexcept
 {
     switch (method) {
     case QCryptographicHash::Sha1: {
@@ -1086,12 +1068,6 @@ void QCryptographicHashPrivate::State::finalizeUnchecked(QCryptographicHash::Alg
         sha1ToHash(&copy, result.data());
         break;
     }
-#ifdef QT_CRYPTOGRAPHICHASH_ONLY_SHA1
-    default:
-        Q_ASSERT_X(false, "QCryptographicHash", "Method not compiled in");
-        Q_UNREACHABLE();
-        break;
-#else
     case QCryptographicHash::Md4: {
         md4_context copy = md4Context;
         md4_final(&copy, result.data());
@@ -1156,7 +1132,6 @@ void QCryptographicHashPrivate::State::finalizeUnchecked(QCryptographicHash::Alg
         blake2s_final(&copy, result.data(), length);
         break;
     }
-#endif
     case QCryptographicHash::NumAlgorithms:
         Q_UNREACHABLE();
     }
@@ -1207,12 +1182,8 @@ QByteArrayView QCryptographicHash::hashInto(QSpan<std::byte> buffer,
     QCryptographicHashPrivate hash(method);
     for (QByteArrayView part : data)
         hash.addData(part);
-    hash.finalizeUnchecked(); // no mutex needed: no-one but us has access to 'hash'
-    auto result = hash.resultView();
-    Q_ASSERT(buffer.size() >= result.size());
-    // ### optimize: have the method directly write into `buffer`
-    memcpy(buffer.data(), result.data(), result.size());
-    return buffer.first(result.size());
+    auto span = QSpan{reinterpret_cast<uchar *>(buffer.data()), buffer.size()};
+    return hash.finalizeUnchecked(span); // no mutex needed: no-one but us has access to 'hash'
 }
 
 /*!
@@ -1294,7 +1265,6 @@ bool QCryptographicHashPrivate::supportsAlgorithm(QCryptographicHash::Algorithm
 {
     switch (method) {
     case QCryptographicHash::Sha1:
-#ifndef QT_CRYPTOGRAPHICHASH_ONLY_SHA1
     case QCryptographicHash::Md4:
     case QCryptographicHash::Md5:
     case QCryptographicHash::Sha224:
@@ -1317,7 +1287,6 @@ bool QCryptographicHashPrivate::supportsAlgorithm(QCryptographicHash::Algorithm
     case QCryptographicHash::Blake2s_160:
     case QCryptographicHash::Blake2s_224:
     case QCryptographicHash::Blake2s_256:
-#endif
         return true;
     case QCryptographicHash::NumAlgorithms: ;
     };
@@ -1330,7 +1299,6 @@ static constexpr int qt_hash_block_size(QCryptographicHash::Algorithm method)
     switch (method) {
     case QCryptographicHash::Sha1:
         return SHA1_Message_Block_Size;
-#ifndef QT_CRYPTOGRAPHICHASH_ONLY_SHA1
     case QCryptographicHash::Md4:
         return 64;
     case QCryptographicHash::Md5:
@@ -1365,7 +1333,6 @@ static constexpr int qt_hash_block_size(QCryptographicHash::Algorithm method)
     case QCryptographicHash::Blake2s_224:
     case QCryptographicHash::Blake2s_256:
         return BLAKE2S_BLOCKBYTES;
-#endif // QT_CRYPTOGRAPHICHASH_ONLY_SHA1
     case QCryptographicHash::NumAlgorithms:
 #if !defined(Q_CC_GNU_ONLY) || Q_CC_GNU >= 900
         // GCC 8 has trouble with Q_UNREACHABLE() in constexpr functions
