@@ -5,10 +5,13 @@
 
 #include <private/qproperty_p.h>
 #include <private/qobject_p.h>
+#include <QtCore/qbasictimer.h>
 #include <QtCore/qcoreevent.h>
 #include <QtCore/qmath.h>
 #include <QtCore/qelapsedtimer.h>
 
+using namespace std::chrono_literals;
+
 QT_BEGIN_NAMESPACE
 
 class QTimeLinePrivate : public QObjectPrivate
@@ -16,6 +19,7 @@ class QTimeLinePrivate : public QObjectPrivate
     Q_DECLARE_PUBLIC(QTimeLine)
 public:
     QElapsedTimer timer;
+    QBasicTimer basicTimer;
     Q_OBJECT_BINDABLE_PROPERTY_WITH_ARGS(QTimeLinePrivate, QEasingCurve, easingCurve,
                                          QEasingCurve::InOutSine)
 
@@ -32,7 +36,6 @@ public:
     void setCurrentTimeForwardToQ(int time) { q_func()->setCurrentTime(time); }
     Q_OBJECT_COMPAT_PROPERTY_WITH_ARGS(QTimeLinePrivate, int, currentTime,
                                        &QTimeLinePrivate::setCurrentTimeForwardToQ, 0)
-    int timerId = 0;
 
     void setDirection(QTimeLine::Direction direction) { q_func()->setDirection(direction); }
     Q_OBJECT_COMPAT_PROPERTY_WITH_ARGS(QTimeLinePrivate, QTimeLine::Direction, direction,
@@ -55,16 +58,16 @@ void QTimeLinePrivate::setCurrentTime(int msecs)
 {
     Q_Q(QTimeLine);
     currentTime.removeBindingUnlessInWrapper();
-    auto previousCurrentTime = currentTime.value();
+    const auto previousCurrentTime = currentTime.valueBypassingBindings();
 
-    qreal lastValue = q->currentValue();
-    int lastFrame = q->currentFrame();
+    const qreal lastValue = q->valueForTime(previousCurrentTime);
+    const int lastFrame = q->frameForTime(previousCurrentTime);
 
     // Determine if we are looping.
-    int elapsed = (direction == QTimeLine::Backward) ? (-msecs + duration) : msecs;
-    int loopCountNow = elapsed / duration;
+    const int elapsed = (direction == QTimeLine::Backward) ? (-msecs + duration) : msecs;
+    const int loopCountNow = elapsed / duration;
 
-    bool looping = (loopCountNow != currentLoopCount);
+    const bool looping = (loopCountNow != currentLoopCount);
 #ifdef QTIMELINE_DEBUG
     qDebug() << "QTimeLinePrivate::setCurrentTime:" << msecs << duration << "with loopCountNow"
              << loopCountNow << "currentLoopCount" << currentLoopCount << "looping" << looping;
@@ -75,7 +78,7 @@ void QTimeLinePrivate::setCurrentTime(int msecs)
     // Normalize msecs to be between 0 and duration, inclusive.
     currentTime.setValueBypassingBindings(elapsed % duration);
     if (direction.value() == QTimeLine::Backward)
-        currentTime.setValueBypassingBindings(duration - currentTime);
+        currentTime.setValueBypassingBindings(duration - currentTime.valueBypassingBindings());
 
     // Check if we have reached the end of loopcount.
     bool finished = false;
@@ -85,12 +88,14 @@ void QTimeLinePrivate::setCurrentTime(int msecs)
         currentLoopCount = loopCount - 1;
     }
 
-    int currentFrame = q->frameForTime(currentTime);
+    const int currentFrame = q->frameForTime(currentTime.valueBypassingBindings());
 #ifdef QTIMELINE_DEBUG
-    qDebug() << "QTimeLinePrivate::setCurrentTime: frameForTime" << currentTime << currentFrame;
+    qDebug() << "QTimeLinePrivate::setCurrentTime: frameForTime"
+             << currentTime.valueBypassingBindings() << currentFrame;
 #endif
-    if (!qFuzzyCompare(lastValue, q->currentValue()))
-        emit q->valueChanged(q->currentValue(), QTimeLine::QPrivateSignal());
+    const qreal currentValue = q->valueForTime(currentTime.valueBypassingBindings());
+    if (!qFuzzyCompare(lastValue, currentValue))
+        emit q->valueChanged(currentValue, QTimeLine::QPrivateSignal());
     if (lastFrame != currentFrame) {
         const int transitionframe = (direction == QTimeLine::Forward ? endFrame : startFrame);
         if (looping && !finished && transitionframe != currentFrame) {
@@ -123,7 +128,7 @@ void QTimeLinePrivate::setCurrentTime(int msecs)
         q->stop();
         emit q->finished(QTimeLine::QPrivateSignal());
     }
-    if (currentTime.value() != previousCurrentTime)
+    if (currentTime.valueBypassingBindings() != previousCurrentTime)
         currentTime.notify();
 }
 QBindable<int> QTimeLine::bindableCurrentTime()
@@ -290,7 +295,7 @@ QTimeLine::State QTimeLine::state() const
     \property QTimeLine::loopCount
     \brief the number of times the timeline should loop before it's finished.
 
-    A loop count of of 0 means that the timeline will loop forever.
+    A loop count of 0 means that the timeline will loop forever.
 
     By default, this property contains a value of 1.
 */
@@ -334,11 +339,12 @@ QTimeLine::Direction QTimeLine::direction() const
 void QTimeLine::setDirection(Direction direction)
 {
     Q_D(QTimeLine);
-    auto previousDirection = d->direction.value();
-    d->direction.setValue(direction);
+    d->direction.removeBindingUnlessInWrapper();
+    const auto previousDirection = d->direction.valueBypassingBindings();
+    d->direction.setValueBypassingBindings(direction);
     d->startTime = d->currentTime;
     d->timer.start();
-    if (previousDirection != d->direction.value())
+    if (previousDirection != d->direction.valueBypassingBindings())
         d->direction.notify();
 }
 
@@ -372,12 +378,11 @@ void QTimeLine::setDuration(int duration)
         qWarning("QTimeLine::setDuration: cannot set duration <= 0");
         return;
     }
-    if (duration == d->duration) {
-        d->duration.removeBindingUnlessInWrapper();
-        return;
+    d->duration.removeBindingUnlessInWrapper();
+    if (duration != d->duration.valueBypassingBindings()) {
+        d->duration.setValueBypassingBindings(duration);
+        d->duration.notify();
     }
-    d->duration.setValue(duration);
-    d->duration.notify();
 }
 
 QBindable<int> QTimeLine::bindableDuration()
@@ -533,7 +538,7 @@ void QTimeLine::setCurrentTime(int msec)
     Q_D(QTimeLine);
     d->startTime = 0;
     d->currentLoopCount = 0;
-    d->timer.restart();
+    d->timer.start();
     d->setCurrentTime(msec);
 }
 
@@ -609,14 +614,14 @@ qreal QTimeLine::valueForTime(int msec) const
 void QTimeLine::start()
 {
     Q_D(QTimeLine);
-    if (d->timerId) {
+    if (d->basicTimer.isActive()) {
         qWarning("QTimeLine::start: already running");
         return;
     }
     int curTime = 0;
     if (d->direction == Backward)
         curTime = d->duration;
-    d->timerId = startTimer(d->updateInterval);
+    d->basicTimer.start(d->updateInterval * 1ms, this);
     d->startTime = curTime;
     d->currentLoopCount = 0;
     d->timer.start();
@@ -637,11 +642,11 @@ void QTimeLine::start()
 void QTimeLine::resume()
 {
     Q_D(QTimeLine);
-    if (d->timerId) {
+    if (d->basicTimer.isActive()) {
         qWarning("QTimeLine::resume: already running");
         return;
     }
-    d->timerId = startTimer(d->updateInterval);
+    d->basicTimer.start(d->updateInterval * 1ms, this);
     d->startTime = d->currentTime;
     d->timer.start();
     d->setState(Running);
@@ -655,10 +660,8 @@ void QTimeLine::resume()
 void QTimeLine::stop()
 {
     Q_D(QTimeLine);
-    if (d->timerId)
-        killTimer(d->timerId);
+    d->basicTimer.stop();
     d->setState(NotRunning);
-    d->timerId = 0;
 }
 
 /*!
@@ -678,12 +681,11 @@ void QTimeLine::setPaused(bool paused)
     }
     if (paused && d->state != Paused) {
         d->startTime = d->currentTime;
-        killTimer(d->timerId);
-        d->timerId = 0;
+        d->basicTimer.stop();
         d->setState(Paused);
     } else if (!paused && d->state == Paused) {
         // Same as resume()
-        d->timerId = startTimer(d->updateInterval);
+        d->basicTimer.start(d->updateInterval * 1ms, this);
         d->startTime = d->currentTime;
         d->timer.start();
         d->setState(Running);
@@ -710,7 +712,7 @@ void QTimeLine::toggleDirection()
 void QTimeLine::timerEvent(QTimerEvent *event)
 {
     Q_D(QTimeLine);
-    if (event->timerId() != d->timerId) {
+    if (event->id() != d->basicTimer.id()) {
         event->ignore();
         return;
     }
