@@ -1,41 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtGui module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:data-parser
 
 /*!
     \class QMovie
@@ -89,7 +54,7 @@
 
     Call supportedFormats() for a list of formats that QMovie supports.
 
-    \sa QLabel, QImageReader, {Movie Example}
+    \sa QLabel, QImageReader
 */
 
 /*! \enum QMovie::MovieState
@@ -139,7 +104,6 @@
 */
 
 /*! \fn void QMovie::frameChanged(int frameNumber)
-    \since 4.1
 
     This signal is emitted when the frame number has changed to
     \a frameNumber.  You can call currentImage() or currentPixmap() to get a
@@ -180,14 +144,17 @@
 #include "qrect.h"
 #include "qelapsedtimer.h"
 #include "qtimer.h"
-#include "qpair.h"
-#include "qmap.h"
 #include "qlist.h"
 #include "qbuffer.h"
 #include "qdir.h"
+#include "qloggingcategory.h"
+#include "private/qimage_p.h"
 #include "private/qobject_p.h"
 #include "private/qproperty_p.h"
 
+#include <map>
+#include <memory>
+
 #define QMOVIE_INVALID_DELAY -1
 
 QT_BEGIN_NAMESPACE
@@ -228,7 +195,10 @@ class QMoviePrivate : public QObjectPrivate
     Q_DECLARE_PUBLIC(QMovie)
 
 public:
-    QMoviePrivate(QMovie *qq);
+    QMoviePrivate();
+
+    void init(QMovie *qq, std::unique_ptr<QImageReader> r);
+
     bool isDone();
     bool next();
     int speedAdjustedDelay(int delay) const;
@@ -248,7 +218,7 @@ public:
     void _q_loadNextFrame();
     void _q_loadNextFrame(bool starting);
 
-    QImageReader *reader = nullptr;
+    std::unique_ptr<QImageReader> reader = nullptr;
 
     void setSpeed(int percentSpeed) { q_func()->setSpeed(percentSpeed); }
     Q_OBJECT_COMPAT_PROPERTY_WITH_ARGS(QMoviePrivate, int, speed, &QMoviePrivate::setSpeed, 100)
@@ -266,25 +236,33 @@ public:
                                          QMovie::CacheNone)
     bool haveReadAll = false;
     bool isFirstIteration = true;
-    QMap<int, QFrameInfo> frameMap;
+    std::map<int, QFrameInfo> frameMap;
     QString absoluteFilePath;
 
-    QTimer nextImageTimer;
+    QTimer *nextImageTimer = nullptr;
 };
 
 /*! \internal
  */
-QMoviePrivate::QMoviePrivate(QMovie *qq)
+QMoviePrivate::QMoviePrivate()
+{
+}
+
+void QMoviePrivate::init(QMovie *qq, std::unique_ptr<QImageReader> r)
 {
     q_ptr = qq;
-    nextImageTimer.setSingleShot(true);
+    reader = std::move(r);
+    nextImageTimer = new QTimer(qq);
+    nextImageTimer->setSingleShot(true);
+    QObject::connect(nextImageTimer, SIGNAL(timeout()),
+                     qq, SLOT(_q_loadNextFrame()));
 }
 
 /*! \internal
  */
 void QMoviePrivate::reset()
 {
-    nextImageTimer.stop();
+    nextImageTimer->stop();
     if (reader->device())
         initialDevicePos = reader->device()->pos();
     currentFrameNumber = -1;
@@ -342,6 +320,19 @@ QFrameInfo QMoviePrivate::infoForFrame(int frameNumber)
         return QFrameInfo(); // Invalid
     }
 
+    // For an animated image format, the tradition is that QMovie calls read()
+    // until canRead() == false, because the number of frames may not be known
+    // in advance; but if we're abusing a multi-frame format as an animation,
+    // canRead() may remain true, and we need to stop after reading the maximum
+    // number of frames that the image provides.
+    const bool supportsAnimation = reader->supportsOption(QImageIOHandler::Animation);
+    const int stopAtFrame = supportsAnimation ? -1 : frameCount();
+
+    // For an animated image format, QImageIOHandler::nextImageDelay() should
+    // provide the time to wait until showing the next frame; but multi-frame
+    // formats are not expected to provide this value, so use 1000 ms by default.
+    const auto nextFrameDelay = [&]() { return supportsAnimation ? reader->nextImageDelay() : 1000; };
+
     if (cacheMode == QMovie::CacheNone) {
         if (frameNumber != currentFrameNumber+1) {
             // Non-sequential frame access
@@ -356,11 +347,10 @@ QFrameInfo QMoviePrivate::infoForFrame(int frameNumber)
                     QIODevice *device = reader->device();
                     QColor bgColor = reader->backgroundColor();
                     QSize scaledSize = reader->scaledSize();
-                    delete reader;
                     if (fileName.isEmpty())
-                        reader = new QImageReader(device, format);
+                        reader = std::make_unique<QImageReader>(device, format);
                     else
-                        reader = new QImageReader(absoluteFilePath, format);
+                        reader = std::make_unique<QImageReader>(absoluteFilePath, format);
                     if (!reader->canRead()) // Provoke a device->open() call
                         emit q->error(reader->error());
                     reader->device()->seek(initialDevicePos);
@@ -371,8 +361,12 @@ QFrameInfo QMoviePrivate::infoForFrame(int frameNumber)
                 }
             }
         }
-        if (reader->canRead()) {
+        qCDebug(lcImageIo, "CacheNone: read frame %d of %d", frameNumber, stopAtFrame);
+        if (stopAtFrame > 0 ? (frameNumber < stopAtFrame) : reader->canRead()) {
             // reader says we can read. Attempt to actually read image
+            // But if it's a non-animated multi-frame format and we know the frame count, stop there.
+            if (stopAtFrame > 0)
+                reader->jumpToImage(frameNumber);
             QImage anImage = reader->read();
             if (anImage.isNull()) {
                 // Reading image failed.
@@ -380,7 +374,7 @@ QFrameInfo QMoviePrivate::infoForFrame(int frameNumber)
             }
             if (frameNumber > greatestFrameNumber)
                 greatestFrameNumber = frameNumber;
-            return QFrameInfo(QPixmap::fromImage(std::move(anImage)), reader->nextImageDelay());
+            return QFrameInfo(QPixmap::fromImage(std::move(anImage)), nextFrameDelay());
         } else if (frameNumber != 0) {
             // We've read all frames now. Return an end marker
             haveReadAll = true;
@@ -396,19 +390,23 @@ QFrameInfo QMoviePrivate::infoForFrame(int frameNumber)
     if (frameNumber > greatestFrameNumber) {
         // Frame hasn't been read from file yet. Try to do it
         for (int i = greatestFrameNumber + 1; i <= frameNumber; ++i) {
-            if (reader->canRead()) {
+            qCDebug(lcImageIo, "CacheAll: read frame %d of %d", frameNumber, stopAtFrame);
+            if (stopAtFrame > 0 ? (frameNumber < stopAtFrame) : reader->canRead()) {
                 // reader says we can read. Attempt to actually read image
+                // But if it's a non-animated multi-frame format and we know the frame count, stop there.
+                if (stopAtFrame > 0)
+                    reader->jumpToImage(frameNumber);
                 QImage anImage = reader->read();
                 if (anImage.isNull()) {
                     // Reading image failed.
                     return QFrameInfo(); // Invalid
                 }
                 greatestFrameNumber = i;
-                QFrameInfo info(QPixmap::fromImage(std::move(anImage)), reader->nextImageDelay());
+                QFrameInfo info(QPixmap::fromImage(std::move(anImage)), nextFrameDelay());
                 // Cache it!
-                frameMap.insert(i, info);
+                auto &e = frameMap[i] = std::move(info);
                 if (i == frameNumber) {
-                    return info;
+                    return e;
                 }
             } else {
                 // We've read all frames now. Return an end marker
@@ -418,7 +416,8 @@ QFrameInfo QMoviePrivate::infoForFrame(int frameNumber)
         }
     }
     // Return info for requested (cached) frame
-    return frameMap.value(frameNumber);
+    const auto it = frameMap.find(frameNumber);
+    return it == frameMap.cend() ? QFrameInfo() : it->second;
 }
 
 /*!
@@ -462,11 +461,7 @@ bool QMoviePrivate::next()
     }
     // Image and delay OK, update internal state
     currentFrameNumber = nextFrameNumber++;
-    QSize scaledSize = reader->scaledSize();
-    if (scaledSize.isValid() && (scaledSize != info.pixmap.size()))
-        currentPixmap = QPixmap::fromImage( info.pixmap.toImage().scaled(scaledSize) );
-    else
-        currentPixmap = info.pixmap;
+    currentPixmap = info.pixmap;
 
     if (!speed)
         return true;
@@ -506,7 +501,7 @@ void QMoviePrivate::_q_loadNextFrame(bool starting)
         emit q->frameChanged(currentFrameNumber);
 
         if (speed && movieState == QMovie::Running)
-            nextImageTimer.start(nextDelay);
+            nextImageTimer->start(nextDelay);
     } else {
         // Could not read another frame
         if (!isDone()) {
@@ -556,7 +551,7 @@ bool QMoviePrivate::jumpToFrame(int frameNumber)
         return true;
     nextFrameNumber = frameNumber;
     if (movieState == QMovie::Running)
-        nextImageTimer.stop();
+        nextImageTimer->stop();
     _q_loadNextFrame();
     return (nextFrameNumber == currentFrameNumber+1);
 }
@@ -589,11 +584,10 @@ bool QMoviePrivate::jumpToNextFrame()
     \sa setFileName(), setDevice(), setFormat()
  */
 QMovie::QMovie(QObject *parent)
-    : QObject(*new QMoviePrivate(this), parent)
+    : QObject(*new QMoviePrivate, parent)
 {
     Q_D(QMovie);
-    d->reader = new QImageReader;
-    connect(&d->nextImageTimer, SIGNAL(timeout()), this, SLOT(_q_loadNextFrame()));
+    d->init(this, std::make_unique<QImageReader>());
 }
 
 /*!
@@ -605,12 +599,11 @@ QMovie::QMovie(QObject *parent)
     The \a parent object is passed to QObject's constructor.
  */
 QMovie::QMovie(QIODevice *device, const QByteArray &format, QObject *parent)
-    : QObject(*new QMoviePrivate(this), parent)
+    : QObject(*new QMoviePrivate, parent)
 {
     Q_D(QMovie);
-    d->reader = new QImageReader(device, format);
+    d->init(this, std::make_unique<QImageReader>(device, format));
     d->initialDevicePos = device->pos();
-    connect(&d->nextImageTimer, SIGNAL(timeout()), this, SLOT(_q_loadNextFrame()));
 }
 
 /*!
@@ -622,14 +615,13 @@ QMovie::QMovie(QIODevice *device, const QByteArray &format, QObject *parent)
     The \a parent object is passed to QObject's constructor.
  */
 QMovie::QMovie(const QString &fileName, const QByteArray &format, QObject *parent)
-    : QObject(*new QMoviePrivate(this), parent)
+    : QObject(*new QMoviePrivate, parent)
 {
     Q_D(QMovie);
+    d->init(this, std::make_unique<QImageReader>(fileName, format));
     d->absoluteFilePath = QDir(fileName).absolutePath();
-    d->reader = new QImageReader(fileName, format);
     if (d->reader->device())
         d->initialDevicePos = d->reader->device()->pos();
-    connect(&d->nextImageTimer, SIGNAL(timeout()), this, SLOT(_q_loadNextFrame()));
 }
 
 /*!
@@ -638,7 +630,7 @@ QMovie::QMovie(const QString &fileName, const QByteArray &format, QObject *paren
 QMovie::~QMovie()
 {
     Q_D(QMovie);
-    delete d->reader;
+    d->reader.reset();
 }
 
 /*!
@@ -905,12 +897,12 @@ void QMovie::setPaused(bool paused)
         if (d->movieState == NotRunning)
             return;
         d->enterState(Paused);
-        d->nextImageTimer.stop();
+        d->nextImageTimer->stop();
     } else {
         if (d->movieState == Running)
             return;
         d->enterState(Running);
-        d->nextImageTimer.start(nextFrameDelay());
+        d->nextImageTimer->start(nextFrameDelay());
     }
 }
 
@@ -928,7 +920,7 @@ void QMovie::setSpeed(int percentSpeed)
 {
     Q_D(QMovie);
     if (!d->speed && d->movieState == Running)
-        d->nextImageTimer.start(nextFrameDelay());
+        d->nextImageTimer->start(nextFrameDelay());
     if (percentSpeed != d->speed) {
         d->speed = percentSpeed;
         d->speed.notify();
@@ -985,13 +977,11 @@ void QMovie::stop()
     if (d->movieState == NotRunning)
         return;
     d->enterState(NotRunning);
-    d->nextImageTimer.stop();
+    d->nextImageTimer->stop();
     d->nextFrameNumber = 0;
 }
 
 /*!
-    \since 4.1
-
     Returns the scaled size of frames.
 
     \sa QImageReader::scaledSize()
@@ -1003,8 +993,6 @@ QSize QMovie::scaledSize()
 }
 
 /*!
-    \since 4.1
-
     Sets the scaled frame size to \a size.
 
     \sa QImageReader::setScaledSize()
@@ -1016,8 +1004,6 @@ void QMovie::setScaledSize(const QSize &size)
 }
 
 /*!
-    \since 4.1
-
     Returns the list of image formats supported by QMovie.
 
     \sa QImageReader::supportedImageFormats()
