@@ -10,6 +10,14 @@
 #include <QtCore/qdebug.h>
 #include <QtCore/qcoreapplication.h>
 #include <QtCore/qoperatingsystemversion.h>
+#include <QtGui/qcolorspace.h>
+#include <QtGui/private/qicon_p.h>
+
+#if defined(Q_OS_MACOS)
+# include <AppKit/AppKit.h>
+#elif defined(QT_PLATFORM_UIKIT)
+# include <UIKit/UIKit.h>
+#endif
 
 QT_USE_NAMESPACE
 
@@ -54,17 +62,6 @@ CGImageRef qt_mac_toCGImage(const QImage &inImage)
     return inImage.convertToFormat(QImage::Format_ARGB32_Premultiplied).toCGImage();
 }
 
-CGImageRef qt_mac_toCGImageMask(const QImage &image)
-{
-    static const auto deleter = [](void *image, const void *, size_t) { delete static_cast<QImage *>(image); };
-    QCFType<CGDataProviderRef> dataProvider =
-            CGDataProviderCreateWithData(new QImage(image), image.bits(),
-                                                    image.sizeInBytes(), deleter);
-
-    return CGImageMaskCreate(image.width(), image.height(), 8, image.depth(),
-                              image.bytesPerLine(), dataProvider, NULL, false);
-}
-
 void qt_mac_drawCGImage(CGContextRef inContext, const CGRect *inBounds, CGImageRef inImage)
 {
     CGContextSaveGState( inContext );
@@ -85,9 +82,34 @@ QImage qt_mac_toQImage(CGImageRef image)
     CGRect rect = CGRectMake(0, 0, w, h);
     QMacCGContext ctx(&ret);
     qt_mac_drawCGImage(ctx, &rect, image);
+
+    CGColorSpaceRef colorSpace = CGImageGetColorSpace(image);
+    QCFType<CFDataRef> iccData = CGColorSpaceCopyICCData(colorSpace);
+    ret.setColorSpace(QColorSpace::fromIccProfile(QByteArray::fromRawCFData(iccData)));
+
     return ret;
 }
 
+QImage qt_mac_padToSquareImage(const QImage &image)
+{
+    if (image.width() == image.height())
+        return image;
+
+    const int size = std::max(image.width(), image.height());
+    QImage squareImage(size, size, image.format());
+    squareImage.setDevicePixelRatio(image.devicePixelRatio());
+    squareImage.fill(Qt::transparent);
+
+    QPoint pos((size - image.width()) / (2.0 * image.devicePixelRatio()),
+               (size - image.height()) / (2.0 * image.devicePixelRatio()));
+
+    QPainter painter(&squareImage);
+    painter.drawImage(pos, image);
+    painter.end();
+
+    return squareImage;
+}
+
 #ifdef Q_OS_MACOS
 
 QT_END_NAMESPACE
@@ -118,22 +140,29 @@ QT_END_NAMESPACE
 
 + (instancetype)imageFromQIcon:(const QIcon &)icon
 {
-    return [NSImage imageFromQIcon:icon withSize:0];
+    return [NSImage imageFromQIcon:icon withSize:QSize()];
 }
 
-+ (instancetype)imageFromQIcon:(const QIcon &)icon withSize:(int)size
++ (instancetype)imageFromQIcon:(const QIcon &)icon withSize:(const QSize &)size
+{
+    return [NSImage imageFromQIcon:icon withSize:size withMode:QIcon::Normal withState:QIcon::Off];
+}
+
++ (instancetype)imageFromQIcon:(const QIcon &)icon withSize:(const QSize &)size
+                    withMode:(QIcon::Mode)mode withState:(QIcon::State)state
+
 {
     if (icon.isNull())
         return nil;
 
     auto availableSizes = icon.availableSizes();
-    if (availableSizes.isEmpty() && size > 0)
-        availableSizes << QSize(size, size);
+    if (availableSizes.isEmpty() && !size.isNull())
+        availableSizes << size;
 
     auto nsImage = [[[NSImage alloc] initWithSize:NSZeroSize] autorelease];
 
     for (QSize size : std::as_const(availableSizes)) {
-        QImage image = icon.pixmap(size).toImage();
+        const QImage image = icon.pixmap(size, mode, state).toImage();
         if (image.isNull())
             continue;
 
@@ -144,6 +173,9 @@ QT_END_NAMESPACE
         auto *imageRep = [[NSBitmapImageRep alloc] initWithCGImage:cgImage];
         imageRep.size = image.deviceIndependentSize().toCGSize();
         [nsImage addRepresentation:[imageRep autorelease]];
+        // Match behavior of loading icns files, where the NSImage size
+        // reflects the largest representation.
+        nsImage.size = imageRep.size;
     }
 
     if (!nsImage.representations.count)
@@ -151,11 +183,26 @@ QT_END_NAMESPACE
 
     [nsImage setTemplate:icon.isMask()];
 
-    if (size)
-        nsImage.size = CGSizeMake(size, size);
+    if (!size.isNull()) {
+        auto imageSize = QSizeF::fromCGSize(nsImage.size);
+        nsImage.size = imageSize.scaled(size, Qt::KeepAspectRatio).toCGSize();
+    }
 
     return nsImage;
 }
+
++ (instancetype)internalImageFromQIcon:(const QT_PREPEND_NAMESPACE(QIcon) &)icon
+{
+    if (icon.isNull())
+        return nil;
+
+    // Check if the icon is backed by an NSImage. If so, we can use that directly.
+    auto *iconPrivate = QIconPrivate::get(&icon);
+    NSImage *iconImage = nullptr;
+    iconPrivate->engine->virtual_hook(QIconPrivate::PlatformIconHook, &iconImage);
+    return iconImage;
+}
+
 @end
 
 QT_BEGIN_NAMESPACE
