@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 
 /*!
@@ -110,7 +111,7 @@
 #endif
 
 #include <QtCore/qdir.h>
-#include <QtCore/qdiriterator.h>
+#include <QtCore/qdirlisting.h>
 #include <QtCore/qfile.h>
 
 QT_BEGIN_NAMESPACE
@@ -134,6 +135,8 @@ QSslCertificatePrivate::QSslCertificatePrivate()
 
 QSslCertificatePrivate::~QSslCertificatePrivate() = default;
 
+QT_DEFINE_QESDP_SPECIALIZATION_DTOR(QSslCertificatePrivate)
+
 /*!
     Constructs a QSslCertificate by reading \a format encoded data
     from \a device and using the first certificate found. You can
@@ -186,7 +189,7 @@ QSslCertificate::QSslCertificate(const QByteArray &data, QSsl::EncodingFormat fo
         return;
     }
 
-    QList<QSslCertificate> certs = X509Reader(data, 1);
+    const QList<QSslCertificate> certs = X509Reader(data, 1);
     if (!certs.isEmpty())
         d = certs.first().d;
 }
@@ -198,6 +201,18 @@ QSslCertificate::QSslCertificate(const QSslCertificate &other) : d(other.d)
 {
 }
 
+/*!
+    \fn QSslCertificate::QSslCertificate(QSslCertificate &&other)
+
+    \since 6.8
+
+    Move-constructs a new QSslCertificate from \a other.
+
+    \note The moved-from object \a other is placed in a partially-formed state,
+    in which the only valid operations are destructions and assignment of a new
+    value.
+*/
+
 /*!
     Destroys the QSslCertificate.
 */
@@ -218,9 +233,7 @@ QSslCertificate &QSslCertificate::operator=(const QSslCertificate &other)
 /*!
     \fn void QSslCertificate::swap(QSslCertificate &other)
     \since 5.0
-
-    Swaps this certificate instance with \a other. This function is
-    very fast and never fails.
+    \memberswap{certificate instance}
 */
 
 /*!
@@ -618,13 +631,19 @@ QList<QSslCertificate> QSslCertificate::fromPath(const QString &path,
                                                  QSsl::EncodingFormat format,
                                                  PatternSyntax syntax)
 {
+    if (path.isEmpty())
+        return {};
+
+    if (syntax == PatternSyntax::FixedString && QFileInfo(path).isFile())
+        return fromFile(path, format);
+
     // $, (,), *, +, ., ?, [, ,], ^, {, | and }.
 
     // make sure to use the same path separators on Windows and Unix like systems.
     QString sourcePath = QDir::fromNativeSeparators(path);
 
     // Find the path without the filename
-    QString pathPrefix = sourcePath.left(sourcePath.lastIndexOf(u'/'));
+    QStringView pathPrefix = QStringView(sourcePath).left(sourcePath.lastIndexOf(u'/'));
 
     // Check if the path contains any special chars
     int pos = -1;
@@ -635,7 +654,7 @@ QList<QSslCertificate> QSslCertificate::fromPath(const QString &path,
     else if (syntax == PatternSyntax::RegularExpression)
         pos = sourcePath.indexOf(QRegularExpression("[\\$\\(\\)\\*\\+\\.\\?\\[\\]\\^\\{\\}\\|]"_L1));
 #else
-    if (syntax == PatternSyntax::Wildcard || syntax == PatternSyntax::RegExp)
+    if (syntax == PatternSyntax::Wildcard || syntax == PatternSyntax::RegularExpression)
         qWarning("Regular expression support is disabled in this build. Only fixed string can be searched");
         return QList<QSslCertificate>();
 #endif
@@ -647,27 +666,22 @@ QList<QSslCertificate> QSslCertificate::fromPath(const QString &path,
         if (lastIndexOfSlash != -1)
             pathPrefix = pathPrefix.left(lastIndexOfSlash);
         else
-            pathPrefix.clear();
+            pathPrefix = {};
     } else {
         // Check if the path is a file.
-        if (QFileInfo(sourcePath).isFile()) {
-            QFile file(sourcePath);
-            QIODevice::OpenMode openMode = QIODevice::ReadOnly;
-            if (format == QSsl::Pem)
-                openMode |= QIODevice::Text;
-            if (file.open(openMode))
-                return QSslCertificate::fromData(file.readAll(), format);
-            return QList<QSslCertificate>();
-        }
+        if (QFileInfo(sourcePath).isFile())
+            return fromFile(sourcePath, format);
     }
 
     // Special case - if the prefix ends up being nothing, use "." instead.
     int startIndex = 0;
     if (pathPrefix.isEmpty()) {
-        pathPrefix = "."_L1;
+        pathPrefix = u".";
         startIndex = 2;
     }
 
+    const QString pathPrefixString = pathPrefix.toString();
+
     // The path can be a file or directory.
     QList<QSslCertificate> certs;
 
@@ -678,9 +692,12 @@ QList<QSslCertificate> QSslCertificate::fromPath(const QString &path,
     QRegularExpression pattern(QRegularExpression::anchoredPattern(sourcePath));
 #endif
 
-    QDirIterator it(pathPrefix, QDir::Files, QDirIterator::FollowSymlinks | QDirIterator::Subdirectories);
-    while (it.hasNext()) {
-        QString filePath = startIndex == 0 ? it.next() : it.next().mid(startIndex);
+    using F = QDirListing::IteratorFlag;
+    constexpr auto iterFlags = F::FollowDirSymlinks | F::Recursive | F::FilesOnly;
+    for (const auto &dirEntry : QDirListing(pathPrefixString, iterFlags)) {
+        QString filePath = dirEntry.filePath();
+        if (startIndex > 0)
+            filePath.remove(0, startIndex);
 
 #if QT_CONFIG(regularexpression)
         if (!pattern.match(filePath).hasMatch())
@@ -690,12 +707,7 @@ QList<QSslCertificate> QSslCertificate::fromPath(const QString &path,
             continue;
 #endif
 
-        QFile file(filePath);
-        QIODevice::OpenMode openMode = QIODevice::ReadOnly;
-        if (format == QSsl::Pem)
-            openMode |= QIODevice::Text;
-        if (file.open(openMode))
-            certs += QSslCertificate::fromData(file.readAll(), format);
+        certs += QSslCertificate::fromFile(filePath, format);
     }
     return certs;
 }
@@ -740,6 +752,30 @@ QList<QSslCertificate> QSslCertificate::fromData(const QByteArray &data, QSsl::E
     return reader(data, -1);
 }
 
+/*!
+    \since 6.10
+
+    Reads the data from the file \a filePath and parses all certificates
+    that are encoded in the specified \a format and returns a list of
+    QSslCertificate objects.
+
+    If \a filePath isn't a regular file, this method will return an empty
+    list.
+
+    \sa fromData(), fromPath()
+*/
+QList<QSslCertificate> QSslCertificate::fromFile(const QString &filePath,
+                                                 QSsl::EncodingFormat format)
+{
+    QFile file(filePath);
+    QIODevice::OpenMode openMode = QIODevice::ReadOnly;
+    if (format == QSsl::Pem)
+        openMode |= QIODevice::Text;
+    if (file.open(openMode))
+        return QSslCertificate::fromData(file.readAll(), format);
+    return {};
+}
+
 #ifndef QT_NO_SSL
 /*!
     Verifies a certificate chain. The chain to be verified is passed in the
@@ -878,7 +914,7 @@ static const char *const certificate_blacklist[] = {
 bool QSslCertificatePrivate::isBlacklisted(const QSslCertificate &certificate)
 {
     for (int a = 0; certificate_blacklist[a] != nullptr; a++) {
-        QString blacklistedCommonName = QString::fromUtf8(certificate_blacklist[(a+1)]);
+        auto blacklistedCommonName = QAnyStringView(QUtf8StringView(certificate_blacklist[(a+1)]));
         if (certificate.serialNumber() == certificate_blacklist[a++] &&
             (certificate.subjectInfo(QSslCertificate::CommonName).contains(blacklistedCommonName) ||
              certificate.issuerInfo(QSslCertificate::CommonName).contains(blacklistedCommonName)))
@@ -889,19 +925,18 @@ bool QSslCertificatePrivate::isBlacklisted(const QSslCertificate &certificate)
 
 QByteArray QSslCertificatePrivate::subjectInfoToString(QSslCertificate::SubjectInfo info)
 {
-    QByteArray str;
     switch (info) {
-    case QSslCertificate::Organization: str = QByteArray("O"); break;
-    case QSslCertificate::CommonName: str = QByteArray("CN"); break;
-    case QSslCertificate::LocalityName: str = QByteArray("L"); break;
-    case QSslCertificate::OrganizationalUnitName: str = QByteArray("OU"); break;
-    case QSslCertificate::CountryName: str = QByteArray("C"); break;
-    case QSslCertificate::StateOrProvinceName: str = QByteArray("ST"); break;
-    case QSslCertificate::DistinguishedNameQualifier: str = QByteArray("dnQualifier"); break;
-    case QSslCertificate::SerialNumber: str = QByteArray("serialNumber"); break;
-    case QSslCertificate::EmailAddress: str = QByteArray("emailAddress"); break;
+    case QSslCertificate::Organization: return "O"_ba;
+    case QSslCertificate::CommonName: return "CN"_ba;
+    case QSslCertificate::LocalityName: return"L"_ba;
+    case QSslCertificate::OrganizationalUnitName: return "OU"_ba;
+    case QSslCertificate::CountryName: return "C"_ba;
+    case QSslCertificate::StateOrProvinceName: return "ST"_ba;
+    case QSslCertificate::DistinguishedNameQualifier: return "dnQualifier"_ba;
+    case QSslCertificate::SerialNumber: return "serialNumber"_ba;
+    case QSslCertificate::EmailAddress: return "emailAddress"_ba;
     }
-    return str;
+    return QByteArray();
 }
 
 /*!
@@ -918,13 +953,13 @@ QString QSslCertificate::issuerDisplayName() const
     QStringList names;
     names = issuerInfo(QSslCertificate::CommonName);
     if (!names.isEmpty())
-        return names.first();
+        return names.constFirst();
     names = issuerInfo(QSslCertificate::Organization);
     if (!names.isEmpty())
-        return names.first();
+        return names.constFirst();
     names = issuerInfo(QSslCertificate::OrganizationalUnitName);
     if (!names.isEmpty())
-        return names.first();
+        return names.constFirst();
 
     return QString();
 }
@@ -943,21 +978,20 @@ QString QSslCertificate::subjectDisplayName() const
     QStringList names;
     names = subjectInfo(QSslCertificate::CommonName);
     if (!names.isEmpty())
-        return names.first();
+        return names.constFirst();
     names = subjectInfo(QSslCertificate::Organization);
     if (!names.isEmpty())
-        return names.first();
+        return names.constFirst();
     names = subjectInfo(QSslCertificate::OrganizationalUnitName);
     if (!names.isEmpty())
-        return names.first();
+        return names.constFirst();
 
     return QString();
 }
 
 /*!
-    Returns the hash value for the \a key, using \a seed to seed the calculation.
     \since 5.4
-    \relates QHash
+    \qhashold{QHash}
 */
 size_t qHash(const QSslCertificate &key, size_t seed) noexcept
 {
@@ -974,15 +1008,15 @@ QDebug operator<<(QDebug debug, const QSslCertificate &certificate)
     QDebugStateSaver saver(debug);
     debug.resetFormat().nospace();
     debug << "QSslCertificate("
-          << certificate.version()
-          << ", " << certificate.serialNumber()
-          << ", " << certificate.digest().toBase64()
-          << ", " << certificate.issuerDisplayName()
-          << ", " << certificate.subjectDisplayName()
-          << ", " << certificate.subjectAlternativeNames()
+          << "Version=" << certificate.version()
+          << ", SerialNumber=" << certificate.serialNumber()
+          << ", Digest=" << certificate.digest().toBase64()
+          << ", Issuer=" << certificate.issuerDisplayName()
+          << ", Subject=" << certificate.subjectDisplayName()
+          << ", AlternativeSubjectNames=" << certificate.subjectAlternativeNames()
 #if QT_CONFIG(datestring)
-          << ", " << certificate.effectiveDate()
-          << ", " << certificate.expiryDate()
+          << ", EffectiveDate=" << certificate.effectiveDate()
+          << ", ExpiryDate=" << certificate.expiryDate()
 #endif
           << ')';
     return debug;
