@@ -43,7 +43,6 @@ public:
 
     void setWindowState(Qt::WindowStates state) override;
     void setWindowFlags(Qt::WindowFlags flags) override;
-    Qt::WindowFlags windowFlags() const;
     void setParent(const QPlatformWindow *window) override;
 
     WId winId() const override;
@@ -58,7 +57,7 @@ public:
 
     void propagateSizeHints() override;
     void requestActivateWindow() override;
-    void updateSystemUiVisibility();
+    void updateSystemUiVisibility(Qt::WindowStates states, Qt::WindowFlags flags);
     void updateFocusedEditText();
     inline bool isRaster() const { return m_isRaster; }
     bool isExposed() const override;
@@ -73,6 +72,8 @@ public:
     void lockSurface() { m_surfaceMutex.lock(); }
     void unlockSurface() { m_surfaceMutex.unlock(); }
 
+    static int surfacesCount();
+
 protected:
     void setGeometry(const QRect &rect) override;
     void createSurface();
@@ -81,9 +82,9 @@ protected:
     bool blockedByModal() const;
     bool isEmbeddingContainer() const;
     virtual void clearSurface() {}
+    static void incrementSurfacesCount();
+    static void decrementSurfacesCount();
 
-    Qt::WindowFlags m_windowFlags;
-    Qt::WindowStates m_windowState;
     bool m_isRaster;
 
     int m_nativeViewId = -1;
@@ -97,7 +98,7 @@ protected:
     // destroy the Surface.
     QtJniTypes::Surface m_androidSurfaceObject;
     QWaitCondition m_surfaceWaitCondition;
-    bool m_surfaceCreated = false;
+    bool m_androidSurfaceCreated = false;
     QMutex m_surfaceMutex;
     QMutex m_destructionMutex;
 
