@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #ifndef QDIRECTFBWINDOW_H
 #define QDIRECTFBWINDOW_H
@@ -15,19 +16,19 @@ class QDirectFbWindow : public QPlatformWindow
 {
 public:
     QDirectFbWindow(QWindow *tlw, QDirectFbInput *inputhandler);
-    ~QDirectFbWindow();
+    ~QDirectFbWindow() override;
 
-    void setGeometry(const QRect &rect);
-    void setOpacity(qreal level);
+    void setGeometry(const QRect &rect) override;
+    void setOpacity(qreal level) override;
 
-    void setVisible(bool visible);
+    void setVisible(bool visible) override;
 
-    void setWindowFlags(Qt::WindowFlags flags);
-    bool setKeyboardGrabEnabled(bool grab);
-    bool setMouseGrabEnabled(bool grab);
-    void raise();
-    void lower();
-    WId winId() const;
+    void setWindowFlags(Qt::WindowFlags flags) override;
+    bool setKeyboardGrabEnabled(bool grab) override;
+    bool setMouseGrabEnabled(bool grab) override;
+    void raise() override;
+    void lower() override;
+    WId winId() const override;
 
     virtual void createDirectFBWindow();
     IDirectFBWindow *dfbWindow() const;
