@@ -6,7 +6,9 @@
 #include "qwasmcompositor.h"
 #include "qwasmopenglcontext.h"
 #include "qwasmtheme.h"
+#if QT_CONFIG(clipboard)
 #include "qwasmclipboard.h"
+#endif
 #include "qwasmaccessibility.h"
 #include "qwasmservices.h"
 #include "qwasmoffscreensurface.h"
@@ -14,7 +16,9 @@
 #include "qwasmwindow.h"
 #include "qwasmbackingstore.h"
 #include "qwasmfontdatabase.h"
+#if QT_CONFIG(draganddrop)
 #include "qwasmdrag.h"
+#endif
 
 #include <qpa/qplatformwindow.h>
 #include <QtGui/qscreen.h>
@@ -22,13 +26,16 @@
 #include <QtCore/qcoreapplication.h>
 #include <qpa/qplatforminputcontextfactory_p.h>
 #include <qpa/qwindowsysteminterface_p.h>
+#include "private/qwasmsuspendresumecontrol_p.h"
 
 #include <emscripten/bind.h>
 #include <emscripten/val.h>
 
 // this is where EGL headers are pulled in, make sure it is last
 #include "qwasmscreen.h"
+#if QT_CONFIG(draganddrop)
 #include <private/qsimpledrag_p.h>
+#endif
 
 QT_BEGIN_NAMESPACE
 
@@ -88,9 +95,12 @@ EMSCRIPTEN_BINDINGS(qtQWasmIntegraton)
 QWasmIntegration *QWasmIntegration::s_instance;
 
 QWasmIntegration::QWasmIntegration()
-    : m_fontDb(nullptr)
+    : m_suspendResume(std::make_shared<QWasmSuspendResumeControl>()) // create early in order to register event handlers at startup
+    , m_fontDb(nullptr)
     , m_desktopServices(nullptr)
+#if QT_CONFIG(clipboard)
     , m_clipboard(new QWasmClipboard)
+#endif
 #if QT_CONFIG(accessibility)
     , m_accessibility(new QWasmAccessibility)
 #endif
@@ -142,7 +152,9 @@ QWasmIntegration::QWasmIntegration()
         visualViewport.call<void>("addEventListener", val("resize"),
                                   val::module_property("qtResizeAllScreens"));
     }
+#if QT_CONFIG(draganddrop)
     m_drag = std::make_unique<QWasmDrag>();
+#endif
 }
 
 QWasmIntegration::~QWasmIntegration()
@@ -178,17 +190,29 @@ bool QWasmIntegration::hasCapability(QPlatformIntegration::Capability cap) const
     case RasterGLSurface: return false; // to enable this you need to fix qopenglwidget and quickwidget for wasm
     case MultipleWindows: return true;
     case WindowManagement: return true;
+    case ForeignWindows: return true;
     case OpenGLOnRasterSurface: return true;
+    case OffscreenSurface: return true;
     default: return QPlatformIntegration::hasCapability(cap);
     }
 }
 
-QPlatformWindow *QWasmIntegration::createPlatformWindow(QWindow *window) const
+QWasmWindow *QWasmIntegration::createWindow(QWindow *window, WId nativeHandle) const
 {
     auto *wasmScreen = QWasmScreen::get(window->screen());
     QWasmCompositor *compositor = wasmScreen->compositor();
     return new QWasmWindow(window, wasmScreen->deadKeySupport(), compositor,
-                           m_backingStores.value(window));
+                           m_backingStores.value(window), nativeHandle);
+}
+
+QPlatformWindow *QWasmIntegration::createPlatformWindow(QWindow *window) const
+{
+    return createWindow(window, 0);
+}
+
+QPlatformWindow *QWasmIntegration::createForeignWindow(QWindow *window, WId nativeHandle) const
+{
+    return createWindow(window, nativeHandle);
 }
 
 QPlatformBackingStore *QWasmIntegration::createPlatformBackingStore(QWindow *window) const
@@ -253,7 +277,7 @@ QPlatformFontDatabase *QWasmIntegration::fontDatabase() const
 
 QAbstractEventDispatcher *QWasmIntegration::createEventDispatcher() const
 {
-    return new QWasmEventDispatcher;
+    return new QWasmEventDispatcher(m_suspendResume);
 }
 
 QVariant QWasmIntegration::styleHint(QPlatformIntegration::StyleHint hint) const
@@ -296,10 +320,12 @@ QPlatformServices *QWasmIntegration::services() const
     return m_desktopServices;
 }
 
+#if QT_CONFIG(clipboard)
 QPlatformClipboard* QWasmIntegration::clipboard() const
 {
     return m_clipboard;
 }
+#endif
 
 #ifndef QT_NO_ACCESSIBILITY
 QPlatformAccessibility *QWasmIntegration::accessibility() const
