@@ -5,21 +5,21 @@
 #define QWASMWINDOW_H
 
 #include "qwasmintegration.h"
-#include <qpa/qplatformwindow.h>
-#include <qpa/qplatformwindow_p.h>
-#include <emscripten/html5.h>
 #include "qwasmbackingstore.h"
 #include "qwasmscreen.h"
 #include "qwasmcompositor.h"
 #include "qwasmwindownonclientarea.h"
 #include "qwasmwindowstack.h"
 #include "qwasmwindowtreenode.h"
+#include "qwasmevent.h"
 
 #include <QtCore/private/qstdweb_p.h>
-#include "QtGui/qopenglcontext.h"
-#include <QtOpenGL/qopengltextureblitter.h>
+#include <qpa/qwindowsysteminterface.h>
+#include <qpa/qplatformwindow.h>
+#include <qpa/qplatformwindow_p.h>
 
 #include <emscripten/val.h>
+#include <emscripten/html5.h>
 
 #include <memory>
 
@@ -29,7 +29,6 @@ namespace qstdweb {
 class EventCallback;
 }
 
-class ClientArea;
 struct KeyEvent;
 struct PointerEvent;
 class QWasmDeadKeySupport;
@@ -38,17 +37,22 @@ struct WheelEvent;
 Q_DECLARE_LOGGING_CATEGORY(qLcQpaWasmInputContext)
 
 class QWasmWindow final : public QPlatformWindow,
-                          public QWasmWindowTreeNode,
+                          public QWasmWindowTreeNode<>,
                           public QNativeInterface::Private::QWasmWindow
 {
 public:
     QWasmWindow(QWindow *w, QWasmDeadKeySupport *deadKeySupport, QWasmCompositor *compositor,
-                QWasmBackingStore *backingStore);
+                QWasmBackingStore *backingStore, WId nativeHandle);
     ~QWasmWindow() final;
 
-    static QWasmWindow *fromWindow(QWindow *window);
+    static QWasmWindow *fromWindow(const QWindow *window);
+    QWasmWindow *transientParent() const;
+    Qt::WindowFlags windowFlags() const;
+    bool isModal() const;
     QSurfaceFormat format() const override;
 
+    void registerEventHandlers();
+
     void paint();
     void setZOrder(int order);
     void setWindowCursor(QByteArray cssCursorName);
@@ -86,6 +90,7 @@ public:
     void setMask(const QRegion &region) final;
     void setParent(const QPlatformWindow *window) final;
     void focus();
+    void onAccessibilityEnable();
 
     QWasmScreen *platformScreen() const;
     void setBackingStore(QWasmBackingStore *store) { m_backingStore = store; }
@@ -96,6 +101,7 @@ public:
     emscripten::val context2d() const { return m_context2d; }
     emscripten::val a11yContainer() const { return m_a11yContainer; }
     emscripten::val inputHandlerElement() const { return m_window; }
+    emscripten::val inputElement() const { return m_inputElement; }
 
     // QNativeInterface::Private::QWasmWindow
     emscripten::val document() const override { return m_document; }
@@ -105,15 +111,23 @@ public:
     emscripten::val containerElement() final;
     QWasmWindowTreeNode *parentNode() final;
 
+public slots:
+    void onTransientParentChanged(QWindow *newTransientParent);
+    void onModalityChanged();
+
 private:
     friend class QWasmScreen;
     static constexpr auto defaultWindowSize = 160;
 
+    QMetaObject::Connection m_transientWindowChangedConnection;
+    QMetaObject::Connection m_modalityChangedConnection;
+
     // QWasmWindowTreeNode:
     QWasmWindow *asWasmWindow() final;
     void onParentChanged(QWasmWindowTreeNode *previous, QWasmWindowTreeNode *current,
-                         QWasmWindowStack::PositionPreference positionPreference) final;
+                         QWasmWindowStack<>::PositionPreference positionPreference) final;
 
+    void shutdown();
     void invalidate();
     bool hasFrame() const;
     bool hasTitleBar() const;
@@ -123,14 +137,25 @@ private:
     void applyWindowState();
     void commitParent(QWasmWindowTreeNode *parent);
 
-    void handleKeyEvent(const emscripten::val &event);
+    void handleKeyEvent(const KeyEvent &event);
     bool processKey(const KeyEvent &event);
-    void handleKeyForInputContextEvent(const emscripten::val &event);
+    void handleKeyForInputContextEvent(const KeyEvent &event);
     bool processKeyForInputContext(const KeyEvent &event);
-    void handlePointerEvent(const PointerEvent &event);
-    bool processPointer(const PointerEvent &event);
+    void handleInputEvent(emscripten::val event);
+    void handleCompositionStartEvent(emscripten::val event);
+    void handleCompositionUpdateEvent(emscripten::val event);
+    void handleCompositionEndEvent(emscripten::val event);
+    void handleBeforeInputEvent(emscripten::val event);
+
+    void handlePointerEnterLeaveEvent(const PointerEvent &event);
+    bool processPointerEnterLeave(const PointerEvent &event);
+    void processPointer(const PointerEvent &event);
+    bool deliverPointerEvent(const PointerEvent &event);
     void handleWheelEvent(const emscripten::val &event);
     bool processWheel(const WheelEvent &event);
+    Qt::WindowFlags fixTopLevelWindowFlags(Qt::WindowFlags) const;
+    bool shouldBeAboveTransientParentFlags(Qt::WindowFlags flags) const;
+    QWasmWindowStack<>::PositionPreference positionPreferenceFromWindowFlags(Qt::WindowFlags) const;
 
     QWasmCompositor *m_compositor = nullptr;
     QWasmBackingStore *m_backingStore = nullptr;
@@ -142,24 +167,45 @@ private:
     emscripten::val m_window;
     emscripten::val m_a11yContainer;
     emscripten::val m_canvas;
+    emscripten::val m_focusHelper;
+    emscripten::val m_inputElement;
+
     emscripten::val m_context2d = emscripten::val::undefined();
 
     std::unique_ptr<NonClientArea> m_nonClientArea;
-    std::unique_ptr<ClientArea> m_clientArea;
 
     QWasmWindowTreeNode *m_commitedParent = nullptr;
 
-    std::unique_ptr<qstdweb::EventCallback> m_keyDownCallback;
-    std::unique_ptr<qstdweb::EventCallback> m_keyUpCallback;
-    std::unique_ptr<qstdweb::EventCallback> m_keyDownCallbackForInputContext;
-    std::unique_ptr<qstdweb::EventCallback> m_keyUpCallbackForInputContext;
+    QWasmEventHandler m_keyDownCallback;
+    QWasmEventHandler m_keyUpCallback;
+    QWasmEventHandler m_keyDownCallbackForInputContext;
+    QWasmEventHandler m_keyUpCallbackForInputContext;
+    QWasmEventHandler m_inputCallback;
+    QWasmEventHandler m_compositionStartCallback;
+    QWasmEventHandler m_compositionUpdateCallback;
+    QWasmEventHandler m_compositionEndCallback;
+    QWasmEventHandler m_beforeInputCallback;
 
-    std::unique_ptr<qstdweb::EventCallback> m_pointerLeaveCallback;
-    std::unique_ptr<qstdweb::EventCallback> m_pointerEnterCallback;
+    QWasmEventHandler m_pointerDownCallback;
+    QWasmEventHandler m_pointerMoveCallback;
+    QWasmEventHandler m_pointerUpCallback;
+    QWasmEventHandler m_pointerCancelCallback;
+    QWasmEventHandler m_pointerLeaveCallback;
+    QWasmEventHandler m_pointerEnterCallback;
 
-    std::unique_ptr<qstdweb::EventCallback> m_dropCallback;
+    QWasmEventHandler m_dragOverCallback;
+    QWasmEventHandler m_dragStartCallback;
+    QWasmEventHandler m_dragEndCallback;
+    QWasmEventHandler m_dropCallback;
+    QWasmEventHandler m_dragLeaveCallback;
 
-    std::unique_ptr<qstdweb::EventCallback> m_wheelEventCallback;
+    QWasmEventHandler m_wheelEventCallback;
+
+    QMap<int, QWindowSystemInterface::TouchPoint> m_pointerIdToTouchPoints;
+
+    QWasmEventHandler m_cutCallback;
+    QWasmEventHandler m_copyCallback;
+    QWasmEventHandler m_pasteCallback;
 
     Qt::WindowStates m_state = Qt::WindowNoState;
     Qt::WindowStates m_previousWindowState = Qt::WindowNoState;
