@@ -39,9 +39,9 @@
 #include <QtGui/private/qabstractfileiconengine_p.h>
 #include <QtGui/private/qwindowsfontdatabase_p.h>
 #include <private/qhighdpiscaling_p.h>
-#include <private/qsystemlibrary_p.h>
 #include <private/qwinregistry_p.h>
 #include <QtCore/private/qfunctions_win_p.h>
+#include <QtGui/private/qwindowsthemecache_p.h>
 
 #include <algorithm>
 
@@ -95,6 +95,12 @@ static constexpr QColor getSysColor(winrt::Windows::UI::Color &&color)
 }
 #endif
 
+static inline QColor getSysColor(int index)
+{
+    COLORREF cr = GetSysColor(index);
+    return QColor(GetRValue(cr), GetGValue(cr), GetBValue(cr));
+}
+
 [[maybe_unused]] [[nodiscard]] static inline QColor qt_accentColor(AccentColorLevel level)
 {
 #if QT_CONFIG(cpp_winrt)
@@ -108,18 +114,28 @@ static constexpr QColor getSysColor(winrt::Windows::UI::Color &&color)
     const QColor accentDarker = getSysColor(settings.GetColorValue(UIColorType::AccentDark2));
     const QColor accentDarkest = getSysColor(settings.GetColorValue(UIColorType::AccentDark3));
 #else
-    const QWinRegistryKey registry(HKEY_CURRENT_USER, LR"(Software\Microsoft\Windows\DWM)");
-    if (!registry.isValid())
+    const QColor accent = []()->QColor {
+        // MS uses the aquatic contrast theme as an example in the URL below:
+        // https://learn.microsoft.com/en-us/windows/win32/api/winuser/nf-winuser-getsyscolor#windows-1011-system-colors
+        if (QWindowsTheme::queryHighContrast())
+            return getSysColor(COLOR_HIGHLIGHT);
+        const QWinRegistryKey registry(HKEY_CURRENT_USER, LR"(Software\Microsoft\Windows\DWM)");
+        if (!registry.isValid())
+            return {};
+        const QVariant value = registry.value(L"AccentColor");
+        if (!value.isValid())
+            return {};
+        // The retrieved value is in the #AABBGGRR format, we need to
+        // convert it to the #AARRGGBB format which Qt expects.
+        const QColor abgr = QColor::fromRgba(qvariant_cast<DWORD>(value));
+        if (!abgr.isValid())
+            return {};
+        return QColor::fromRgb(abgr.blue(), abgr.green(), abgr.red(), abgr.alpha());
+
+    }();
+    if (!accent.isValid())
         return {};
-    const QVariant value = registry.value(L"AccentColor");
-    if (!value.isValid())
-        return {};
-    // The retrieved value is in the #AABBGGRR format, we need to
-    // convert it to the #AARRGGBB format which Qt expects.
-    const QColor abgr = QColor::fromRgba(qvariant_cast<DWORD>(value));
-    if (!abgr.isValid())
-        return {};
-    const QColor accent = QColor::fromRgb(abgr.blue(), abgr.green(), abgr.red(), abgr.alpha());
+
     const QColor accentLight = accent.lighter(120);
     const QColor accentLighter = accentLight.lighter(120);
     const QColor accentLightest = accentLighter.lighter(120);
@@ -145,12 +161,6 @@ static constexpr QColor getSysColor(winrt::Windows::UI::Color &&color)
     }
 }
 
-static inline QColor getSysColor(int index)
-{
-    COLORREF cr = GetSysColor(index);
-    return QColor(GetRValue(cr), GetGValue(cr), GetBValue(cr));
-}
-
 // QTBUG-48823/Windows 10: SHGetFileInfo() (as called by item views on file system
 // models has been observed to trigger a WM_PAINT on the mainwindow. Suppress the
 // behavior by running it in a thread.
@@ -284,6 +294,8 @@ static QColor placeHolderColor(QColor textColor)
 */
 void QWindowsTheme::populateLightSystemBasePalette(QPalette &result)
 {
+    const bool highContrastEnabled = queryHighContrast();
+
     const QColor background = getSysColor(COLOR_BTNFACE);
     const QColor textColor = getSysColor(COLOR_WINDOWTEXT);
 
@@ -292,7 +304,8 @@ void QWindowsTheme::populateLightSystemBasePalette(QPalette &result)
     const QColor accentDarker = qt_accentColor(AccentColorDarker);
     const QColor accentDarkest = qt_accentColor(AccentColorDarkest);
 
-    const QColor linkColor = accentDarker;
+    const QColor linkColor = highContrastEnabled ? getSysColor(COLOR_HOTLIGHT) : accentDarker;
+    const QColor linkColorVisited = highContrastEnabled ? linkColor.darker(120) : accentDarkest;
     const QColor btnFace = background;
     const QColor btnHighlight = getSysColor(COLOR_BTNHIGHLIGHT);
 
@@ -306,7 +319,7 @@ void QWindowsTheme::populateLightSystemBasePalette(QPalette &result)
     result.setColor(QPalette::PlaceholderText, placeHolderColor(textColor));
     result.setColor(QPalette::BrightText, btnHighlight);
     result.setColor(QPalette::Base, getSysColor(COLOR_WINDOW));
-    result.setColor(QPalette::Window, btnFace);
+    result.setColor(QPalette::Window, highContrastEnabled ? getSysColor(COLOR_WINDOW) : btnFace);
     result.setColor(QPalette::ButtonText, getSysColor(COLOR_BTNTEXT));
     result.setColor(QPalette::Midlight, getSysColor(COLOR_3DLIGHT));
     result.setColor(QPalette::Shadow, getSysColor(COLOR_3DDKSHADOW));
@@ -314,12 +327,15 @@ void QWindowsTheme::populateLightSystemBasePalette(QPalette &result)
     result.setColor(QPalette::Accent, accentDark); // default accent color for controls on Light mode is AccentDark1
 
     result.setColor(QPalette::Link, linkColor);
-    result.setColor(QPalette::LinkVisited, accentDarkest);
+    result.setColor(QPalette::LinkVisited, linkColorVisited);
     result.setColor(QPalette::Inactive, QPalette::Button, result.button().color());
     result.setColor(QPalette::Inactive, QPalette::Window, result.window().color());
     result.setColor(QPalette::Inactive, QPalette::Light, result.light().color());
     result.setColor(QPalette::Inactive, QPalette::Dark, result.dark().color());
 
+    if (highContrastEnabled)
+        result.setColor(QPalette::Inactive, QPalette::WindowText, getSysColor(COLOR_GRAYTEXT));
+
     if (result.midlight() == result.button())
         result.setColor(QPalette::Midlight, result.button().color().lighter(110));
 }
@@ -395,13 +411,14 @@ void QWindowsTheme::populateDarkSystemBasePalette(QPalette &result)
     result.setColor(QPalette::All, QPalette::Accent, accentLighter);
 }
 
-static inline QPalette toolTipPalette(const QPalette &systemPalette, bool light)
+static inline QPalette toolTipPalette(const QPalette &systemPalette, bool light, bool highContrastEnabled)
 {
     QPalette result(systemPalette);
-    const QColor tipBgColor = light ? getSysColor(COLOR_INFOBK)
-                                    : systemPalette.button().color();
-    const QColor tipTextColor = light ? getSysColor(COLOR_INFOTEXT)
-                                      : systemPalette.buttonText().color().darker(120);
+    static const bool isWindows11 = QOperatingSystemVersion::current() >= QOperatingSystemVersion::Windows11;
+    const QColor tipBgColor = highContrastEnabled ? (isWindows11 ? getSysColor(COLOR_WINDOW) : getSysColor(COLOR_BTNFACE)) :
+                                    (light ? getSysColor(COLOR_INFOBK) : systemPalette.button().color());
+    const QColor tipTextColor = highContrastEnabled ? (isWindows11 ? getSysColor(COLOR_WINDOWTEXT) : getSysColor(COLOR_BTNTEXT)) :
+                                    (light ? getSysColor(COLOR_INFOTEXT) : systemPalette.buttonText().color().darker(120));
 
     result.setColor(QPalette::All, QPalette::Button, tipBgColor);
     result.setColor(QPalette::All, QPalette::Window, tipBgColor);
@@ -481,6 +498,39 @@ static inline QPalette *menuBarPalette(const QPalette &menuPalette, bool light)
 const char *QWindowsTheme::name = "windows";
 QWindowsTheme *QWindowsTheme::m_instance = nullptr;
 
+LRESULT QT_WIN_CALLBACK qThemeChangeObserverWndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
+{
+    switch (message) {
+    case WM_SETTINGCHANGE:
+        // Only refresh the theme if the user changes the personalize settings
+        if (!((wParam == 0) && (lParam != 0) // lParam sometimes may be NULL.
+            && (wcscmp(reinterpret_cast<LPCWSTR>(lParam), L"ImmersiveColorSet") == 0)))
+            break;
+        Q_FALLTHROUGH();
+    case WM_THEMECHANGED:
+    case WM_SYSCOLORCHANGE:
+    case WM_DWMCOLORIZATIONCOLORCHANGED:
+        qCDebug(lcQpaTheme) << "Handling theme change due to"
+            << qUtf8Printable(decodeMSG(MSG{hwnd, message, wParam, lParam, 0, {0, 0}}).trimmed());
+        QWindowsTheme::handleThemeChange();
+
+        MSG msg; // Clear the message queue, we've already reacted to the change
+        while (PeekMessage(&msg, hwnd, 0, 0, PM_REMOVE));
+
+        // FIXME: Despite clearing the message queue above, Windows will send
+        // us redundant theme change events for our single window. We want the
+        // theme change delivery to be synchronous, so we can't easily debounce
+        // them by peeking into the QWSI event queue, but perhaps there are other
+        // ways.
+
+        break;
+    default:
+        break;
+    }
+
+    return DefWindowProc(hwnd, message, wParam, lParam);
+}
+
 QWindowsTheme::QWindowsTheme()
 {
     m_instance = this;
@@ -490,6 +540,16 @@ QWindowsTheme::QWindowsTheme()
     std::fill(m_palettes, m_palettes + NPalettes, nullptr);
     refresh();
     refreshIconPixmapSizes();
+
+    auto className = QWindowsContext::instance()->registerWindowClass(
+        QWindowsContext::classNamePrefix() + QLatin1String("ThemeChangeObserverWindow"),
+        qThemeChangeObserverWndProc);
+    // HWND_MESSAGE windows do not get the required theme events,
+    // so we use a real top-level window that we never show.
+    m_themeChangeObserver = CreateWindowEx(0, reinterpret_cast<LPCWSTR>(className.utf16()),
+        nullptr, WS_TILED, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
+        nullptr, nullptr, GetModuleHandle(nullptr), nullptr);
+    Q_ASSERT(m_themeChangeObserver);
 }
 
 QWindowsTheme::~QWindowsTheme()
@@ -499,6 +559,13 @@ QWindowsTheme::~QWindowsTheme()
     m_instance = nullptr;
 }
 
+void QWindowsTheme::destroyThemeChangeWindow()
+{
+    qCDebug(lcQpaTheme) << "Destroying theme change window";
+    DestroyWindow(m_themeChangeObserver);
+    m_themeChangeObserver = nullptr;
+}
+
 static inline QStringList iconThemeSearchPaths()
 {
     const QFileInfo appDir(QCoreApplication::applicationDirPath() + "/icons"_L1);
@@ -595,26 +662,39 @@ Qt::ColorScheme QWindowsTheme::effectiveColorScheme()
 void QWindowsTheme::requestColorScheme(Qt::ColorScheme scheme)
 {
     s_colorSchemeOverride = scheme;
-    handleSettingsChanged();
+    handleThemeChange();
 }
 
-void QWindowsTheme::handleSettingsChanged()
+Qt::ContrastPreference QWindowsTheme::contrastPreference() const
 {
+    return queryHighContrast() ? Qt::ContrastPreference::HighContrast
+                               : Qt::ContrastPreference::NoPreference;
+}
+
+void QWindowsTheme::handleThemeChange()
+{
+    auto *integration = QWindowsIntegration::instance();
+    if (!integration)
+        return;
+
+    QWindowsThemeCache::clearAllThemeCaches();
+
     const auto oldColorScheme = s_colorScheme;
     s_colorScheme = Qt::ColorScheme::Unknown; // make effectiveColorScheme() query registry
-    const auto newColorScheme = effectiveColorScheme();
-    const bool colorSchemeChanged = newColorScheme != oldColorScheme;
-    s_colorScheme = newColorScheme;
-    auto integration = QWindowsIntegration::instance();
-    integration->updateApplicationBadge();
-    if (integration->darkModeHandling().testFlag(QWindowsApplication::DarkModeStyle)) {
-        QWindowsTheme::instance()->refresh();
-        QWindowSystemInterface::handleThemeChange<QWindowSystemInterface::SynchronousDelivery>();
-    }
-    if (colorSchemeChanged) {
+    s_colorScheme = effectiveColorScheme();
+    if (s_colorScheme != oldColorScheme) {
+        // Only propagate color scheme changes if the scheme actually changed
+        integration->updateApplicationBadge();
+
         for (QWindowsWindow *w : std::as_const(QWindowsContext::instance()->windows()))
             w->setDarkBorder(s_colorScheme == Qt::ColorScheme::Dark);
     }
+
+    // But always reset palette and fonts, and signal the theme
+    // change, as other parts of the theme could have changed,
+    // such as the accent color.
+    QWindowsTheme::instance()->refresh();
+    QWindowSystemInterface::handleThemeChange<QWindowSystemInterface::SynchronousDelivery>();
 }
 
 void QWindowsTheme::clearPalettes()
@@ -632,7 +712,7 @@ void QWindowsTheme::refreshPalettes()
         || !QWindowsIntegration::instance()->darkModeHandling().testFlag(QWindowsApplication::DarkModeStyle);
     clearPalettes();
     m_palettes[SystemPalette] = new QPalette(QWindowsTheme::systemPalette(s_colorScheme));
-    m_palettes[ToolTipPalette] = new QPalette(toolTipPalette(*m_palettes[SystemPalette], light));
+    m_palettes[ToolTipPalette] = new QPalette(toolTipPalette(*m_palettes[SystemPalette], light, queryHighContrast()));
     m_palettes[MenuPalette] = new QPalette(menuPalette(*m_palettes[SystemPalette], light));
     m_palettes[MenuBarPalette] = menuBarPalette(*m_palettes[MenuPalette], light);
     if (!light) {
@@ -675,9 +755,15 @@ QPalette QWindowsTheme::systemPalette(Qt::ColorScheme colorScheme)
                          result.light(), result.dark(), result.mid(),
                          result.text(), result.brightText(), result.base(),
                          result.window());
-    result.setColor(QPalette::Disabled, QPalette::WindowText, disabled);
-    result.setColor(QPalette::Disabled, QPalette::Text, disabled);
-    result.setColor(QPalette::Disabled, QPalette::ButtonText, disabled);
+
+    const bool highContrastEnabled = queryHighContrast();
+    const QColor disabledTextColor = highContrastEnabled ? getSysColor(COLOR_GRAYTEXT) : disabled;
+    result.setColor(QPalette::Disabled, QPalette::WindowText, disabledTextColor);
+    result.setColor(QPalette::Disabled, QPalette::Text, disabledTextColor);
+    result.setColor(QPalette::Disabled, QPalette::ButtonText, disabledTextColor);
+    if (highContrastEnabled)
+        result.setColor(QPalette::Disabled, QPalette::Button, result.button().color().darker(150));
+
     result.setColor(QPalette::Disabled, QPalette::Highlight, result.color(QPalette::Highlight));
     result.setColor(QPalette::Disabled, QPalette::HighlightedText, result.color(QPalette::HighlightedText));
     result.setColor(QPalette::Disabled, QPalette::Accent, disabled);
@@ -779,12 +865,6 @@ QPlatformSystemTrayIcon *QWindowsTheme::createPlatformSystemTrayIcon() const
 }
 #endif
 
-void QWindowsTheme::windowsThemeChanged(QWindow * window)
-{
-    refresh();
-    QWindowSystemInterface::handleThemeChange(window);
-}
-
 static int fileIconSizes[FileIconSizeCount];
 
 void QWindowsTheme::refreshIconPixmapSizes()
@@ -806,15 +886,15 @@ Q_GUI_EXPORT QPixmap qt_pixmapFromWinHICON(HICON icon);
 
 static QPixmap loadIconFromShell32(int resourceId, QSizeF size)
 {
-    if (const HMODULE hmod = QSystemLibrary::load(L"shell32")) {
-        auto iconHandle =
-            static_cast<HICON>(LoadImage(hmod, MAKEINTRESOURCE(resourceId),
-                                         IMAGE_ICON, int(size.width()), int(size.height()), 0));
-        if (iconHandle) {
-            QPixmap iconpixmap = qt_pixmapFromWinHICON(iconHandle);
-            DestroyIcon(iconHandle);
-            return iconpixmap;
-        }
+    HMODULE shell32 = ::GetModuleHandleW(L"shell32.dll");
+    Q_ASSERT(shell32);
+    auto iconHandle =
+        static_cast<HICON>(LoadImage(shell32, MAKEINTRESOURCE(resourceId),
+                                     IMAGE_ICON, int(size.width()), int(size.height()), 0));
+    if (iconHandle) {
+        QPixmap iconpixmap = qt_pixmapFromWinHICON(iconHandle);
+        DestroyIcon(iconHandle);
+        return iconpixmap;
     }
     return QPixmap();
 }
