@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qxcbintegration.h"
 #include "qxcbconnection.h"
@@ -25,7 +26,7 @@
 #include <xcb/xcb.h>
 
 #include <QtGui/private/qgenericunixfontdatabase_p.h>
-#include <QtGui/private/qgenericunixservices_p.h>
+#include <QtGui/private/qdesktopunixservices_p.h>
 
 #include <stdio.h>
 
@@ -43,7 +44,10 @@
 #endif
 
 #include <qpa/qplatforminputcontextfactory_p.h>
-#include <private/qgenericunixthemes_p.h>
+#include <private/qgenericunixtheme_p.h>
+#if QT_CONFIG(dbus)
+#include <private/qkdetheme_p.h>
+#endif
 #include <qpa/qplatforminputcontext.h>
 
 #include <QtGui/QOpenGLContext>
@@ -71,7 +75,7 @@ using namespace Qt::StringLiterals;
 // or, for older Linuxes, read out 'cmdline'.
 static bool runningUnderDebugger()
 {
-#if defined(QT_DEBUG) && defined(Q_OS_LINUX)
+#if defined(Q_OS_LINUX)
     const QString parentProc = "/proc/"_L1 + QString::number(getppid());
     const QFileInfo parentProcExe(parentProc + "/exe"_L1);
     if (parentProcExe.isSymLink())
@@ -93,10 +97,12 @@ static bool runningUnderDebugger()
 #endif
 }
 
-class QXcbUnixServices : public QGenericUnixServices
+class QXcbUnixServices : public QDesktopUnixServices, public QXcbObject
 {
 public:
     QString portalWindowIdentifier(QWindow *window) override;
+    void registerDBusMenuForWindow(QWindow *window, const QString &service, const QString &path) override;
+    void unregisterDBusMenuForWindow(QWindow *window) override;
 };
 
 
@@ -155,12 +161,10 @@ QXcbIntegration::QXcbIntegration(const QStringList &parameters, int &argc, char
         doGrabArg = false;
     }
 
-#if defined(QT_DEBUG)
     if (!noGrabArg && !doGrabArg && underDebugger) {
         qCDebug(lcQpaXcb, "Qt: gdb: -nograb added to command-line options.\n"
                 "\t Use the -dograb option to enforce grabbing.");
     }
-#endif
     m_canGrab = (!underDebugger && !noGrabArg) || (underDebugger && doGrabArg);
 
     static bool canNotGrabEnv = qEnvironmentVariableIsSet("QT_XCB_NO_GRAB_SERVER");
@@ -173,6 +177,7 @@ QXcbIntegration::QXcbIntegration(const QStringList &parameters, int &argc, char
         m_connection = nullptr;
         return;
     }
+    m_services->setConnection(m_connection);
 
     m_fontDatabase.reset(new QGenericUnixFontDatabase());
 
@@ -316,6 +321,8 @@ bool QXcbIntegration::hasCapability(QPlatformIntegration::Capability cap) const
     case SyncState:
     case RasterGLSurface:
         return true;
+    case OffscreenSurface:
+        return m_connection->glIntegration() && m_connection->glIntegration()->canCreatePlatformOffscreenSurface();
 
     case SwitchableWidgetComposition:
     {
@@ -332,7 +339,6 @@ QAbstractEventDispatcher *QXcbIntegration::createEventDispatcher() const
     return QXcbEventDispatcher::createEventDispatcher(connection());
 }
 
-using namespace Qt::Literals::StringLiterals;
 static const auto xsNetCursorBlink = "Net/CursorBlink"_ba;
 static const auto xsNetCursorBlinkTime = "Net/CursorBlinkTime"_ba;
 static const auto xsNetDoubleClickTime = "Net/DoubleClickTime"_ba;
@@ -528,7 +534,7 @@ QByteArray QXcbIntegration::wmClass() const
         if (m_instanceName)
             name = QString::fromLocal8Bit(m_instanceName);
         if (name.isEmpty() && qEnvironmentVariableIsSet(resourceNameVar))
-            name = QString::fromLocal8Bit(qgetenv(resourceNameVar));
+            name = qEnvironmentVariable(resourceNameVar);
         if (name.isEmpty())
             name = argv0BaseName();
 
@@ -592,7 +598,7 @@ QPlatformVulkanInstance *QXcbIntegration::createPlatformVulkanInstance(QVulkanIn
 
 void QXcbIntegration::setApplicationBadge(qint64 number)
 {
-    auto unixServices = dynamic_cast<QGenericUnixServices *>(services());
+    auto unixServices = dynamic_cast<QDesktopUnixServices *>(services());
     unixServices->setApplicationBadge(number);
 }
 
@@ -601,4 +607,30 @@ QString QXcbUnixServices::portalWindowIdentifier(QWindow *window)
     return "x11:"_L1 + QString::number(window->winId(), 16);
 }
 
+void QXcbUnixServices::registerDBusMenuForWindow(QWindow *window, const QString &service, const QString &path)
+{
+    const QByteArray serviceValue = service.toLatin1();
+    const QByteArray pathValue = path.toLatin1();
+
+    xcb_change_property(xcb_connection(),
+                        XCB_PROP_MODE_REPLACE, window->winId(),
+                        atom(QXcbAtom::Atom_KDE_NET_WM_APPMENU_SERVICE_NAME),
+                        XCB_ATOM_STRING, 8,
+                        serviceValue.length(),
+                        serviceValue.constData());
+
+    xcb_change_property(xcb_connection(),
+                        XCB_PROP_MODE_REPLACE, window->winId(),
+                        atom(QXcbAtom::Atom_KDE_NET_WM_APPMENU_OBJECT_PATH),
+                        XCB_ATOM_STRING, 8,
+                        pathValue.length(),
+                        pathValue.constData());
+}
+
+void QXcbUnixServices::unregisterDBusMenuForWindow(QWindow *window)
+{
+    xcb_delete_property(xcb_connection(), window->winId(), atom(QXcbAtom::Atom_KDE_NET_WM_APPMENU_SERVICE_NAME));
+    xcb_delete_property(xcb_connection(), window->winId(), atom(QXcbAtom::Atom_KDE_NET_WM_APPMENU_OBJECT_PATH));
+}
+
 QT_END_NAMESPACE
