@@ -1,5 +1,6 @@
 // Copyright (C) 2022 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 //
 //  W A R N I N G
@@ -15,6 +16,9 @@
 #include "qgtk3json_p.h"
 #include "qgtk3storage_p.h"
 #include <qpa/qwindowsysteminterface.h>
+#if QT_CONFIG(dbus)
+#  include <QtGui/private/qgnomeportalinterface_p.h>
+#endif
 
 QT_BEGIN_NAMESPACE
 
@@ -22,11 +26,13 @@ QGtk3Storage::QGtk3Storage()
 {
     m_interface.reset(new QGtk3Interface(this));
 #if QT_CONFIG(dbus)
-    m_portalInterface.reset(new QGtk3PortalInterface(this));
+    m_portalInterface.reset(new QGnomePortalInterface);
 #endif
     populateMap();
 }
 
+QGtk3Storage::~QGtk3Storage() { }
+
 /*!
     \internal
     \enum QGtk3Storage::SourceType
