@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:data-parser
 
 #include "qsql_db2_p.h"
 #include <qcoreapplication.h>
@@ -168,18 +169,7 @@ static QString qDB2Warn(const QDB2DriverPrivate* d, QStringList *errorCodes = nu
 static QString qDB2Warn(const QDB2ResultPrivate* d, QStringList *errorCodes = nullptr)
 {
     int errorCode = 0;
-    QString error = qWarnDB2Handle(SQL_HANDLE_ENV, d->drv_d_func()->hEnv, &errorCode);
-    if (errorCodes && errorCode != 0) {
-        *errorCodes << QString::number(errorCode);
-        errorCode = 0;
-    }
-    if (!error.isEmpty())
-        error += u' ';
-    error += qWarnDB2Handle(SQL_HANDLE_DBC, d->drv_d_func()->hDbc, &errorCode);
-    if (errorCodes && errorCode != 0) {
-        *errorCodes << QString::number(errorCode);
-        errorCode = 0;
-    }
+    QString error = qDB2Warn(d->drv_d_func());
     if (!error.isEmpty())
         error += u' ';
     error += qWarnDB2Handle(SQL_HANDLE_STMT, d->hStmt, &errorCode);
@@ -188,34 +178,20 @@ static QString qDB2Warn(const QDB2ResultPrivate* d, QStringList *errorCodes = nu
     return error;
 }
 
-static void qSqlWarning(const QString& message, const QDB2DriverPrivate* d)
+template <typename T>
+static void qSqlWarning(const QString &message, const T *d)
 {
     qWarning("%s\tError: %s", message.toLocal8Bit().constData(),
                               qDB2Warn(d).toLocal8Bit().constData());
 }
 
-static void qSqlWarning(const QString& message, const QDB2ResultPrivate* d)
-{
-    qWarning("%s\tError: %s", message.toLocal8Bit().constData(),
-                              qDB2Warn(d).toLocal8Bit().constData());
-}
-
-static QSqlError qMakeError(const QString& err, QSqlError::ErrorType type,
-                            const QDB2DriverPrivate* p)
+template <typename T>
+static QSqlError qMakeError(const QString &err, QSqlError::ErrorType type,
+                            const T *p)
 {
     QStringList errorCodes;
     const QString error = qDB2Warn(p, &errorCodes);
-    return QSqlError(QStringLiteral("QDB2: ") + err, error, type,
-                     errorCodes.join(u';'));
-}
-
-static QSqlError qMakeError(const QString& err, QSqlError::ErrorType type,
-                            const QDB2ResultPrivate* p)
-{
-    QStringList errorCodes;
-    const QString error = qDB2Warn(p, &errorCodes);
-    return QSqlError(QStringLiteral("QDB2: ") + err, error, type,
-                     errorCodes.join(u';'));
+    return QSqlError("QDB2: "_L1 + err, error, type, errorCodes.join(u';'));
 }
 
 static QMetaType qDecodeDB2Type(SQLSMALLINT sqltype)
