@@ -1,9 +1,11 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include <qsqldriverplugin.h>
 #include <qstringlist.h>
 #include "qsql_sqlite_p.h"
+#include "qsql_sqlite_vfs_p.h"
 
 QT_BEGIN_NAMESPACE
 
@@ -23,6 +25,7 @@ public:
 QSQLiteDriverPlugin::QSQLiteDriverPlugin()
     : QSqlDriverPlugin()
 {
+    register_qt_vfs();
 }
 
 QSqlDriver* QSQLiteDriverPlugin::create(const QString &name)
@@ -31,7 +34,8 @@ QSqlDriver* QSQLiteDriverPlugin::create(const QString &name)
         QSQLiteDriver* driver = new QSQLiteDriver();
         return driver;
     }
-    return 0;
+
+    return nullptr;
 }
 
 QT_END_NAMESPACE
