@@ -3,6 +3,7 @@
 // Copyright (C) 2014 Governikus GmbH & Co. KG.
 // Copyright (C) 2016 Richard J. Moore <rich@kde.org>
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:cryptography
 
 #include <QtNetwork/qsslsocket.h>
 #include <QtNetwork/qssldiffiehellmanparameters.h>
@@ -28,31 +29,29 @@ Q_GLOBAL_STATIC(bool, forceSecurityLevel)
 namespace QTlsPrivate
 {
 // These callback functions are defined in qtls_openssl.cpp.
-extern "C" int q_X509Callback(int ok, X509_STORE_CTX *ctx);
-extern "C" int q_X509CallbackDirect(int ok, X509_STORE_CTX *ctx);
+int q_X509Callback(int ok, X509_STORE_CTX *ctx);
+int q_X509CallbackDirect(int ok, X509_STORE_CTX *ctx);
 
 #if QT_CONFIG(ocsp)
-extern "C" int qt_OCSP_status_server_callback(SSL *ssl, void *);
+int qt_OCSP_status_server_callback(SSL *ssl, void *);
 #endif // ocsp
 
+#ifdef TLS1_3_VERSION
+int q_ssl_sess_set_new_cb(SSL *context, SSL_SESSION *session);
+#endif // TLS1_3_VERSION
+
 } // namespace QTlsPrivate
 
 #if QT_CONFIG(dtls)
 // defined in qdtls_openssl.cpp:
 namespace dtlscallbacks
 {
-extern "C" int q_X509DtlsCallback(int ok, X509_STORE_CTX *ctx);
-extern "C" int q_generate_cookie_callback(SSL *ssl, unsigned char *dst,
-                                          unsigned *cookieLength);
-extern "C" int q_verify_cookie_callback(SSL *ssl, const unsigned char *cookie,
-                                        unsigned cookieLength);
-}
+int q_X509DtlsCallback(int ok, X509_STORE_CTX *ctx);
+int q_generate_cookie_callback(SSL *ssl, unsigned char *dst, unsigned *cookieLength);
+int q_verify_cookie_callback(SSL *ssl, const unsigned char *cookie, unsigned cookieLength);
+} // namespace dtlscallbacks
 #endif // dtls
 
-#ifdef TLS1_3_VERSION
-extern "C" int q_ssl_sess_set_new_cb(SSL *context, SSL_SESSION *session);
-#endif // TLS1_3_VERSION
-
 static inline QString msgErrorSettingBackendConfig(const QString &why)
 {
     return QSslSocket::tr("Error when setting the OpenSSL configuration (%1)").arg(why);
@@ -666,7 +665,7 @@ QT_WARNING_POP
 #ifdef TLS1_3_VERSION
     // NewSessionTicket callback:
     if (mode == QSslSocket::SslClientMode && !isDtls) {
-        q_SSL_CTX_sess_set_new_cb(sslContext->ctx, q_ssl_sess_set_new_cb);
+        q_SSL_CTX_sess_set_new_cb(sslContext->ctx, QTlsPrivate::q_ssl_sess_set_new_cb);
         q_SSL_CTX_set_session_cache_mode(sslContext->ctx, SSL_SESS_CACHE_CLIENT);
     }
 
@@ -772,8 +771,7 @@ void QSslContext::applyBackendConfig(QSslContext *sslContext)
             if (!i.value().canConvert(QMetaType(QMetaType::QByteArray))) {
                 sslContext->errorCode = QSslError::UnspecifiedError;
                 sslContext->errorStr = msgErrorSettingBackendConfig(
-                QSslSocket::tr("Expecting QByteArray for %1").arg(
-                               QString::fromUtf8(i.key())));
+                QSslSocket::tr("Expecting QByteArray for %1").arg(i.key()));
                 return;
             }
 
@@ -786,18 +784,16 @@ void QSslContext::applyBackendConfig(QSslContext *sslContext)
             switch (result) {
             case 0:
                 sslContext->errorStr = msgErrorSettingBackendConfig(
-                    QSslSocket::tr("An error occurred attempting to set %1 to %2").arg(
-                        QString::fromUtf8(i.key()), QString::fromUtf8(value)));
+                    QSslSocket::tr("An error occurred attempting to set %1 to %2")
+                            .arg(i.key(), value));
                 return;
             case 1:
                 sslContext->errorStr = msgErrorSettingBackendConfig(
-                    QSslSocket::tr("Wrong value for %1 (%2)").arg(
-                        QString::fromUtf8(i.key()), QString::fromUtf8(value)));
+                    QSslSocket::tr("Wrong value for %1 (%2)").arg(i.key(), value));
                 return;
             default:
                 sslContext->errorStr = msgErrorSettingBackendConfig(
-                    QSslSocket::tr("Unrecognized command %1 = %2").arg(
-                        QString::fromUtf8(i.key()), QString::fromUtf8(value)));
+                    QSslSocket::tr("Unrecognized command %1 = %2").arg(i.key(), value));
                 return;
             }
         }
