@@ -389,7 +389,8 @@ QList<QSslCertificate> systemCaCertificates()
     {
         const QList<QByteArray> directories = QSslSocketPrivate::unixRootCertDirectories();
         QSet<QString> certFiles = {
-            QStringLiteral("/etc/pki/tls/certs/ca-bundle.crt"), // Fedora, Mandriva
+            QStringLiteral("/etc/pki/ca-trust/extracted/pem/tls-ca-bundle.pem"), // Red Hat 2013+
+            QStringLiteral("/etc/pki/tls/certs/ca-bundle.crt"), // Red Hat older, Mandriva
             QStringLiteral("/usr/local/share/certs/ca-root-nss.crt") // FreeBSD's ca_root_nss
         };
 
@@ -406,12 +407,17 @@ QList<QSslCertificate> systemCaCertificates()
         for (const QByteArray &directory : directories) {
             for (const auto &dirEntry : QDirListing(QFile::decodeName(directory), flags)) {
                 // use canonical path here to not load the same certificate twice if symlinked
-                if (hasMatchingExtension(dirEntry.fileName()))
-                    certFiles.insert(dirEntry.canonicalFilePath());
+                if (hasMatchingExtension(dirEntry.fileName())) {
+                    QString canonicalPath = dirEntry.canonicalFilePath();
+                    // skip broken symlinks to not end up adding "" to the list which will then
+                    // just be rejected by `QSslCertificate::fromFile`
+                    if (!canonicalPath.isEmpty())
+                        certFiles.insert(canonicalPath);
+                }
             }
         }
         for (const QString& file : std::as_const(certFiles))
-            systemCerts.append(QSslCertificate::fromPath(file, QSsl::Pem));
+            systemCerts.append(QSslCertificate::fromFile(file, QSsl::Pem));
     }
 #endif // platform
 #ifdef QSSLSOCKET_DEBUG
