@@ -1,41 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtWidgets module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #ifndef QGRAPHICSITEM_H
 #define QGRAPHICSITEM_H
@@ -468,7 +433,7 @@ private:
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(QGraphicsItem::GraphicsItemFlags)
-#ifndef Q_CLANG_QDOC
+#ifndef Q_QDOC
 Q_DECLARE_INTERFACE(QGraphicsItem, "org.qt-project.Qt.QGraphicsItem")
 #endif
 
@@ -536,13 +501,10 @@ class Q_WIDGETS_EXPORT QGraphicsObject : public QObject, public QGraphicsItem
 #if QT_CONFIG(graphicseffect)
     Q_PROPERTY(QGraphicsEffect *effect READ graphicsEffect WRITE setGraphicsEffect)
 #endif
-    Q_PRIVATE_PROPERTY(QGraphicsItem::d_func(), QDeclarativeListProperty<QGraphicsObject> children
-                       READ childrenList DESIGNABLE false NOTIFY childrenChanged)
     Q_PRIVATE_PROPERTY(QGraphicsItem::d_func(), qreal width READ width WRITE setWidth
                        NOTIFY widthChanged RESET resetWidth FINAL)
     Q_PRIVATE_PROPERTY(QGraphicsItem::d_func(), qreal height READ height WRITE setHeight
                        NOTIFY heightChanged RESET resetHeight FINAL)
-    Q_CLASSINFO("DefaultProperty", "children")
     Q_INTERFACES(QGraphicsItem)
 public:
     explicit QGraphicsObject(QGraphicsItem *parent = nullptr);
@@ -941,9 +903,6 @@ protected:
 
 private:
     Q_DISABLE_COPY(QGraphicsTextItem)
-    Q_PRIVATE_SLOT(dd, void _q_updateBoundingRect(const QSizeF &))
-    Q_PRIVATE_SLOT(dd, void _q_update(QRectF))
-    Q_PRIVATE_SLOT(dd, void _q_ensureVisible(QRectF))
     QGraphicsTextItemPrivate *dd;
     friend class QGraphicsTextItemPrivate;
 };
@@ -1011,15 +970,33 @@ private:
 template <class T> inline T qgraphicsitem_cast(QGraphicsItem *item)
 {
     typedef typename std::remove_cv<typename std::remove_pointer<T>::type>::type Item;
-    return int(Item::Type) == int(QGraphicsItem::Type)
-        || (item && int(Item::Type) == item->type()) ? static_cast<T>(item) : 0;
+    constexpr int ItemType = int(Item::Type);
+    if constexpr (int(Item::Type) == int(QGraphicsItem::Type))
+        return static_cast<T>(item);
+    else
+        return (item && ItemType == item->type()) ? static_cast<T>(item) : nullptr;
 }
 
 template <class T> inline T qgraphicsitem_cast(const QGraphicsItem *item)
 {
     typedef typename std::remove_cv<typename std::remove_pointer<T>::type>::type Item;
-    return int(Item::Type) == int(QGraphicsItem::Type)
-        || (item && int(Item::Type) == item->type()) ? static_cast<T>(item) : 0;
+    constexpr int ItemType = int(Item::Type);
+    if constexpr (ItemType == int(QGraphicsItem::Type))
+        return static_cast<T>(item);
+    else
+        return (item && ItemType == item->type()) ? static_cast<T>(item) : nullptr;
+}
+
+template <>
+inline QGraphicsItem *qgraphicsitem_cast<QGraphicsItem *>(QGraphicsItem *item)
+{
+    return item;
+}
+
+template <>
+inline const QGraphicsItem *qgraphicsitem_cast<const QGraphicsItem *>(const QGraphicsItem *item)
+{
+    return item;
 }
 
 #ifndef QT_NO_DEBUG_STREAM
@@ -1032,10 +1009,6 @@ Q_WIDGETS_EXPORT QDebug operator<<(QDebug debug, QGraphicsItem::GraphicsItemFlag
 
 QT_END_NAMESPACE
 
-Q_DECLARE_METATYPE(QGraphicsItem *)
-
-QT_BEGIN_NAMESPACE
-
-QT_END_NAMESPACE
+QT_DECL_METATYPE_EXTERN_TAGGED(QGraphicsItem*, QGraphicsItem_ptr, Q_WIDGETS_EXPORT)
 
 #endif // QGRAPHICSITEM_H
