@@ -1,5 +1,6 @@
 // Copyright (C) 2019 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qtreewidget.h"
 
@@ -42,7 +43,6 @@ public:
 
 /*!
     \enum QTreeWidgetItem::ChildIndicatorPolicy
-    \since 4.3
 
     \value ShowIndicator     The controls for expanding and collapsing will be shown for this item even if there are no children.
     \value DontShowIndicator   The controls for expanding and collapsing will never be shown even if there are children.  If the node is forced open the user will not be able to expand or collapse the item.
@@ -51,7 +51,6 @@ public:
 
 /*!
     \fn void QTreeWidgetItem::setDisabled(bool disabled)
-    \since 4.3
 
     Disables the item if \a disabled is true; otherwise enables the item.
 
@@ -60,7 +59,6 @@ public:
 
 /*!
     \fn bool QTreeWidgetItem::isDisabled() const
-    \since 4.3
 
     Returns \c true if the item is disabled; otherwise returns \c false.
 
@@ -740,8 +738,19 @@ Qt::DropActions QTreeModel::supportedDropActions() const
     return view()->supportedDropActions();
 }
 
+Qt::DropActions QTreeModel::supportedDragActions() const
+{
+#if QT_CONFIG(draganddrop)
+    return view()->supportedDragActions();
+#else
+    return Qt::DropActions(Qt::IgnoreAction);
+#endif
+}
+
 void QTreeModel::itemChanged(QTreeWidgetItem *item)
 {
+    if (item->columnCount() <= 0)
+        return;
     SkipSorting skipSorting(this); //this is kind of wrong, but not doing this would kill performance
     QModelIndex left = index(item, 0);
     QModelIndex right = index(item, item->columnCount() - 1);
@@ -952,7 +961,6 @@ void QTreeModel::timerEvent(QTimerEvent *ev)
 
 /*!
     \fn void QTreeWidgetItem::sortChildren(int column, Qt::SortOrder order)
-    \since 4.2
 
     Sorts the children of the item using the given \a order,
     by the values in the given \a column.
@@ -969,7 +977,6 @@ void QTreeModel::timerEvent(QTimerEvent *ev)
 
 /*!
   \fn void QTreeWidgetItem::setSelected(bool select)
-  \since 4.2
 
   Sets the selected state of the item to \a select.
 
@@ -989,7 +996,6 @@ void QTreeWidgetItem::setSelected(bool select)
 
 /*!
   \fn bool QTreeWidgetItem::isSelected() const
-  \since 4.2
 
   Returns \c true if the item is selected, otherwise returns \c false.
 
@@ -1002,7 +1008,6 @@ bool QTreeWidgetItem::isSelected() const
 
 /*!
   \fn void QTreeWidgetItem::setHidden(bool hide)
-  \since 4.2
 
   Hides the item if \a hide is true, otherwise shows the item.
   \note A call to this function has no effect if the item is not currently in a view. In particular,
@@ -1028,7 +1033,6 @@ void QTreeWidgetItem::setHidden(bool hide)
 
 /*!
   \fn bool QTreeWidgetItem::isHidden() const
-  \since 4.2
 
   Returns \c true if the item is hidden, otherwise returns \c false.
 
@@ -1050,7 +1054,6 @@ bool QTreeWidgetItem::isHidden() const
 
 /*!
   \fn void QTreeWidgetItem::setExpanded(bool expand)
-  \since 4.2
 
   Expands the item if \a expand is true, otherwise collapses the item.
   \warning The QTreeWidgetItem must be added to the QTreeWidget before calling this function.
@@ -1068,7 +1071,6 @@ void QTreeWidgetItem::setExpanded(bool expand)
 
 /*!
   \fn bool QTreeWidgetItem::isExpanded() const
-  \since 4.2
 
   Returns \c true if the item is expanded, otherwise returns \c false.
 
@@ -1085,7 +1087,6 @@ bool QTreeWidgetItem::isExpanded() const
 
 /*!
   \fn void QTreeWidgetItem::setFirstColumnSpanned(bool span)
-  \since 4.3
 
   Sets the first section to span all columns if \a span is true;
   otherwise all item sections are shown.
@@ -1103,7 +1104,6 @@ void QTreeWidgetItem::setFirstColumnSpanned(bool span)
 
 /*!
   \fn bool QTreeWidgetItem::isFirstColumnSpanned() const
-  \since 4.3
 
   Returns \c true if the item is spanning all the columns in a row; otherwise returns \c false.
 
@@ -1218,7 +1218,6 @@ bool QTreeWidgetItem::isFirstColumnSpanned() const
 
 /*!
     \fn QBrush QTreeWidgetItem::background(int column) const
-    \since 4.2
 
     Returns the brush used to render the background of the specified \a column.
 
@@ -1227,7 +1226,6 @@ bool QTreeWidgetItem::isFirstColumnSpanned() const
 
 /*!
     \fn void QTreeWidgetItem::setBackground(int column, const QBrush &brush)
-    \since 4.2
 
     Sets the background brush of the label in the given \a column to the
     specified \a brush.
@@ -1243,7 +1241,6 @@ bool QTreeWidgetItem::isFirstColumnSpanned() const
 
 /*!
     \fn QBrush QTreeWidgetItem::foreground(int column) const
-    \since 4.2
 
     Returns the brush used to render the foreground (e.g. text) of the
     specified \a column.
@@ -1255,7 +1252,6 @@ bool QTreeWidgetItem::isFirstColumnSpanned() const
 
 /*!
     \fn void QTreeWidgetItem::setForeground(int column, const QBrush &brush)
-    \since 4.2
 
     Sets the foreground brush of the label in the given \a column to the
     specified \a brush.
@@ -1281,7 +1277,6 @@ bool QTreeWidgetItem::isFirstColumnSpanned() const
 
 /*!
   \fn QSize QTreeWidgetItem::sizeHint(int column) const
-  \since 4.1
 
   Returns the size hint set for the tree item in the given
   \a column (see \l{QSize}).
@@ -1289,7 +1284,6 @@ bool QTreeWidgetItem::isFirstColumnSpanned() const
 
 /*!
   \fn void QTreeWidgetItem::setSizeHint(int column, const QSize &size)
-  \since 4.1
 
   Sets the size hint for the tree item in the given \a column to be \a size.
   If no size hint is set or \a size is invalid, the item
@@ -1895,8 +1889,6 @@ void QTreeWidgetItem::write(QDataStream &out) const
 #endif // QT_NO_DATASTREAM
 
 /*!
-    \since 4.1
-
     Constructs a copy of \a other. Note that type() and treeWidget()
     are not copied.
 
@@ -2032,8 +2024,6 @@ QTreeWidgetItem *QTreeWidgetItem::takeChild(int index)
 }
 
 /*!
-  \since 4.1
-
   Appends the given list of \a children to the item.
 
   \sa insertChildren(), takeChildren()
@@ -2044,8 +2034,6 @@ void QTreeWidgetItem::addChildren(const QList<QTreeWidgetItem*> &children)
 }
 
 /*!
-  \since 4.1
-
   Inserts the given list of \a children into the list of the item children at \a index .
 
   Children that have already been inserted somewhere else won't be inserted.
@@ -2099,8 +2087,6 @@ void QTreeWidgetItem::insertChildren(int index, const QList<QTreeWidgetItem*> &c
 }
 
 /*!
-  \since 4.1
-
   Removes the list of children and returns it, otherwise returns an empty list.
 */
 QList<QTreeWidgetItem*> QTreeWidgetItem::takeChildren()
@@ -2221,8 +2207,6 @@ QVariant QTreeWidgetItem::childrenCheckState(int column) const
 }
 
 /*!
-  \since 4.5
-
   Causes the model associated with this item to emit a
   \l{QAbstractItemModel::dataChanged()}{dataChanged}() signal for this
   item.
@@ -2417,7 +2401,7 @@ void QTreeWidgetPrivate::dataChanged(const QModelIndex &topLeft,
   \ingroup model-view
   \inmodule QtWidgets
 
-  \image fusion-treeview.png
+  \image fusion-treeview.png {Directory displaying its contents as a tree}
 
   The QTreeWidget class is a convenience class that provides a standard
   tree widget with a classic item-based interface similar to that used by
@@ -2562,8 +2546,6 @@ void QTreeWidgetPrivate::dataChanged(const QModelIndex &topLeft,
 */
 
 /*!
-  \since 4.3
-
   \fn void QTreeWidget::removeItemWidget(QTreeWidgetItem *item, int column)
 
   Removes the widget set in the given \a item in the given \a column.
@@ -2643,8 +2625,6 @@ void QTreeWidget::setColumnCount(int columns)
 }
 
 /*!
-  \since 4.2
-
   Returns the tree widget's invisible root item.
 
   The invisible root item provides access to the tree widget's top-level items
@@ -2702,8 +2682,6 @@ void QTreeWidget::insertTopLevelItem(int index, QTreeWidgetItem *item)
 }
 
 /*!
-    \since 4.1
-
     Appends the \a item as a top-level item in the widget.
 
     \sa insertTopLevelItem()
@@ -2740,8 +2718,6 @@ int QTreeWidget::indexOfTopLevelItem(QTreeWidgetItem *item) const
 }
 
 /*!
-  \since 4.1
-
   Inserts the list of \a items at \a index in the top level in the view.
 
   Items that have already been inserted somewhere else won't be inserted.
@@ -2827,7 +2803,6 @@ void QTreeWidget::setHeaderLabels(const QStringList &labels)
 
 /*!
     \fn void QTreeWidget::setHeaderLabel(const QString &label)
-    \since 4.2
 
     Same as setHeaderLabels(QStringList(\a label)).
 */
@@ -2844,7 +2819,6 @@ QTreeWidgetItem *QTreeWidget::currentItem() const
 }
 
 /*!
-    \since 4.1
     Returns the current column in the tree widget.
 
     \sa setCurrentItem(), columnCount()
@@ -2868,7 +2842,6 @@ void QTreeWidget::setCurrentItem(QTreeWidgetItem *item)
 }
 
 /*!
-  \since 4.1
   Sets the current \a item in the tree widget and the current column to \a column.
 
   \sa currentItem()
@@ -2880,7 +2853,6 @@ void QTreeWidget::setCurrentItem(QTreeWidgetItem *item, int column)
 }
 
 /*!
-  \since 4.4
   Sets the current \a item in the tree widget and the current column to \a column,
   using the given \a command.
 
@@ -2933,8 +2905,6 @@ QRect QTreeWidget::visualItemRect(const QTreeWidgetItem *item) const
 }
 
 /*!
-  \since 4.1
-
   Returns the column used to sort the contents of the widget.
 
   \sa sortItems()
@@ -3014,8 +2984,6 @@ bool QTreeWidget::isPersistentEditorOpen(QTreeWidgetItem *item, int column) cons
 }
 
 /*!
-    \since 4.1
-
     Returns the widget displayed in the cell specified by \a item and the given \a column.
 
     \sa setItemWidget(), removeItemWidget()
@@ -3027,8 +2995,6 @@ QWidget *QTreeWidget::itemWidget(QTreeWidgetItem *item, int column) const
 }
 
 /*!
-    \since 4.1
-
     Sets the given \a widget to be displayed in the cell specified by the given
     \a item and \a column.
 
@@ -3094,8 +3060,6 @@ QList<QTreeWidgetItem*> QTreeWidget::findItems(const QString &text, Qt::MatchFla
 
 
 /*!
-  \since 4.3
-
   Returns the item above the given \a item.
 */
 QTreeWidgetItem *QTreeWidget::itemAbove(const QTreeWidgetItem *item) const
@@ -3109,8 +3073,6 @@ QTreeWidgetItem *QTreeWidget::itemAbove(const QTreeWidgetItem *item) const
 }
 
 /*!
-  \since 4.3
-
   Returns the item visually below the given \a item.
 */
 QTreeWidgetItem *QTreeWidget::itemBelow(const QTreeWidgetItem *item) const
@@ -3241,7 +3203,7 @@ QMimeData *QTreeWidget::mimeData(const QList<QTreeWidgetItem *> &items) const
     successfully handled by decoding the mime data and inserting it
     into the model; otherwise it returns \c false.
 
-    \sa supportedDropActions()
+    \sa supportedDropActions(), supportedDragActions
 */
 bool QTreeWidget::dropMimeData(QTreeWidgetItem *parent, int index,
                                const QMimeData *data, Qt::DropAction action)
@@ -3254,13 +3216,34 @@ bool QTreeWidget::dropMimeData(QTreeWidgetItem *parent, int index,
 /*!
   Returns the drop actions supported by this view.
 
-  \sa Qt::DropActions
+  \sa Qt::DropActions, supportedDragActions, dropMimeData()
 */
 Qt::DropActions QTreeWidget::supportedDropActions() const
 {
     return model()->QAbstractItemModel::supportedDropActions() | Qt::MoveAction;
 }
 
+#if QT_CONFIG(draganddrop)
+/*!
+    \property QTreeWidget::supportedDragActions
+    \brief the drag actions supported by this view
+
+    \since 6.10
+    \sa Qt::DropActions, supportedDropActions()
+*/
+Qt::DropActions QTreeWidget::supportedDragActions() const
+{
+    Q_D(const QTreeWidget);
+    return d->supportedDragActions.value_or(supportedDropActions());
+}
+
+void QTreeWidget::setSupportedDragActions(Qt::DropActions actions)
+{
+    Q_D(QTreeWidget);
+    d->supportedDragActions = actions;
+}
+#endif // QT_CONFIG(draganddrop)
+
 /*!
     Returns the QModelIndex associated with the given \a item in the given \a column.
 
