@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qapplication.h"
 #include "qdebug.h"
@@ -1073,9 +1074,13 @@ QLayoutItem* QFormLayoutPrivate::replaceAt(int index, QLayoutItem *newitem)
         \li Qt Extended styles
     \row
         \li \inlineimage qformlayout-win.png
+            {Form layout in traditional desktop style}
         \li \inlineimage qformlayout-mac.png
+            {Form layout in macOS Aqua style}
         \li \inlineimage qformlayout-kde.png
+            {Form layout in KDE Plastique style}
         \li \inlineimage qformlayout-qpe.png
+            {Form layout in Qt Extended style}
     \row
         \li Traditional style used for Windows, GNOME, and earlier
            versions of KDE. Labels are left aligned, and expanding
