@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qstandardgestures_p.h"
 #include "qgesture.h"
@@ -16,6 +17,8 @@
 
 #ifndef QT_NO_GESTURES
 
+using namespace std::chrono_literals;
+
 QT_BEGIN_NAMESPACE
 
 // If the change in scale for a single touch event is out of this range,
@@ -397,12 +400,11 @@ QGestureRecognizer::Result QTapGestureRecognizer::recognize(QGesture *state,
     QTapGesture *q = static_cast<QTapGesture *>(state);
     QTapGesturePrivate *d = q->d_func();
 
-    const QTouchEvent *ev = static_cast<const QTouchEvent *>(event);
-
     QGestureRecognizer::Result result = QGestureRecognizer::CancelGesture;
 
     switch (event->type()) {
     case QEvent::TouchBegin: {
+        const auto ev = static_cast<const QTouchEvent *>(event);
         d->position = ev->points().at(0).position();
         q->setHotSpot(ev->points().at(0).globalPosition());
         result = QGestureRecognizer::TriggerGesture;
@@ -410,6 +412,7 @@ QGestureRecognizer::Result QTapGestureRecognizer::recognize(QGesture *state,
     }
     case QEvent::TouchUpdate:
     case QEvent::TouchEnd: {
+        const auto ev = static_cast<const QTouchEvent *>(event);
         if (q->state() != Qt::NoGesture && ev->points().size() == 1) {
             const QEventPoint &p = ev->points().at(0);
             QPoint delta = p.position().toPoint() - p.pressPosition().toPoint();
@@ -469,8 +472,7 @@ QTapAndHoldGestureRecognizer::recognize(QGesture *state, QObject *object,
     QTapAndHoldGesturePrivate *d = q->d_func();
 
     if (object == state && event->type() == QEvent::Timer) {
-        q->killTimer(d->timerId);
-        d->timerId = 0;
+        d->tapAndHoldTimer.stop();
         return QGestureRecognizer::FinishGesture | QGestureRecognizer::ConsumeEventHint;
     }
 
@@ -482,9 +484,7 @@ QTapAndHoldGestureRecognizer::recognize(QGesture *state, QObject *object,
         const QGraphicsSceneMouseEvent *gsme = static_cast<const QGraphicsSceneMouseEvent *>(event);
         d->position = gsme->screenPos();
         q->setHotSpot(d->position);
-        if (d->timerId)
-            q->killTimer(d->timerId);
-        d->timerId = q->startTimer(QTapAndHoldGesturePrivate::Timeout);
+        d->tapAndHoldTimer.start(QTapAndHoldGesturePrivate::Timeout * 1ms, q);
         return QGestureRecognizer::MayBeGesture; // we don't show a sign of life until the timeout
     }
 #endif
@@ -492,18 +492,14 @@ QTapAndHoldGestureRecognizer::recognize(QGesture *state, QObject *object,
         const QMouseEvent *me = static_cast<const QMouseEvent *>(event);
         d->position = me->globalPosition().toPoint();
         q->setHotSpot(d->position);
-        if (d->timerId)
-            q->killTimer(d->timerId);
-        d->timerId = q->startTimer(QTapAndHoldGesturePrivate::Timeout);
+        d->tapAndHoldTimer.start(QTapAndHoldGesturePrivate::Timeout * 1ms, q);
         return QGestureRecognizer::MayBeGesture; // we don't show a sign of life until the timeout
     }
     case QEvent::TouchBegin: {
         const QTouchEvent *ev = static_cast<const QTouchEvent *>(event);
         d->position = ev->points().at(0).globalPressPosition();
         q->setHotSpot(d->position);
-        if (d->timerId)
-            q->killTimer(d->timerId);
-        d->timerId = q->startTimer(QTapAndHoldGesturePrivate::Timeout);
+        d->tapAndHoldTimer.start(QTapAndHoldGesturePrivate::Timeout * 1ms, q);
         return QGestureRecognizer::MayBeGesture; // we don't show a sign of life until the timeout
     }
 #if QT_CONFIG(graphicsview)
@@ -514,7 +510,7 @@ QTapAndHoldGestureRecognizer::recognize(QGesture *state, QObject *object,
         return QGestureRecognizer::CancelGesture; // get out of the MayBeGesture state
     case QEvent::TouchUpdate: {
         const QTouchEvent *ev = static_cast<const QTouchEvent *>(event);
-        if (d->timerId && ev->points().size() == 1) {
+        if (d->tapAndHoldTimer.isActive() && ev->points().size() == 1) {
             const QEventPoint &p = ev->points().at(0);
             QPoint delta = p.position().toPoint() - p.pressPosition().toPoint();
             if (delta.manhattanLength() <= TapRadius)
@@ -525,7 +521,7 @@ QTapAndHoldGestureRecognizer::recognize(QGesture *state, QObject *object,
     case QEvent::MouseMove: {
         const QMouseEvent *me = static_cast<const QMouseEvent *>(event);
         QPoint delta = me->globalPosition().toPoint() - d->position.toPoint();
-        if (d->timerId && delta.manhattanLength() <= TapRadius)
+        if (d->tapAndHoldTimer.isActive() && delta.manhattanLength() <= TapRadius)
             return QGestureRecognizer::MayBeGesture;
         return QGestureRecognizer::CancelGesture;
     }
@@ -533,7 +529,7 @@ QTapAndHoldGestureRecognizer::recognize(QGesture *state, QObject *object,
     case QEvent::GraphicsSceneMouseMove: {
         const QGraphicsSceneMouseEvent *gsme = static_cast<const QGraphicsSceneMouseEvent *>(event);
         QPoint delta = gsme->screenPos() - d->position.toPoint();
-        if (d->timerId && delta.manhattanLength() <= TapRadius)
+        if (d->tapAndHoldTimer.isActive() && delta.manhattanLength() <= TapRadius)
             return QGestureRecognizer::MayBeGesture;
         return QGestureRecognizer::CancelGesture;
     }
@@ -549,9 +545,7 @@ void QTapAndHoldGestureRecognizer::reset(QGesture *state)
     QTapAndHoldGesturePrivate *d = q->d_func();
 
     d->position = QPointF();
-    if (d->timerId)
-        q->killTimer(d->timerId);
-    d->timerId = 0;
+    d->tapAndHoldTimer.stop();
 
     QGestureRecognizer::reset(state);
 }
