@@ -1,31 +1,27 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include <QtWidgets/private/qtwidgetsglobal_p.h>
 
 #include <qapplication.h>
 #include <qevent.h>
-#include <qpointer.h>
 #include <qstyle.h>
 #include <qstyleoption.h>
 #include <qstylepainter.h>
 #if QT_CONFIG(effects)
 #include <private/qeffects_p.h>
 #endif
-#include <qtextdocument.h>
-#include <qdebug.h>
 #include <qpa/qplatformscreen.h>
 #include <qpa/qplatformcursor.h>
 #if QT_CONFIG(style_stylesheet)
 #include <private/qstylesheetstyle_p.h>
 #endif
 
-#include <qlabel.h>
 #include <QtWidgets/private/qlabel_p.h>
 #include <QtGui/private/qhighdpiscaling_p.h>
 #include <qtooltip.h>
-
-#include <QtCore/qbasictimer.h>
+#include <QtWidgets/private/qtooltip_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -90,57 +86,6 @@ using namespace Qt::StringLiterals;
     \sa QWidget::toolTip, QAction::toolTip
 */
 
-class QTipLabel : public QLabel
-{
-    Q_OBJECT
-public:
-    QTipLabel(const QString &text, const QPoint &pos, QWidget *w, int msecDisplayTime);
-    ~QTipLabel();
-    static QTipLabel *instance;
-
-    void adjustTooltipScreen(const QPoint &pos);
-    void updateSize(const QPoint &pos);
-
-    bool eventFilter(QObject *, QEvent *) override;
-
-    QBasicTimer hideTimer, expireTimer;
-
-    bool fadingOut;
-
-    void reuseTip(const QString &text, int msecDisplayTime, const QPoint &pos);
-    void hideTip();
-    void hideTipImmediately();
-    void setTipRect(QWidget *w, const QRect &r);
-    void restartExpireTimer(int msecDisplayTime);
-    bool tipChanged(const QPoint &pos, const QString &text, QObject *o);
-    void placeTip(const QPoint &pos, QWidget *w);
-
-    static QScreen *getTipScreen(const QPoint &pos, QWidget *w);
-protected:
-    void timerEvent(QTimerEvent *e) override;
-    void paintEvent(QPaintEvent *e) override;
-    void mouseMoveEvent(QMouseEvent *e) override;
-    void resizeEvent(QResizeEvent *e) override;
-
-#if QT_CONFIG(style_stylesheet)
-public slots:
-    /** \internal
-      Cleanup the _q_stylesheet_parent property.
-     */
-    void styleSheetParentDestroyed() {
-        setProperty("_q_stylesheet_parent", QVariant());
-        styleSheetParent = nullptr;
-    }
-
-private:
-    QWidget *styleSheetParent;
-#endif
-
-private:
-    QWidget *widget;
-    QRect rect;
-};
-
 QTipLabel *QTipLabel::instance = nullptr;
 
 QTipLabel::QTipLabel(const QString &text, const QPoint &pos, QWidget *w, int msecDisplayTime)
@@ -149,6 +94,7 @@ QTipLabel::QTipLabel(const QString &text, const QPoint &pos, QWidget *w, int mse
     , styleSheetParent(nullptr)
 #endif
     , widget(nullptr)
+    , fadingOut(false)
 {
     delete instance;
     instance = this;
@@ -163,7 +109,6 @@ QTipLabel::QTipLabel(const QString &text, const QPoint &pos, QWidget *w, int mse
     qApp->installEventFilter(this);
     setWindowOpacity(style()->styleHint(QStyle::SH_ToolTipLabel_Opacity, nullptr, this) / 255.0);
     setMouseTracking(true);
-    fadingOut = false;
     reuseTip(text, msecDisplayTime, pos);
 }
 
@@ -237,10 +182,10 @@ void QTipLabel::resizeEvent(QResizeEvent *e)
 void QTipLabel::mouseMoveEvent(QMouseEvent *e)
 {
     if (!rect.isNull()) {
-        QPoint pos = e->globalPosition().toPoint();
+        QPointF pos = e->globalPosition();
         if (widget)
             pos = widget->mapFromGlobal(pos);
-        if (!rect.contains(pos))
+        if (!rect.contains(pos.toPoint()))
             hideTip();
     }
     QLabel::mouseMoveEvent(e);
@@ -387,9 +332,9 @@ void QTipLabel::placeTip(const QPoint &pos, QWidget *w)
 
         QRect screenRect = screen->geometry();
         if (p.x() + this->width() > screenRect.x() + screenRect.width())
-        p.rx() -= 4 + this->width();
+            p.rx() -= 4 + this->width();
         if (p.y() + this->height() > screenRect.y() + screenRect.height())
-        p.ry() -= 24 + this->height();
+            p.ry() -= 24 + this->height();
         if (p.y() < screenRect.y())
             p.setY(screenRect.y());
         if (p.x() + this->width() > screenRect.x() + screenRect.width())
@@ -416,6 +361,15 @@ bool QTipLabel::tipChanged(const QPoint &pos, const QString &text, QObject *o)
        return false;
 }
 
+/** \internal
+  Cleanup the _q_stylesheet_parent property.
+ */
+void QTipLabel::styleSheetParentDestroyed()
+{
+    setProperty("_q_stylesheet_parent", QVariant());
+    styleSheetParent = nullptr;
+}
+
 /*!
     Shows \a text as a tool tip, with the global position \a pos as
     the point of interest. The tool tip will be shown with a platform
@@ -577,4 +531,4 @@ void QToolTip::setFont(const QFont &font)
 
 QT_END_NAMESPACE
 
-#include "qtooltip.moc"
+#include "moc_qtooltip_p.cpp"
