@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qlineedit.h"
 #include "qlineedit_p.h"
@@ -99,7 +100,7 @@ void QLineEdit::initStyleOption(QStyleOptionFrame *option) const
     \ingroup basicwidgets
     \inmodule QtWidgets
 
-    \image fusion-lineedit.png
+    \image fusion-lineedit.png {Line edit showing a text greeting}
 
     A line edit allows users to enter and edit a single line of
     plain text with useful editing functions, including undo and redo, cut and
