@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 /*!
     \class QMdiArea
@@ -52,7 +53,9 @@
 
     \table
     \row \li \inlineimage mdi-cascade.png
+             {MDI windows in cascading order}
          \li \inlineimage mdi-tile.png
+             {MDI windows in a tiling pattern}
     \endtable
 
     \note The default scroll bar property for QMdiArea is Qt::ScrollBarAlwaysOff.
@@ -1170,6 +1173,8 @@ void QMdiAreaPrivate::updateScrollBars()
     const int startX = q->isLeftToRight() ? childrenRect.left() : viewportRect.right()
                                                                   - childrenRect.right();
 
+    const auto singleStep = defaultSingleStep();
+
     // Horizontal scroll bar.
     if (isSubWindowsTiled && hbar->value() != 0)
         hbar->setValue(0);
@@ -1177,7 +1182,7 @@ void QMdiAreaPrivate::updateScrollBars()
     hbar->setRange(qMin(0, xOffset),
                    qMax(0, xOffset + childrenRect.width() - viewportRect.width()));
     hbar->setPageStep(childrenRect.width());
-    hbar->setSingleStep(childrenRect.width() / 20);
+    hbar->setSingleStep(childrenRect.width() / singleStep);
 
     // Vertical scroll bar.
     if (isSubWindowsTiled && vbar->value() != 0)
@@ -1186,7 +1191,7 @@ void QMdiAreaPrivate::updateScrollBars()
     vbar->setRange(qMin(0, yOffset),
                    qMax(0, yOffset + childrenRect.height() - viewportRect.height()));
     vbar->setPageStep(childrenRect.height());
-    vbar->setSingleStep(childrenRect.height() / 20);
+    vbar->setSingleStep(childrenRect.height() / singleStep);
 }
 
 /*!
