@@ -2,7 +2,7 @@
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QtCore/qtresource.h>
-#include <QtTest/QtTest>
+#include <QtTest/QTest>
 
 class TestManualResourceInit : public QObject
 {
