@@ -1,19 +1,24 @@
 // Copyright (C) 2021 The Qt Company Ltd.
 // Copyright (C) 2017 Intel Corporation.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QTest>
 #include <qcoreapplication.h>
-#include <qstring.h>
-#include <qtemporarydir.h>
-#include <qtemporaryfile.h>
 #include <qfile.h>
 #include <qdatetime.h>
 #include <qdir.h>
-#include <qset.h>
+#include <qstandardpaths.h>
+#include <qstring.h>
+#include <qtemporarydir.h>
+#include <qtemporaryfile.h>
 
 #include <QtTest/private/qtesthelpers_p.h>
 
+#include <QtCore/private/qduplicatetracker_p.h>
+#include <QtCore/qscopeguard.h>
+
+#include <optional>
+
 #if defined(Q_OS_WIN)
 # include <shlwapi.h>
 # include <qt_windows.h>
@@ -61,8 +66,10 @@ private slots:
     void resize();
     void openOnRootDrives();
     void stressTest();
+    void rename_data();
     void rename();
     void renameFdLeak();
+    void moveToTrash();
     void reOpenThroughQFile();
     void keepOpenMode();
     void resetTemplateAfterError();
@@ -73,6 +80,7 @@ private slots:
     void QTBUG_4796_data();
     void QTBUG_4796();
     void guaranteeUnique();
+    void stdfilesystem();
 private:
     QTemporaryDir m_temporaryDir;
     QString m_previousCurrent;
@@ -80,6 +88,9 @@ private:
 
 void tst_QTemporaryFile::initTestCase()
 {
+    QStandardPaths::setTestModeEnabled(true);
+    QDir().mkpath(QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation));
+
     QVERIFY2(m_temporaryDir.isValid(), qPrintable(m_temporaryDir.errorString()));
     m_previousCurrent = QDir::currentPath();
     QVERIFY(QDir::setCurrent(m_temporaryDir.path()));
@@ -174,6 +185,7 @@ void tst_QTemporaryFile::fileTemplate_data()
     QTest::newRow("constructor with XXXXX prefix") << "qt_XXXXX" << "qt_XXXXX." << "" << "";
     QTest::newRow("constructor with XXXX  prefix and suffix") << "qt_XXXX_XXXXXX_XXXX" << "qt_XXXX_" << "_XXXX" << "";
     QTest::newRow("constructor with XXXXX prefix and suffix") << "qt_XXXXX_XXXXXX_XXXXX" << "qt_XXXXX_" << "_XXXXX" << "";
+    QTest::newRow("constructor with two placeholders") << "qt_XXXXXX_XXXXXX_" << "qt_XXXXXX_" << "_" << "";
 
     QTest::newRow("set template, no suffix") << "" << "foo" << "" << "foo";
     QTest::newRow("set template, with lowercase XXXXXX") << "" << "qt_" << "xxxxxx" << "qt_XXXXXXxxxxxx";
@@ -248,7 +260,7 @@ void tst_QTemporaryFile::fileName()
     QString absoluteTempPath = QDir(tempPath).absolutePath();
     QTemporaryFile file;
     file.setAutoRemove(true);
-    file.open();
+    QVERIFY(file.open());
     QString fileName = file.fileName();
     QVERIFY2(fileName.contains("/tst_qtemporaryfile."), qPrintable(fileName));
     QVERIFY(QFile::exists(fileName));
@@ -420,7 +432,7 @@ void tst_QTemporaryFile::io()
 
     file.reset();
     QFile compare(file.fileName());
-    compare.open(QIODevice::ReadOnly);
+    QVERIFY(compare.open(QIODevice::ReadOnly));
     QCOMPARE(compare.readAll() , data);
     QCOMPARE(compare.fileTime(QFile::FileModificationTime), mtime);
 }
@@ -454,7 +466,7 @@ void tst_QTemporaryFile::removeAndReOpen()
     QString fileName;
     {
         QTemporaryFile file;
-        file.open();
+        QVERIFY(file.open());
         fileName = file.fileName();     // materializes any unnamed file
         QVERIFY(QFile::exists(fileName));
 
@@ -474,7 +486,7 @@ void tst_QTemporaryFile::removeAndReOpen()
 void tst_QTemporaryFile::removeUnnamed()
 {
     QTemporaryFile file;
-    file.open();
+    QVERIFY(file.open());
 
     // we did not call fileName(), so the file name may not have a name
     QVERIFY(file.remove());
@@ -522,7 +534,8 @@ void tst_QTemporaryFile::openOnRootDrives()
 #endif
     // If it's possible to create a file in the root directory, it
     // must be possible to create a temp file there too.
-    foreach (QFileInfo driveInfo, QDir::drives()) {
+    const auto drives = QDir::drives();
+    for (const QFileInfo &driveInfo : drives) {
         QFile testFile(driveInfo.filePath() + "XXXXXX.txt");
         if (testFile.open(QIODevice::ReadWrite)) {
             testFile.remove();
@@ -541,25 +554,45 @@ void tst_QTemporaryFile::openOnRootDrives()
 
 void tst_QTemporaryFile::stressTest()
 {
-    const int iterations = 1000;
+    constexpr int iterations = 1000;
+
+    QDuplicateTracker<QString, iterations> names;
+
+    const auto remover = qScopeGuard([&] {
+            for (const QString &s : std::as_const(names))
+                QFile::remove(s);
+        });
 
-    QSet<QString> names;
     for (int i = 0; i < iterations; ++i) {
         QTemporaryFile file;
         file.setAutoRemove(false);
         QVERIFY2(file.open(), qPrintable(file.errorString()));
-        QVERIFY(!names.contains(file.fileName()));
-        names.insert(file.fileName());
-    }
-    for (QSet<QString>::const_iterator it = names.constBegin(); it != names.constEnd(); ++it) {
-        QFile::remove(*it);
+        QVERIFY(!names.hasSeen(file.fileName()));
     }
 }
 
+void tst_QTemporaryFile::rename_data()
+{
+    QTest::addColumn<bool>("targetExists");
+    QTest::addColumn<bool>("expectedRenameResult");
+
+    QTest::addRow("success") << false << true;
+    QTest::addRow("failure") << true  << false;
+}
+
+static QByteArray read_all(const QString &fileName)
+{
+    QFile file(fileName);
+    if (file.open(QFile::ReadOnly))
+        return file.readAll();
+    else
+        return QByteArray();
+}
+
 void tst_QTemporaryFile::rename()
 {
-    // This test checks that the temporary file is deleted, even after a
-    // rename.
+    QFETCH(const bool, targetExists);
+    QFETCH(const bool, expectedRenameResult);
 
     QDir dir;
     QVERIFY(!dir.exists("temporary-file.txt"));
@@ -569,13 +602,35 @@ void tst_QTemporaryFile::rename()
         QTemporaryFile file(dir.filePath("temporary-file.XXXXXX"));
 
         QVERIFY(file.open());
+        QCOMPARE(file.write("I am the tmpfile"), 16);
         tempname = file.fileName();
         QVERIFY(dir.exists(tempname));
 
-        QVERIFY(file.rename("temporary-file.txt"));
-        QVERIFY(!dir.exists(tempname));
+        std::optional<QFile> renameBlocker;
+        const auto cleanup = qScopeGuard([&] {
+                if (renameBlocker)
+                    renameBlocker->remove();
+            });
+
+        if (targetExists) {
+            renameBlocker.emplace(dir.filePath("temporary-file.txt"_L1));
+            QVERIFY2(renameBlocker->open(QFile::ReadWrite),
+                     qPrintable(renameBlocker->errorString()));
+            QCOMPARE(renameBlocker->write("I am the blocker"), 16);
+            renameBlocker->close();
+            QVERIFY(QFile::exists(dir.filePath("temporary-file.txt"_L1)));
+        }
+
+#ifdef Q_OS_ANDROID
+        QEXPECT_FAIL("failure", "QTBUG-138610", Abort);
+#endif
+        QCOMPARE(file.rename("temporary-file.txt"_L1), expectedRenameResult);
+        QCOMPARE(dir.exists(tempname), !expectedRenameResult);
         QVERIFY(dir.exists("temporary-file.txt"));
-        QCOMPARE(file.fileName(), QString("temporary-file.txt"));
+        QCOMPARE(read_all(dir.filePath("temporary-file.txt"_L1)),
+                 expectedRenameResult ? "I am the tmpfile" : "I am the blocker");
+        QCOMPARE(file.fileName(),
+                 expectedRenameResult ? QString("temporary-file.txt") : tempname);
     }
 
     QVERIFY(!dir.exists(tempname));
@@ -584,8 +639,14 @@ void tst_QTemporaryFile::rename()
 
 void tst_QTemporaryFile::renameFdLeak()
 {
+#if defined(Q_OS_VXWORKS)
+    QSKIP("QTBUG-130066");
+#endif
+
 #if defined(Q_OS_UNIX) && !defined(Q_OS_ANDROID)
-    const QByteArray sourceFile = QFile::encodeName(QFINDTESTDATA("CMakeLists.txt"));
+    QTemporaryFile file;
+    QVERIFY(file.open());
+    const QByteArray sourceFile = QFile::encodeName(file.fileName());
     QVERIFY(!sourceFile.isEmpty());
     // Test this on Unix only
 
@@ -621,6 +682,55 @@ void tst_QTemporaryFile::renameFdLeak()
 #endif
 }
 
+void tst_QTemporaryFile::moveToTrash()
+{
+#if defined(Q_OS_ANDROID) || defined(Q_OS_WEBOS) || defined(Q_OS_VXWORKS)
+    QSKIP("This platform doesn't implement a trash bin");
+#endif
+#ifdef Q_OS_WIN
+    // QTemporaryFile won't really close the file with close(), so this is
+    // expected to fail with a sharing violation error.
+    constexpr bool expectSuccess = false;
+#else
+    constexpr bool expectSuccess = true;
+#endif
+    const QByteArrayView contents = "Hello, World\n";
+
+    QTemporaryFile f(QDir::homePath() + "/tst_qtemporaryfile.moveToTrash.XXXXXX");
+    QString origFileName;
+    auto cleanup = qScopeGuard([&] {
+        if (!origFileName.isEmpty())
+            QFile::remove(origFileName);
+        if (QString fn = f.fileName(); !fn.isEmpty() && fn != origFileName)
+            QFile::remove(fn);
+    });
+
+    if (!f.open())
+        QSKIP("Failed to create temporary file");
+    f.write(contents.data(), contents.size());
+
+    // we need an actual file name:
+    // 1) so we can delete it in the clean-up guard in case we fail to trash
+    // 2) so that the file exists on Linux in the first place (no sense in
+    //    trashing an unnamed file)
+    origFileName = f.fileName();
+
+    if (expectSuccess) {
+        QVERIFY2(f.moveToTrash(), qPrintable(f.errorString()));
+        QCOMPARE_NE(f.fileName(), origFileName);        // must have changed!
+        QCOMPARE_NE(f.fileName(), QString());
+        QVERIFY(!QFile::exists(origFileName));
+        QVERIFY(QFile::exists(f.fileName()));
+        QCOMPARE(QFileInfo(f.fileName()).size(), contents.size());
+    } else {
+        QVERIFY(!f.moveToTrash());
+        QCOMPARE(f.fileName(), origFileName);           // mustn't have changed!
+        QCOMPARE_NE(f.error(), QFile::NoError);
+        QCOMPARE_NE(f.errorString(), "Unknown error");
+        QVERIFY(QFile::exists(origFileName));
+    }
+}
+
 void tst_QTemporaryFile::reOpenThroughQFile()
 {
     QByteArray data("abcdefghij");
@@ -669,6 +779,9 @@ void tst_QTemporaryFile::keepOpenMode()
 
 void tst_QTemporaryFile::resetTemplateAfterError()
 {
+#if defined(Q_OS_VXWORKS)
+    QSKIP("QTBUG-130066");
+#endif
     // calling setFileTemplate on a failed open
 
     QString tempPath = QDir::tempPath();
@@ -723,6 +836,9 @@ void tst_QTemporaryFile::resetTemplateAfterError()
 
 void tst_QTemporaryFile::setTemplateAfterOpen()
 {
+#if defined(Q_OS_VXWORKS)
+    QSKIP("QTBUG-130066");
+#endif
     QTemporaryFile temp;
 
     QVERIFY( temp.fileName().isEmpty() );
@@ -749,37 +865,29 @@ void tst_QTemporaryFile::setTemplateAfterOpen()
 
 void tst_QTemporaryFile::autoRemoveAfterFailedRename()
 {
-    struct CleanOnReturn
-    {
-        ~CleanOnReturn()
-        {
+#if defined(Q_OS_VXWORKS)
+    QSKIP("QTBUG-130066");
+#endif
+
+    QString tempName;
+    auto cleaner = qScopeGuard([&] {
             if (!tempName.isEmpty())
                 QFile::remove(tempName);
-        }
-
-        void reset()
-        {
-            tempName.clear();
-        }
-
-        QString tempName;
-    };
-
-    CleanOnReturn cleaner;
+        });
 
     {
         QTemporaryFile file;
         QVERIFY( file.open() );
-        cleaner.tempName = file.fileName();
+        tempName = file.fileName();
 
-        QVERIFY( QFile::exists(cleaner.tempName) );
+        QVERIFY(QFile::exists(tempName));
         QVERIFY( !QFileInfo("i-do-not-exist").isDir() );
         QVERIFY( !file.rename("i-do-not-exist/file.txt") );
-        QVERIFY( QFile::exists(cleaner.tempName) );
+        QVERIFY(QFile::exists(tempName));
     }
 
-    QVERIFY( !QFile::exists(cleaner.tempName) );
-    cleaner.reset();
+    QVERIFY(!QFile::exists(tempName));
+    cleaner.dismiss(); // would fail: file is known to no longer exist
 }
 
 void tst_QTemporaryFile::createNativeFile_data()
@@ -814,7 +922,7 @@ void tst_QTemporaryFile::createNativeFile()
 
     QFile f(filePath);
     if (currentPos != -1) {
-        f.open(QIODevice::ReadOnly);
+        QVERIFY(f.open(QIODevice::ReadOnly));
         f.seek(currentPos);
     }
     QTemporaryFile *tempFile = QTemporaryFile::createNativeFile(f);
@@ -857,7 +965,7 @@ void tst_QTemporaryFile::QTBUG_4796()
     {
         ~CleanOnReturn()
         {
-            Q_FOREACH(QString tempName, tempNames)
+            for (const QString &tempName : std::as_const(tempNames))
                 QFile::remove(tempName);
         }
 
@@ -945,7 +1053,7 @@ void tst_QTemporaryFile::QTBUG_4796()
         }
     }
 
-    Q_FOREACH(QString const &tempName, cleaner.tempNames)
+    for (const QString &tempName : std::as_const(cleaner.tempNames))
         QVERIFY( !QFile::exists(tempName) );
 
     cleaner.reset();
@@ -953,13 +1061,16 @@ void tst_QTemporaryFile::QTBUG_4796()
 
 void tst_QTemporaryFile::guaranteeUnique()
 {
+#if defined(Q_OS_VXWORKS)
+    QSKIP("QTBUG-130066");
+#endif
     QDir dir(QDir::tempPath());
     QString takenFileName;
 
     // First pass. See which filename QTemporaryFile will try first.
     {
         QTemporaryFile tmpFile("testFile1.XXXXXX");
-        tmpFile.open();
+        QVERIFY(tmpFile.open());
         takenFileName = tmpFile.fileName();
         QVERIFY(QFile::exists(takenFileName));
     }
@@ -981,5 +1092,49 @@ void tst_QTemporaryFile::guaranteeUnique()
     QVERIFY(dir.rmdir(takenFileName));
 }
 
+void tst_QTemporaryFile::stdfilesystem()
+{
+#if !QT_CONFIG(cxx17_filesystem)
+    QSKIP("std::filesystem not available");
+#else
+    // ctor
+    {
+        std::filesystem::path testFile("testFile1.XXXXXX");
+        QTemporaryFile file(testFile);
+        QCOMPARE(file.fileTemplate(), QtPrivate::fromFilesystemPath(testFile));
+    }
+    // rename
+    {
+        QTemporaryFile file("testFile1.XXXXXX");
+        QVERIFY(file.open());
+        QByteArray payload = "abc123 I am a string";
+        file.write(payload);
+        QVERIFY(file.rename(std::filesystem::path("./test")));
+        file.close();
+
+        QFile f(u"./test"_s);
+        QVERIFY(f.exists());
+        QVERIFY(f.open(QFile::ReadOnly));
+        QCOMPARE(f.readAll(), payload);
+    }
+    // createNativeFile
+    {
+        std::filesystem::path resource(":/resources/test.txt");
+        std::unique_ptr<QTemporaryFile> tmp(QTemporaryFile::createNativeFile(resource));
+        QVERIFY(tmp);
+        QFile file(resource);
+        QVERIFY(file.open(QFile::ReadOnly));
+        QCOMPARE(tmp->readAll(), file.readAll());
+    }
+    // setFileTemplate
+    {
+        QTemporaryFile file;
+        std::filesystem::path testFile("testFile1.XXXXXX");
+        file.setFileTemplate(testFile);
+        QCOMPARE(file.fileTemplate(), QtPrivate::fromFilesystemPath(testFile));
+    }
+#endif
+}
+
 QTEST_MAIN(tst_QTemporaryFile)
 #include "tst_qtemporaryfile.moc"
