@@ -75,6 +75,7 @@ private slots:
     void loadLocale_data();
     void loadLocale();
     void threadLoad();
+    void install();
     void testLanguageChange();
     void plural();
     void translate_qm_file_generated_with_msgfmt();
@@ -186,6 +187,13 @@ void tst_QTranslator::loadLocale_data()
                             << QLocale::system()
                             << QStringList{"en-AU", "en-NZ", "de-DE", "en-GB"};
     QTest::addRow("System, Taiwan") << QLocale::system() << QStringList{"zh-TW", "zh"};
+
+    // This tests that, when "just the language" is not a faithful
+    // representation of the entry it's derived from, we try later entries
+    // before it.
+    QTest::addRow("System, multi-script language")
+                            << QLocale::system()
+                            << QStringList{"pa-Arab-GB", "pa-PK", "en-GB"};
 }
 
 void tst_QTranslator::loadLocale()
@@ -243,7 +251,7 @@ void tst_QTranslator::loadLocale()
     for (const auto &filePath : files)
         QVERIFY(QFile::exists(filePath));
 
-    const QRegularExpression localeExpr("foo-(.*)(\\.qm|$)");
+    const QRegularExpression localeExpr("foo-(.*)(\\.qm|)$");
     QTranslator tor;
     // Load the translation for the wanted locale
     QVERIFY(tor.load(wantedLocale, "foo", "-", path, ".qm"));
@@ -285,6 +293,30 @@ void tst_QTranslator::threadLoad()
     QVERIFY(thread.wait(10 * 1000));
 }
 
+void tst_QTranslator::install()
+{
+    {
+        // normal translation
+        QTranslator tor;
+        QVERIFY(tor.load("hellotr_la.qm"));
+        QCOMPARE(qApp->installTranslator(&tor), true);
+        QCOMPARE(qApp->removeTranslator(&tor), true);
+    }
+    {
+        // empty translation
+        QTranslator tor;
+        QVERIFY(tor.load("hellotr_empty.qm"));
+        QCOMPARE(qApp->installTranslator(&tor), true);
+        QCOMPARE(qApp->removeTranslator(&tor), true);
+    }
+    {
+        // nullptr
+        QCOMPARE(qApp->installTranslator(nullptr), false);
+        QCOMPARE(qApp->removeTranslator(nullptr), false);
+    }
+
+}
+
 void tst_QTranslator::testLanguageChange()
 {
     languageChangeEventCounter = 0;
