@@ -26,9 +26,13 @@ Q_DECLARE_JNI_CLASS(Rect, "android/graphics/Rect")
 Q_DECLARE_JNI_CLASS(View, "android/view/View")
 Q_DECLARE_JNI_CLASS(Window, "android/view/Window")
 Q_DECLARE_JNI_CLASS(WindowInsets, "android/view/WindowInsets")
+Q_DECLARE_JNI_CLASS(Insets, "android/view/Insets")
+Q_DECLARE_JNI_CLASS(GraphicsInsets, "android/graphics/Insets")
+Q_DECLARE_JNI_CLASS(DisplayCutout, "android/view/DisplayCutout")
 Q_DECLARE_JNI_CLASS(WindowManager, "android/view/WindowManager")
 Q_DECLARE_JNI_CLASS(WindowMetrics, "android/view/WindowMetrics")
 Q_DECLARE_JNI_CLASS(ApplicationInfo, "android/content/pm/ApplicationInfo")
+Q_DECLARE_JNI_CLASS(WindowInsetsType, "android/view/WindowInsets$Type")
 
 class tst_Android : public QObject
 {
@@ -41,8 +45,9 @@ private slots:
     void testAndroidActivity();
     void testRunOnAndroidMainThread();
 #if QT_CONFIG(widgets)
+    void safeAreaWithWindowFlagsAndStates_data();
+    void safeAreaWithWindowFlagsAndStates();
     void testFullScreenDimensions();
-    void orientationChange_data();
     void orientationChange();
 #endif
 };
@@ -219,9 +224,130 @@ void tst_Android::testRunOnAndroidMainThread()
 }
 
 #if QT_CONFIG(widgets)
+void tst_Android::safeAreaWithWindowFlagsAndStates_data()
+{
+    QTest::addColumn<Qt::WindowStates>("windowStates");
+    QTest::addColumn<Qt::WindowFlags>("windowFlags");
+
+    QTest::newRow("Normal")
+        << Qt::WindowStates(Qt::WindowNoState)
+        << Qt::WindowFlags();
+
+    QTest::newRow("Expanded Client Area")
+        << Qt::WindowStates(Qt::WindowNoState)
+        << Qt::WindowFlags(Qt::ExpandedClientAreaHint);
+
+    QTest::newRow("Fullscreen")
+        << Qt::WindowStates(Qt::WindowFullScreen)
+        << Qt::WindowFlags();
+
+    QTest::newRow("Fullscreen and Expanded Client Area")
+        << Qt::WindowStates(Qt::WindowFullScreen)
+        << Qt::WindowFlags(Qt::ExpandedClientAreaHint);
+}
+
+void tst_Android::safeAreaWithWindowFlagsAndStates()
+{
+    QFETCH(Qt::WindowStates, windowStates);
+    QFETCH(Qt::WindowFlags, windowFlags);
+
+    if ((QNativeInterface::QAndroidApplication::sdkVersion() > __ANDROID_API_V__) &&
+             qgetenv("QTEST_ENVIRONMENT").split(' ').contains("ci") &
+                (!(windowFlags & Qt::ExpandedClientAreaHint) &&
+                     !(windowStates & Qt::WindowFullScreen)))
+        QSKIP("Normal fails on Android 16 (QTBUG-140846).");
+
+    QWidget widget;
+    QPalette palette = widget.palette();
+    palette.setColor(QPalette::Window, Qt::red);
+    widget.setAutoFillBackground(true);
+    widget.setPalette(palette);
+    widget.setWindowFlags(windowFlags);
+
+    const bool fullscreen = windowStates & Qt::WindowFullScreen;
+    if (fullscreen)
+        widget.showFullScreen();
+    else
+        widget.show();
+
+    QVERIFY(QTest::qWaitForWindowExposed(&widget));
+
+    using namespace QtJniTypes;
+    const int sdkVersion = QNativeInterface::QAndroidApplication::sdkVersion();
+    auto activity = QNativeInterface::QAndroidApplication::context();
+
+    // Android 15 enables edge-to-edge by default.
+    bool edgeToEdge = sdkVersion >= __ANDROID_API_V__;
+
+    // Detect camera cutout
+    bool cameraCutout = false;
+    if (sdkVersion >= __ANDROID_API_R__) {
+        Window window = activity.callMethod<Window>("getWindow");
+        View decorView = window.callMethod<View>("getDecorView");
+        WindowInsets insets = decorView.callMethod<WindowInsets>("getRootWindowInsets");
+        if (insets.isValid()) {
+            DisplayCutout cutout = insets.callMethod<DisplayCutout>("getDisplayCutout");
+            if (cutout.isValid()) {
+                const int top = cutout.callMethod<jint>("getSafeInsetTop");
+                const int left = cutout.callMethod<jint>("getSafeInsetLeft");
+                const int right = cutout.callMethod<jint>("getSafeInsetRight");
+                const int bottom = cutout.callMethod<jint>("getSafeInsetBottom");
+                cameraCutout = (top > 0) || (left > 0) || (right > 0) || (bottom > 0);
+            }
+        }
+    } else {
+        // Android 9 and 10 cutout API support was buggy
+        cameraCutout = true;
+    }
+
+    const bool runsOnCI = qgetenv("QTEST_ENVIRONMENT").split(' ').contains("ci");
+    if (sdkVersion == __ANDROID_API_V__ && runsOnCI) {
+        // Furthermore, it's not reporting camera cutout margins.
+        cameraCutout = false;
+    }
+
+    const bool expandedClientArea = windowFlags & Qt::ExpandedClientAreaHint;
+    const bool normalMode = !expandedClientArea && !fullscreen;
+    if ((normalMode && !edgeToEdge) || (fullscreen && !cameraCutout)) {
+        QTRY_COMPARE(widget.windowHandle()->safeAreaMargins(), QMargins());
+    } else {
+        QTRY_COMPARE_NE(widget.windowHandle()->safeAreaMargins(), QMargins());
+
+        // Make sure the margins we get are the same as the system bars sizes,
+        // that way we make sure we don't end up with margins bigger than expected.
+        // So, retrieve the static system bars height.
+        Window window = activity.callMethod<Window>("getWindow");
+        View decorView = window.callMethod<View>("getDecorView");
+        WindowInsets insets = decorView.callMethod<WindowInsets>("getRootWindowInsets");
+        QVERIFY(insets.isValid());
+
+        // Other margins can vary between Android versions, so let's only check for top
+        int top = 0;
+        if (sdkVersion >= __ANDROID_API_R__) {
+            jint systemBarsType = WindowInsetsType::callStaticMethod<jint>("systemBars");
+            jint displayCutoutType = WindowInsetsType::callStaticMethod<jint>("displayCutout");
+            jint combinedType = systemBarsType | displayCutoutType;
+
+            GraphicsInsets insetsIgnoreVisibility = insets.callMethod<GraphicsInsets>(
+                "getInsetsIgnoringVisibility", combinedType);
+            QVERIFY(insetsIgnoreVisibility.isValid());
+            top = insetsIgnoreVisibility.getField<jint>("top");
+        } else {
+            top    = insets.callMethod<jint>("getStableInsetTop");
+        }
+
+        qreal dpr = widget.devicePixelRatio();
+        QCOMPARE_LE(widget.windowHandle()->safeAreaMargins().top(), qRound(top / dpr));
+    }
+}
+
 // QTBUG-107604
 void tst_Android::testFullScreenDimensions()
 {
+    if ((QNativeInterface::QAndroidApplication::sdkVersion() > __ANDROID_API_V__) &&
+            qgetenv("QTEST_ENVIRONMENT").split(' ').contains("ci") )
+        QSKIP("Keep on failing on Android 16 (QTBUG-141712).");
+
     QJniObject activity = QNativeInterface::QAndroidApplication::context();
     QVERIFY(activity.isValid());
 
@@ -250,6 +376,10 @@ void tst_Android::testFullScreenDimensions()
     display.callMethod<void>("getRealSize", realSize);
 
     QWidget widget;
+    QPalette palette = widget.palette();
+    palette.setColor(QPalette::Window, Qt::red);
+    widget.setAutoFillBackground(true);
+    widget.setPalette(palette);
     QPlatformScreen *screen = QGuiApplication::primaryScreen()->handle();
     {
         // Normal -
@@ -337,74 +467,87 @@ void tst_Android::testFullScreenDimensions()
     }
 }
 
-void tst_Android::orientationChange_data()
-{
-    QTest::addColumn<int>("nativeOrientation");
-    QTest::addColumn<Qt::ScreenOrientation>("expected");
-    QTest::addColumn<QSize>("screenSize");
-
-    const QSize portraitSize = QGuiApplication::primaryScreen()->size();
-    const QSize landscapeSize = QSize(portraitSize.height(), portraitSize.width());
-
-    // Rotations without 180 degree or inverted portrait, assuming that the device is in portrait
-    // position. These are ok for Android 6(API 23), 8 (API 27) and 14 (API 34)
-    QTest::newRow("InvertedLandscape") << 8 << Qt::InvertedLandscapeOrientation << landscapeSize;
-    QTest::newRow("Portrait") << 1 << Qt::PortraitOrientation << portraitSize;
-    QTest::newRow("Landscape") << 0 << Qt::LandscapeOrientation << landscapeSize;
-    QTest::newRow("Portrait2") << 1 << Qt::PortraitOrientation << portraitSize;
-
-    // Rotations over inverted portrait doing only 90 degree turns.
-    QTest::newRow("InvertedLandscape2") << 8 << Qt::InvertedLandscapeOrientation << landscapeSize;
-    QTest::newRow("InvertedPortrait") << 9 << Qt::InvertedPortraitOrientation << portraitSize;
-    QTest::newRow("Landscape2") << 0 << Qt::LandscapeOrientation << landscapeSize;
-    QTest::newRow("InvertedPortrait2") << 9 << Qt::InvertedPortraitOrientation << portraitSize;
-    QTest::newRow("InvertedLandscape3") << 8 << Qt::InvertedLandscapeOrientation << landscapeSize;
-
-    // Rotations with 180 degree turns.
-    // Android 6 (API23) Does not understand these transitions.
-    if (QNativeInterface::QAndroidApplication::sdkVersion() > __ANDROID_API_M__) {
-        QTest::newRow("Landscape3") << 0 << Qt::LandscapeOrientation << landscapeSize;
-        QTest::newRow("InvertedLandscape4")
-                << 8 << Qt::InvertedLandscapeOrientation << landscapeSize;
-        QTest::newRow("Portrait3") << 1 << Qt::PortraitOrientation << portraitSize;
-    } else {
-        qWarning() << "180 degree turn rotation test cases are not run on Android 6 (API 23) and "
-                      "below.";
-    }
-    // Android 8 (API 27) does not understand portrait-'inverted portrait'-portrait transition.
-    if (QNativeInterface::QAndroidApplication::sdkVersion() > __ANDROID_API_O_MR1__) {
-        QTest::newRow("InvertedPortrait3") << 9 << Qt::InvertedPortraitOrientation << portraitSize;
-        QTest::newRow("Portrait4") << 1 << Qt::PortraitOrientation << portraitSize;
-    } else {
-        qWarning() << "Portrait-'Inverted portrait'-Portrait rotation test cases are not run on "
-                      "Android 8 (API 27) and below.";
-    }
-}
-
 void tst_Android::orientationChange()
 {
-    QFETCH(int, nativeOrientation);
-    QFETCH(Qt::ScreenOrientation, expected);
-    QFETCH(QSize, screenSize);
-
     if (QNativeInterface::QAndroidApplication::sdkVersion() == __ANDROID_API_P__)
         QSKIP("Android 9 orientation changes callbacks are buggy (QTBUG-124890).");
 
-    // For QTBUG-94459 to check that the widget size are consistent after orientation changes
     QWidget widget;
+    QPalette palette = widget.palette();
+    palette.setColor(QPalette::Window, Qt::red);
+    widget.setAutoFillBackground(true);
+    widget.setPalette(palette);
     widget.show();
 
     QScreen *screen = QGuiApplication::primaryScreen();
-    QSignalSpy orientationSpy(screen, SIGNAL(orientationChanged(Qt::ScreenOrientation)));
+    QSignalSpy orientationSpy(screen, &QScreen::orientationChanged);
 
     auto context = QNativeInterface::QAndroidApplication::context();
-    context.callMethod<void>("setRequestedOrientation", nativeOrientation);
 
-    orientationSpy.wait();
-    QTRY_COMPARE(screen->orientation(), expected);
-    QCOMPARE(orientationSpy.size(), 1);
-    QCOMPARE(screen->size(), screenSize);
-    QCOMPARE(widget.size(), screen->availableSize());
+    enum NativeOrientation {
+        Landscape = 0,
+        Portrait = 1,
+        InvertedLandscape = 8,
+        InvertedPortrait = 9
+    };
+
+    auto nativeOrientation = [](Qt::ScreenOrientation orientation) {
+        switch (orientation) {
+        case(Qt::LandscapeOrientation):
+            return Landscape;
+        case(Qt::PortraitOrientation):
+            return Portrait;
+        case(Qt::InvertedLandscapeOrientation):
+            return InvertedLandscape;
+        case(Qt::InvertedPortraitOrientation):
+            return InvertedPortrait;
+        default:
+            return Portrait;
+        }
+    };
+
+    auto requestOrientation = [nativeOrientation, context](Qt::ScreenOrientation expected) {
+        context.callMethod("setRequestedOrientation", nativeOrientation(expected));
+    };
+
+    auto restoreOrientation = qScopeGuard([&] {
+        requestOrientation(Qt::PortraitOrientation);
+        orientationSpy.wait();
+        QTRY_COMPARE(screen->orientation(), Qt::PortraitOrientation);
+    });
+
+    auto testOrientation = [&](Qt::ScreenOrientation expected, const QSize &screenSize) {
+        requestOrientation(expected);
+        orientationSpy.wait();
+        QTRY_COMPARE(screen->orientation(), expected);
+        QCOMPARE(orientationSpy.size(), 1);
+        // For QTBUG-94459 to verify widget size consistency after orientation changes.
+        // In general we can't guarantee the order though, since Android might send the
+        // orientation and size change at any order, so we need to use QTRY_COMPARE().
+        QTRY_COMPARE(screen->size(), screenSize);
+        QTRY_COMPARE(widget.size(), screen->availableSize());
+        orientationSpy.clear();
+    };
+
+    const QSize portraitSize = screen->size();
+    const QSize landscapeSize = QSize(portraitSize.height(), portraitSize.width());
+
+    // Sequential 90 degrees clock-wise rotations
+    testOrientation(Qt::InvertedLandscapeOrientation, landscapeSize);
+    testOrientation(Qt::InvertedPortraitOrientation, portraitSize);
+    testOrientation(Qt::LandscapeOrientation, landscapeSize);
+    testOrientation(Qt::PortraitOrientation, portraitSize);
+
+    // Sequential 90 degrees counter-clockwise rotations
+    testOrientation(Qt::LandscapeOrientation, landscapeSize);
+    testOrientation(Qt::InvertedPortraitOrientation, portraitSize);
+    testOrientation(Qt::InvertedLandscapeOrientation, landscapeSize);
+
+    // 180 degree rotations
+    testOrientation(Qt::InvertedPortraitOrientation, portraitSize);
+    testOrientation(Qt::PortraitOrientation, portraitSize);
+    testOrientation(Qt::InvertedLandscapeOrientation, landscapeSize);
+    testOrientation(Qt::LandscapeOrientation, landscapeSize);
 }
 #endif // QT_CONFIG(widgets)
 
