@@ -1,8 +1,6 @@
 // Copyright (C) 2017 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
-#undef QT_NO_FOREACH // this file contains unported legacy Q_FOREACH uses
-
 #include <QTest>
 
 #include <QtCore/qcoreapplication.h>
@@ -69,7 +67,7 @@ void tst_QKeyEvent::basicEventDelivery()
     QCOMPARE(window.keyEvents.size(), 2);
     QCOMPARE(window.keyEvents.first()->type(), QKeyEvent::KeyPress);
     QCOMPARE(window.keyEvents.last()->type(), QKeyEvent::KeyRelease);
-    foreach (const QKeyEvent *event, window.keyEvents) {
+    for (const QKeyEvent *event : std::as_const(window.keyEvents)) {
         QCOMPARE(Qt::Key(event->key()), key);
         QCOMPARE(Qt::KeyboardModifiers(event->modifiers()), modifiers);
     }
@@ -139,7 +137,7 @@ void tst_QKeyEvent::modifiers_data()
     std::sort(modifierCombinations.begin(), modifierCombinations.end(), orderByModifier);
 
     QTest::addColumn<Qt::KeyboardModifiers>("modifiers");
-    foreach (const QList<int> combination, modifierCombinations) {
+    for (const QList<int> &combination : std::as_const(modifierCombinations)) {
         int keys[4] = {};
         Qt::KeyboardModifiers mods;
         for (int i = 0; i < combination.size(); ++i) {
