@@ -1,5 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
 #include <QTest>
@@ -34,6 +34,8 @@ private slots:
     void projectivePathMapping();
     void mapInt();
     void mapPathWithPoint();
+    void mapRectToPolygon(); // QTBUG-127723
+    void quadToQuad(); // QTBUG-21329
 
 private:
     void mapping_data();
@@ -700,6 +702,27 @@ void tst_QTransform::mapPathWithPoint()
     QCOMPARE(p.currentPosition(), QPointF(20, 20));
 }
 
+void tst_QTransform::mapRectToPolygon()
+{
+    QRectF r(7, 7, 36, 36);
+    QTransform tx(2, 0, 0, 2, 0, 0);
+    QPolygonF polygon1 = tx.mapToPolygon(r.toRect()).toPolygonF();
+    QPolygonF polygon2 = tx.map(QPolygonF(r));
+    if (polygon1.size() > 4)
+        polygon1.removeLast();
+    if (polygon2.size() > 4)
+        polygon2.removeLast();
+    QCOMPARE(polygon1, polygon2);
+}
+
+void tst_QTransform::quadToQuad() // QTBUG-21329
+{
+    QTransform result;
+    QVERIFY(QTransform::quadToQuad(QRectF(0, 0, 1, 1), QRectF(0, 0, 1, 1), result));
+    QPolygonF trapezoid({{0, 0}, {10, 0}, {11, 11}, {0, 10}});
+    QVERIFY(QTransform::quadToQuad(trapezoid, trapezoid.boundingRect(), result));
+}
+
 QTEST_APPLESS_MAIN(tst_QTransform)
 
 
