@@ -16,17 +16,19 @@ QT_REQUIRE_CONFIG(process);
 #include <QtCore/QFileInfo>
 #include <QtCore/QDir>
 #include <QtCore/QTemporaryDir>
+#include <QtCore/QProcess>
 
 #include <QTest>
 
-#include <QProcess>
-
-#include <private/cycle_p.h>
+#include <regex>
+#include <private/cycle_include_p.h>
 
 #include <QtTest/private/qemulationdetector_p.h>
 
 using namespace Qt::StringLiterals;
 
+enum class Throw { OnFail = 1 };
+
 struct BenchmarkResult
 {
     qint64  total;
@@ -86,7 +88,8 @@ static bool compareBenchmarkResult(BenchmarkResult const &r1, BenchmarkResult co
 
 // Split the passed block of text into an array of lines, replacing any
 // filenames and line numbers with generic markers to avoid failing the test
-// due to compiler-specific behaviour.
+// due to compiler-specific behavior. For some known differences in output,
+// it normalizes the stored text.
 static QList<QByteArray> splitLines(QByteArray ba)
 {
     ba.replace('\r', "");
@@ -112,6 +115,32 @@ static QList<QByteArray> splitLines(QByteArray ba)
             }
             line.replace(index, end-index + 1, markers[j][1]);
         }
+
+        // There's some difference on how floating point numbers are printed by
+        // the various snprintf(), both in decimal and in hexadecimal form. The
+        // following regex catches them so we can normalize the output to the
+        // current running libc.
+        //
+        // Examples (most but not all attested):
+        //  1    0x1p+0     0x8p-3
+        //  1.00000000000000000   0x1.0000000000000p+0
+        //  1.5  0x1.8p+0   0xcp-3
+        //  1e6  1e+06      1e+6
+        //  1e-7 1e-07      1e-007
+        //  -1.797693134862316e+308 -0xf.ffffffffffff8p+1020 -0x1.fffffffffffffp+1023
+        static std::regex fpValueRx(R"(-?\d+(?:\.\d*)?(?:e[-+]\d\d+)? \((-?0x[\da-f.]+p[-+]?\d+)\))");
+        if (std::cmatch match; std::regex_search(line.cbegin(), line.cend(), match, fpValueRx)) {
+            if (double value; sscanf(match[1].first, "%la", &value) == 1) {
+                // remove the decimal and hexadecimal string representations
+                line.truncate(match[0].first - line.cbegin());
+
+                // append normalized hexfloat
+                // this won't fail and the buffer is definitely big enough
+                char buf[128];
+                int n = snprintf(buf, sizeof(buf), "%a", value);
+                line += std::string_view(buf, n);
+            }
+        }
     }
 
     return out;
@@ -638,11 +667,6 @@ bool TestLogger::shouldIgnoreTest(const QString &test) const
         return true;
 #endif
 
-    if (!qEnvironmentVariableIsEmpty("WAYLAND_DISPLAY")) {
-        qDebug() << "TestLogger::shouldIgnoreTest() ignore" << test << "on wayland/xwayland!";
-        return true;
-    }
-
     // These tests are affected by timing and whether the CPU tick counter
     // is monotonically increasing. They won't work on some machines so
     // leave them off by default. Feel free to enable them for your own
@@ -673,6 +697,12 @@ bool TestLogger::shouldIgnoreTest(const QString &test) const
         return true;
 #endif
 
+#if defined(QT_NO_EXCEPTIONS) || !QT_CONFIG(future) || defined(QT_NO_CONCURRENT)
+    // This test requires exceptions, QException, and QtConcurrent::run():
+    if (test == "throwonfailandskip")
+        return true;
+#endif
+
     if (test == "benchlibcallgrind") {
 #if defined(__GNUC__) && (defined(__i386) || defined(__x86_64)) && defined(Q_OS_LINUX)
 #  ifdef __AVX512F__
@@ -693,6 +723,13 @@ bool TestLogger::shouldIgnoreTest(const QString &test) const
 #endif
     }
 
+#ifndef __cpp_lib_three_way_comparison
+    if (test == "threewaycompare") {
+        WARN("The threewaycompare test requires C++20 support. Skipping.");
+        return true;
+    }
+#endif
+
     if (logger != QTestLog::Plain || outputMode == FileOutput) {
         // The following tests only work with plain text output to stdout,
         // either because they execute multiple test objects or because
@@ -1025,12 +1062,10 @@ TestProcessResult runTestProcess(const QString &test, const QStringList &argumen
     return { process.exitCode(), standardOutput, standardError };
 }
 
-enum class Throw { OnFail = 1 };
-
 /*
     Runs a single test and verifies the output against the expected results.
 */
-void runTest(const QString &test, const TestLoggers &requestedLoggers, Throw throwing = {})
+void runTest(const QString &test, const TestLoggers &requestedLoggers)
 {
     TestLoggers loggers;
     for (auto logger : requestedLoggers) {
@@ -1044,10 +1079,6 @@ void runTest(const QString &test, const TestLoggers &requestedLoggers, Throw thr
     QStringList arguments;
     for (auto logger : loggers)
         arguments += logger.arguments(test);
-    if (throwing == Throw::OnFail) // don't distinguish between throwonfail/throwonskip
-        arguments += {"-throwonfail", "-throwonskip"};
-    else
-        arguments += {"-nothrowonfail", "-nothrowonskip"};
 
     CAPTURE(test);
     CAPTURE(arguments);
@@ -1074,9 +1105,9 @@ void runTest(const QString &test, const TestLoggers &requestedLoggers, Throw thr
 /*
     Runs a single test and verifies the output against the expected result.
 */
-void runTest(const QString &test, const TestLogger &logger, Throw t = {})
+void runTest(const QString &test, const TestLogger &logger)
 {
-    runTest(test, TestLoggers{logger}, t);
+    runTest(test, TestLoggers{logger});
 }
 
 // ----------------------- Catch helpers -----------------------
@@ -1219,12 +1250,7 @@ SCENARIO("Test output of the loggers is as expected")
     GIVEN("The " << logger << " logger") {
         for (QString test : tests) {
             AND_GIVEN("The " << test << " subtest") {
-                WHEN("Throwing on failure or skip") {
-                    runTest(test, TestLogger(logger, StdoutOutput), Throw::OnFail);
-                }
-                WHEN("Returning on failure or skip") {
-                    runTest(test, TestLogger(logger, StdoutOutput));
-                }
+                runTest(test, TestLogger(logger, StdoutOutput));
             }
         }
     }
