@@ -1,5 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QTest>
 #include <QtWidgets/qgraphicsanchorlayout.h>
@@ -304,7 +304,7 @@ void tst_QGraphicsAnchorLayout::layoutDirection()
     view->show();
 
     QVERIFY(p->layout());
-    QCOMPARE(checkReverseDirection(p), true);
+    QTRY_COMPARE(checkReverseDirection(p), true);
 
     if (hasSimplification) {
         QVERIFY(usedSimplex(l, Qt::Horizontal));
