@@ -1,28 +1,32 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
-#include <QTest>
-#include <QSignalSpy>
-#include <QMdiSubWindow>
-#include <QMdiArea>
+#include <QtTest/qtest.h>
+#include <QtTest/qsignalspy.h>
 
-#include <QApplication>
-#include <QMainWindow>
-#include <QMenuBar>
-#include <QPushButton>
-#include <QStyle>
-#include <QStyleOption>
-#include <QVBoxLayout>
-#include <QLineEdit>
-#include <QDockWidget>
-#include <QScrollBar>
-#include <QTextEdit>
+#include <QtWidgets/qapplication.h>
+#include <QtWidgets/qboxlayout.h>
+#include <QtWidgets/qdockwidget.h>
+#include <QtWidgets/qlineedit.h>
+#include <QtWidgets/qmainwindow.h>
+#include <QtWidgets/qmdiarea.h>
+#include <QtWidgets/qmdisubwindow.h>
+#include <QtWidgets/qmenubar.h>
+#include <QtWidgets/qpushbutton.h>
+#include <QtWidgets/qscrollbar.h>
+#include <QtWidgets/qstyle.h>
+#include <QtWidgets/qstyleoption.h>
+#include <QtWidgets/qtextedit.h>
+
+#include <QtGui/qstylehints.h>
 #ifndef QT_NO_OPENGL
-#include <QtOpenGL>
-#include <QOpenGLContext>
+#  include <QtGui/qopenglcontext.h>
 #endif
-#include <QStyleHints>
+
+#include <QtWidgets/private/qapplication_p.h>
+
+using namespace Qt::StringLiterals;
 
 static const Qt::WindowFlags DefaultWindowFlags
     = Qt::SubWindow | Qt::WindowSystemMenuHint
@@ -174,7 +178,7 @@ static bool verifyArrangement(QMdiArea *mdiArea, Arrangement arrangement, const
         // QWidget::childAt with the position of the first one and subsequently adding
         // dx and dy.
         QPoint subWindowPos(20, 5);
-        foreach (int expectedIndex, expectedIndices) {
+        for (int expectedIndex : expectedIndices) {
             QMdiSubWindow *expected = subWindows.at(expectedIndex);
             expected->raise();
             if (mdiArea->viewport()->childAt(subWindowPos) != expected)
@@ -185,7 +189,7 @@ static bool verifyArrangement(QMdiArea *mdiArea, Arrangement arrangement, const
         }
 
         // Restore stacking order.
-        foreach (QMdiSubWindow *subWindow, activationOrderList) {
+        for (QMdiSubWindow *subWindow : activationOrderList) {
             mdiArea->setActiveSubWindow(subWindow);
             qApp->processEvents();
         }
@@ -258,9 +262,11 @@ private slots:
     void task_236750();
     void qtbug92240_title_data();
     void qtbug92240_title();
+    void tabbedview_singleSubWindow();
     void tabbedview_activefirst();
     void tabbedview_activesecond();
     void tabbedview_activethird();
+    void tabbedview_closeInactive();
 
 private:
     QMdiSubWindow *activeWindow;
@@ -304,7 +310,7 @@ void tst_QMdiArea::subWindowActivated()
     QSignalSpy spy(workspace, SIGNAL(subWindowActivated(QMdiSubWindow*)));
     connect( workspace, SIGNAL(subWindowActivated(QMdiSubWindow*)), this, SLOT(activeChanged(QMdiSubWindow*)));
     mw.show();
-    qApp->setActiveWindow(&mw);
+    QApplicationPrivate::setActiveWindow(&mw);
 
     QFETCH( int, count );
     int i;
@@ -410,18 +416,6 @@ void tst_QMdiArea::subWindowActivated()
     }
 }
 
-#ifdef Q_OS_MAC
-#include <Security/AuthSession.h>
-bool macHasAccessToWindowsServer()
-{
-    SecuritySessionId mySession;
-    SessionAttributeBits sessionInfo;
-    SessionGetInfo(callerSecuritySession, &mySession, &sessionInfo);
-    return (sessionInfo & sessionHasGraphicAccess);
-}
-#endif
-
-
 void tst_QMdiArea::subWindowActivated2()
 {
     if (QGuiApplication::platformName().startsWith(QLatin1String("wayland"), Qt::CaseInsensitive))
@@ -478,10 +472,6 @@ void tst_QMdiArea::subWindowActivated2()
     // Check that we only emit _one_ signal and the active window
     // is unchanged after showMinimized/showNormal.
     mdiArea.showMinimized();
-#if defined (Q_OS_MAC)
-    if (!macHasAccessToWindowsServer())
-        QEXPECT_FAIL("", "showMinimized doesn't really minimize if you don't have access to the server", Abort);
-#endif
 #ifdef Q_OS_MAC
     QSKIP("QTBUG-25298: This test is unstable on Mac.");
 #endif
@@ -512,7 +502,6 @@ void tst_QMdiArea::subWindowActivatedWithMinimize()
     QSignalSpy spy(workspace, SIGNAL(subWindowActivated(QMdiSubWindow*)));
     connect( workspace, SIGNAL(subWindowActivated(QMdiSubWindow*)), this, SLOT(activeChanged(QMdiSubWindow*)) );
     mw.show();
-    qApp->setActiveWindow(&mw);
     QWidget *widget = new QWidget(workspace);
     widget->setAttribute(Qt::WA_DeleteOnClose);
     QMdiSubWindow *window1 = workspace->addSubWindow(widget);
@@ -657,7 +646,6 @@ void tst_QMdiArea::changeWindowTitle()
 #endif
 
     mw->show();
-    qApp->setActiveWindow(mw);
 
 #ifdef USE_SHOW
     mw->showFullScreen();
@@ -902,7 +890,7 @@ void tst_QMdiArea::setActiveSubWindow()
 
     QSignalSpy spy(&workspace, SIGNAL(subWindowActivated(QMdiSubWindow*)));
     connect(&workspace, SIGNAL(subWindowActivated(QMdiSubWindow*)), this, SLOT(activeChanged(QMdiSubWindow*)));
-    qApp->setActiveWindow(&workspace);
+    QApplicationPrivate::setActiveWindow(&workspace);
 
     // Activate hidden windows
     const int windowCount = 10;
@@ -961,7 +949,6 @@ void tst_QMdiArea::activeSubWindow()
     mainWindow.addDockWidget(Qt::LeftDockWidgetArea, dockWidget);
 
     mainWindow.show();
-    qApp->setActiveWindow(&mainWindow);
     QVERIFY(QTest::qWaitForWindowActive(&mainWindow));
     QCOMPARE(mdiArea->activeSubWindow(), subWindow);
     QCOMPARE(qApp->focusWidget(), (QWidget *)subWindowLineEdit);
@@ -984,15 +971,13 @@ void tst_QMdiArea::activeSubWindow()
     dummyTopLevel.show();
     QVERIFY(QTest::qWaitForWindowExposed(&dummyTopLevel));
 
-    qApp->setActiveWindow(&dummyTopLevel);
     QCOMPARE(mdiArea->activeSubWindow(), subWindow);
 
-    qApp->setActiveWindow(&mainWindow);
     QCOMPARE(mdiArea->activeSubWindow(), subWindow);
 
     //task 202657
     dockWidgetLineEdit->setFocus();
-    qApp->setActiveWindow(&mainWindow);
+    QApplicationPrivate::setActiveWindow(&mainWindow);
     QVERIFY(dockWidgetLineEdit->hasFocus());
 }
 
@@ -1005,7 +990,7 @@ void tst_QMdiArea::currentSubWindow()
     for (int i = 0; i < 5; ++i)
         mdiArea.addSubWindow(new QLineEdit)->show();
 
-    qApp->setActiveWindow(&mdiArea);
+    QApplicationPrivate::setActiveWindow(&mdiArea);
     QCOMPARE(qApp->activeWindow(), (QWidget *)&mdiArea);
 
     // Check that the last added window is the active and the current.
@@ -1017,10 +1002,10 @@ void tst_QMdiArea::currentSubWindow()
     QLineEdit dummyTopLevel;
     dummyTopLevel.show();
     QVERIFY(QTest::qWaitForWindowExposed(&dummyTopLevel));
+    QVERIFY(QTest::qWaitForWindowActive(&dummyTopLevel));
 
     // Move focus to another top-level and check that we still
     // have an active window.
-    qApp->setActiveWindow(&dummyTopLevel);
     QCOMPARE(qApp->activeWindow(), (QWidget *)&dummyTopLevel);
     QVERIFY(mdiArea.activeSubWindow());
 
@@ -1033,7 +1018,7 @@ void tst_QMdiArea::currentSubWindow()
     QCOMPARE(mdiArea.currentSubWindow(), mdiArea.subWindowList().front());
 
     // Activate mdi area and check that active == current.
-    qApp->setActiveWindow(&mdiArea);
+    QApplicationPrivate::setActiveWindow(&mdiArea);
     active = mdiArea.activeSubWindow();
     QVERIFY(active);
     QCOMPARE(mdiArea.activeSubWindow(), mdiArea.subWindowList().front());
@@ -1042,11 +1027,9 @@ void tst_QMdiArea::currentSubWindow()
     QCOMPARE(mdiArea.activeSubWindow(), active);
     QCOMPARE(mdiArea.currentSubWindow(), active);
 
-    qApp->setActiveWindow(&dummyTopLevel);
     QVERIFY(mdiArea.activeSubWindow());
     QCOMPARE(mdiArea.currentSubWindow(), active);
 
-    qApp->setActiveWindow(&mdiArea);
     active->show();
     QCOMPARE(mdiArea.activeSubWindow(), active);
 
@@ -1133,7 +1116,8 @@ void tst_QMdiArea::addAndRemoveWindows()
     }
 
     // removeSubWindow
-    foreach (QWidget *window, workspace.subWindowList()) {
+    const auto subWindows = workspace.subWindowList();
+    for (QWidget *window : subWindows) {
         workspace.removeSubWindow(window);
         delete window;
     }
@@ -1251,7 +1235,6 @@ void tst_QMdiArea::closeWindows()
 {
     QMdiArea workspace;
     workspace.show();
-    qApp->setActiveWindow(&workspace);
 
     // Close widget
     QWidget *widget = new QWidget;
@@ -1303,7 +1286,6 @@ void tst_QMdiArea::activateNextAndPreviousWindow()
 {
     QMdiArea workspace;
     workspace.show();
-    qApp->setActiveWindow(&workspace);
 
     const int windowCount = 10;
     QMdiSubWindow *windows[windowCount];
@@ -1387,7 +1369,6 @@ void tst_QMdiArea::subWindowList()
 
     QMdiArea workspace;
     workspace.show();
-    qApp->setActiveWindow(&workspace);
     QVERIFY(QTest::qWaitForWindowActive(&workspace));
 
     QList<QMdiSubWindow *> activationOrder;
@@ -1683,7 +1664,8 @@ void tst_QMdiArea::tileSubWindows()
     QTRY_COMPARE(workspace.size(), QSize(350, 150));
 
     const QSize minSize(600, 130);
-    foreach (QMdiSubWindow *subWindow, workspace.subWindowList())
+    const auto subWindows = workspace.subWindowList();
+    for (QMdiSubWindow *subWindow : subWindows)
         subWindow->setMinimumSize(minSize);
 
     QCOMPARE(workspace.size(), QSize(350, 150));
@@ -1841,7 +1823,7 @@ void tst_QMdiArea::resizeMaximizedChildWindows()
 
     int newSize = startSize + increment * windowCount;
     QCOMPARE(workspaceSize, QSize(newSize, newSize));
-    foreach (QWidget *window, windows)
+    for (QWidget *window : std::as_const(windows))
         QCOMPARE(window->rect(), workspace.contentsRect());
 }
 
@@ -1866,7 +1848,7 @@ void tst_QMdiArea::focusWidgetAfterAddSubWindow()
 
     mdiArea.show();
     view->show();
-    qApp->setActiveWindow(&mdiArea);
+    QApplicationPrivate::setActiveWindow(&mdiArea);
     QCOMPARE(qApp->focusWidget(), static_cast<QWidget *>(lineEdit2));
 }
 
@@ -1875,7 +1857,6 @@ void tst_QMdiArea::dontMaximizeSubWindowOnActivation()
     QMdiArea mdiArea;
     mdiArea.show();
     QVERIFY(QTest::qWaitForWindowExposed(&mdiArea));
-    qApp->setActiveWindow(&mdiArea);
 
     // Add one maximized window.
     mdiArea.addSubWindow(new QWidget)->showMaximized();
@@ -1891,7 +1872,7 @@ void tst_QMdiArea::dontMaximizeSubWindowOnActivation()
     }
 
     // Verify that activated windows still are maximized on activation.
-    QList<QMdiSubWindow *> subWindows = mdiArea.subWindowList();
+    const QList<QMdiSubWindow *> subWindows = mdiArea.subWindowList();
     for (int i = 0; i < subWindows.size(); ++i) {
         mdiArea.activateNextSubWindow();
         QMdiSubWindow *window = subWindows.at(i);
@@ -1928,7 +1909,7 @@ void tst_QMdiArea::dontMaximizeSubWindowOnActivation()
     QVERIFY(mdiArea.activeSubWindow()->isMaximized());
 
     // Minimize all windows.
-    foreach (QMdiSubWindow *window, subWindows) {
+    for (QMdiSubWindow *window : subWindows) {
         window->showMinimized();
         QVERIFY(window->isMinimized());
         qApp->processEvents();
@@ -2260,9 +2241,8 @@ void tst_QMdiArea::tabBetweenSubWindows()
     mdiArea.show();
     QVERIFY(QTest::qWaitForWindowExposed(&mdiArea));
 
-    qApp->setActiveWindow(&mdiArea);
     QWidget *focusWidget = subWindows.back()->widget();
-    QCOMPARE(qApp->focusWidget(), focusWidget);
+    QTRY_COMPARE(qApp->focusWidget(), focusWidget);
 
     QSignalSpy spy(&mdiArea, SIGNAL(subWindowActivated(QMdiSubWindow*)));
     QCOMPARE(spy.size(), 0);
@@ -2315,7 +2295,7 @@ void tst_QMdiArea::setViewMode()
     QVERIFY(QTest::qWaitForWindowExposed(&mdiArea));
 
     QMdiSubWindow *activeSubWindow = mdiArea.activeSubWindow();
-    QList<QMdiSubWindow *> subWindows = mdiArea.subWindowList();
+    const QList<QMdiSubWindow *> subWindows = mdiArea.subWindowList();
 
     // Default.
     QVERIFY(!activeSubWindow->isMaximized());
@@ -2384,13 +2364,12 @@ void tst_QMdiArea::setViewMode()
     QVERIFY(tabBar->isTabEnabled(tabIndex));
 
     // Remove sub-windows and make sure the tab is removed.
-    foreach (QMdiSubWindow *subWindow, subWindows) {
+    for (QMdiSubWindow *subWindow : subWindows) {
         if (subWindow != activeSubWindow) {
             mdiArea.removeSubWindow(subWindow);
             delete subWindow;
         }
     }
-    subWindows.clear();
     QCOMPARE(tabBar->count(), 1);
 
     // Go back to default (QMdiArea::SubWindowView).
@@ -2591,8 +2570,11 @@ void tst_QMdiArea::nativeSubWindows()
 
     // No native widgets.
     QVERIFY(!mdiArea.viewport()->internalWinId());
-    foreach (QMdiSubWindow *subWindow, mdiArea.subWindowList())
-        QVERIFY(!subWindow->internalWinId());
+    {
+        const auto subWindows = mdiArea.subWindowList();
+        for (QMdiSubWindow *subWindow : subWindows)
+            QVERIFY(!subWindow->internalWinId());
+    }
 
     QWidget *nativeWidget = new QWidget;
     QVERIFY(nativeWidget->winId()); // enforce native window.
@@ -2601,8 +2583,11 @@ void tst_QMdiArea::nativeSubWindows()
 
     // The viewport and all the sub-windows must be native.
     QVERIFY(mdiArea.viewport()->internalWinId());
-    foreach (QMdiSubWindow *subWindow, mdiArea.subWindowList())
-        QVERIFY(subWindow->internalWinId());
+    {
+        const auto subWindows = mdiArea.subWindowList();
+        for (QMdiSubWindow *subWindow : subWindows)
+            QVERIFY(subWindow->internalWinId());
+    }
 
     // Add a non-native widget. This should become native.
     QMdiSubWindow *subWindow = new QMdiSubWindow;
@@ -2623,8 +2608,11 @@ void tst_QMdiArea::nativeSubWindows()
 
     // The viewport and all the sub-windows must be native.
     QVERIFY(mdiArea.viewport()->internalWinId());
-    foreach (QMdiSubWindow *subWindow, mdiArea.subWindowList())
-        QVERIFY(subWindow->internalWinId());
+    {
+        const auto subWindows = mdiArea.subWindowList();
+        for (QMdiSubWindow *subWindow : subWindows)
+            QVERIFY(subWindow->internalWinId());
+    }
     }
 
     { // Make a sub-window native *after* it's added to the area.
@@ -2640,9 +2628,12 @@ void tst_QMdiArea::nativeSubWindows()
 
     // All the sub-windows should be native at this point
     QVERIFY(mdiArea.viewport()->internalWinId());
-    foreach (QMdiSubWindow *subWindow, mdiArea.subWindowList())
+    {
+        const auto subWindows = mdiArea.subWindowList();
+        for (QMdiSubWindow *subWindow : subWindows)
             QVERIFY(subWindow->internalWinId());
     }
+    }
 }
 
 void tst_QMdiArea::task_209615()
@@ -2713,6 +2704,21 @@ void tst_QMdiArea::qtbug92240_title()
     QTRY_COMPARE(w.windowTitle(), QLatin1String("QTBUG-92240 - [2]"));
 }
 
+void tst_QMdiArea::tabbedview_singleSubWindow()
+{
+    // With only one sub-window, setViewMode() before addSubWindow(); and addSubWindow()
+    // before show(), ensure the sub-window is properly activated.
+    QMdiArea mdiArea;
+    mdiArea.setViewMode(QMdiArea::TabbedView);
+    auto *w = new QWidget(&mdiArea);
+    mdiArea.addSubWindow(w);
+    mdiArea.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&mdiArea));
+    auto *sub = mdiArea.subWindowList().at(0);
+    QCOMPARE(mdiArea.activeSubWindow(), sub);
+    QVERIFY(sub->isMaximized());
+}
+
 static void setupMdiAreaWithTabbedView(QMdiArea &mdiArea)
 {
     mdiArea.setViewMode(QMdiArea::TabbedView);
@@ -2766,6 +2772,46 @@ void tst_QMdiArea::tabbedview_activethird()
     QCOMPARE(mdiArea.activeSubWindow(), sub2);
 }
 
+void tst_QMdiArea::tabbedview_closeInactive()
+{
+    QMdiArea mdiArea;
+    auto createNewWindow = [&mdiArea](const QString &name){
+        QMdiSubWindow *subWindow = new QMdiSubWindow;
+        subWindow->setObjectName(name);
+        subWindow->setAttribute(Qt::WA_DeleteOnClose);
+        subWindow->setWindowTitle(name);
+        mdiArea.addSubWindow(subWindow);
+        subWindow->show();
+        return subWindow;
+    };
+
+    mdiArea.setViewMode(QMdiArea::TabbedView);
+    mdiArea.setTabsClosable(true);
+    mdiArea.setTabPosition(QTabWidget::South);
+    mdiArea.setOption(QMdiArea::DontMaximizeSubWindowOnActivation, true);
+    mdiArea.setActivationOrder(QMdiArea::ActivationHistoryOrder);
+
+    mdiArea.resize(800, 600);
+    mdiArea.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&mdiArea));
+    // This is needed for QMdiAreaPrivate::updateTabBarGeometry to update the
+    // viewport margins.
+    mdiArea.setStyleSheet(uR"qss(
+        QTabBar::tab:bottom:selected {
+            border-bottom: 1px solid;
+        }
+    )qss"_s);
+
+    QPointer<QMdiSubWindow> mdi1 = createNewWindow(u"mdi1"_s);
+    QPointer<QMdiSubWindow> mdi2 = createNewWindow(u"mdi2"_s);
+    QTRY_COMPARE(mdiArea.subWindowList().size() , 2);
+    QCOMPARE(mdiArea.activeSubWindow(), mdi2.data());
+
+    mdi1->close();
+
+    QTRY_COMPARE(mdiArea.subWindowList().size() , 1);
+    QTRY_VERIFY(!mdi1);
+}
 
 QTEST_MAIN(tst_QMdiArea)
 #include "tst_qmdiarea.moc"
